package daruma.bus.processor.csv;

/**
 * DARUMAスキーマのビルトインデータ型を定義するクラス
 * @author shumi
 *
 */
public class MispSchemaBuiltIn {
	/**
	 * デフォルトビルトイン要素のデータ型
	 */
	public final static String dftBuiltinElementType = "anySimpleType,anyURI,base64Binary,boolean,byte,date,dateTime,decimal,double,duration,ENTITIES,ENTITY,float,gDay,geometryPropertyType,gMonth,gMonthDay,gYear,gYearMonth,hexBinary,ID,IDREF,IDREFS,int,integer,language,long,Name,NCName,negativeInteger,NMTOKEN,NMTOKENS,nonNegativeInteger,nonPostiveInteger,normalizedString,NOTATION,positiveInteger,QName,short,string,timedate,TimePositionType,token,unsignedByte,unsignedInt,unsignedLong,unsignedShort";

	/**
	 * デフォルトビルトイン属性のデータ型
	 */
	public final static String dftBuiltinAttributeType = "anySimpleType,anyURI,base64Binary,boolean,byte,date,dateTime,decimal,double,duration,ENTITIES,ENTITY,float,gDay,gMonth,gMonthDay,gYear,gYearMonth,hexBinary,ID,IDREF,IDREFS,int,integer,language,long,Name,NCName,negativeInteger,NMTOKEN,NMTOKENS,nonNegativeInteger,nonPostiveInteger,normalizedString,NOTATION,positiveInteger,QName,short,string,timedate,token,unsignedByte,unsignedInt,unsignedLong,unsignedShort";

	/**
	 * ビルトイン要素のデータ型
	 */
	private static String builtinElementType = dftBuiltinElementType;

	/**
	 * ビルトイン属性のデータ型
	 */
	private static String builtinAttributeType = dftBuiltinAttributeType;

	/**
	 * ビルトイン要素のデータ型配列
	 */
	private static String[] builtinElementTypes = null;
	
	/**
	 * ビルトイン属性のデータ型配列
	 */
	private static String[] builtinAttributeTypes = null;

	/**
	 *　ビルトイン要素データ型かどうかをチェックする
	 * @param typeName データ型名
	 * @return　ビルトインデータ型の場合True, それ以外はFalseを返す
	 */
	public static boolean isBuiltinElementType(String typeName) {
		return ("," + builtinElementType + ",").indexOf("," + typeName
				+ ",") >= 0;
	}
	/**
	 *　ビルトイン属性データ型かどうかをチェックする
	 * @param typeName データ型名
	 * @return　ビルトインデータ型の場合True, それ以外はFalseを返す
	 */
	public static boolean isBuiltinAttributeType(String typeName) {
		return ("," + builtinAttributeType + ",").indexOf("," + typeName
				+ ",") >= 0;
	}
	
	/**
	 * ビルトイン要素のデータ型を取得する
	 * @return　ビルトイン要素のデータ型
	 */
	public static String getBuiltinElementType() {
		return builtinElementType;
	}

	/**
	 * ビルトイン要素のデータ型を設定する
	 * @param builtinElementDataType　設定するビルトイン要素のデータ型
	 */
	public static void setBuiltinElementType(String builtinElementDataType) {
		MispSchemaBuiltIn.builtinElementType = builtinElementDataType;
		makeBuiltinElementTypes();
	}
	
	/**
	 * ビルトイン要素のデータ型配列を取得する
	 * @return　ビルトイン要素のデータ型配列
	 */
	public static String[] getBuiltinElementTypes() {
		if(builtinElementTypes == null)
			makeBuiltinElementTypes();
		return builtinElementTypes;
	}

	/**
	 *  ビルトイン要素のデータ型配列を生成する
	 */
	private static void makeBuiltinElementTypes() {
		builtinElementTypes = MispSchemaBuiltIn.builtinElementType.split(",");
		if(builtinElementTypes == null)
			builtinElementTypes = new String[0];
	}

	/**
	 * ビルトイン属性のデータ型を取得する
	 * @return　ビルトイン属性のデータ型
	 */
	public static String getBuiltinAttributeType() {
		return builtinAttributeType;
	}

	/**
	 * ビルトイン属性のデータ型を設定する
	 * @param builtinAttributeDataType　設定するビルトイン属性のデータ型
	 */
	public static void setBuiltinAttributeType(
			String builtinAttributeDataType) {
		MispSchemaBuiltIn.builtinAttributeType = builtinAttributeDataType;
		makeBuiltinAttributeTypes();
	}

	/**
	 * ビルトイン属性のデータ型配列を取得する
	 * @return　ビルトイン属性のデータ型配列
	 */
	public static String[] getBuiltinAttributeTypes() {
		if(builtinAttributeTypes == null)
			makeBuiltinAttributeTypes();
		return builtinAttributeTypes;
	}

	/**
	 *  ビルトイン属性のデータ型配列を生成する
	 */
	private static void makeBuiltinAttributeTypes() {
		builtinAttributeTypes = MispSchemaBuiltIn.builtinAttributeType.split(",");
		if(builtinAttributeTypes == null)
			builtinAttributeTypes = new String[0];
	}

}
