package daruma.bus.processor.csv;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;


/**
 * DARUMAスキーマのCsv2Xml変換ルールを定義するクラスです
 * 
 * @author shumi
 * 
 */
public class MispSchemaCx {
	/**
	 * スキーマ
	 */
	MispSchema schema;

	/**
	 * CSV項目リスト
	 */
	List csvxColumnList = new ArrayList();

	public MispSchemaCx() {
		this.schema = new MispSchema();
	}

	public MispSchemaCx(MispSchema schema) {
		this.schema = schema;
	}

	/**
	 * ロガー
	 */
	static private Log log = LogFactory.getLog(MispSchemaCx.class);

	/**
	 * Csv2Xml変換項目
	 * 
	 * @author shumi
	 * 
	 */
	public class MispSchemaCxColumn implements Cloneable {
		/**
		 * XMLデータXPath
		 */
		private String xpath = ""; //$NON-NLS-1$

		/**
		 * CSV項目名
		 */
		private String name = ""; //$NON-NLS-1$

		/**
		 * CxColumnのインデックス(内部用)
		 */
		private int index = -1;
		
		/**
		 * POS(内部用)
		 */
		private String pos;

		/**
		 * CSV項目名を取得する
		 * 
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}

		/**
		 * CSV項目名をセットする
		 * 
		 * @param name
		 *            The name to set.
		 */
		public void setName(String name) {
			this.name = name;
		}

		/**
		 * XMLデータXPathを取得する
		 * 
		 * @return Returns the xpath.
		 */
		public String getXpath() {
			return xpath;
		}

		/**
		 * XMLデータXPathを取得する
		 * 
		 * @return Returns the xpath.
		 */
		public String getFullXpath(String prefix) {
			String fullPath = "";
			XpathDetail def = parserXpath();
			for (int cnt = 0; cnt < def.tagDefList.size(); cnt++) {
				XmlTagDef tagDef = (XmlTagDef) def.tagDefList.get(cnt);
				String name = tagDef.name;
				if (name.indexOf(':') <= 0)
					name = prefix + ":" + name;
				if (tagDef.index > 0)
					name += "[" + tagDef.index + "]";
				fullPath += name + "/";
			}
			String name = def.name;
			if (name.indexOf(':') <= 0)
				name = prefix + ":" + name;
			if (def.blAttr)
				fullPath += "@" + name;
			else
				fullPath += name;
			if (def.index > 0)
				fullPath += "[" + def.index + "]";

			return fullPath;
		}

		/**
		 * XMLデータXPathをセットする
		 * 
		 * @param xpath
		 *            The xpath to set.
		 */
		public void setXpath(String xpath) {
			this.xpath = xpath;
		}

		/**
		 * Csv2Xml変換項目をコピーする
		 * 
		 * @param column
		 *            コピー先のCsv2Xml変換項目
		 */
		public void copyTo(MispSchemaCxColumn column) {
			column.xpath = new String(this.xpath);
			column.name = new String(this.name);
		}

		/**
		 * indexを取得する
		 * 
		 * @return Returns the index.
		 */
		public int getIndex() {
			return index;
		}

		/**
		 * Csv2Xml項目の検証を行う
		 * 
		 * @param blFirst
		 *            全エラーを返すかか最初のエラーのみを返すフラグ
		 * @return エラーがある場合該当エラーを返す それ以外は空文字を返す
		 */
		public String validate(boolean blFirst) {
			String validate = ""; //$NON-NLS-1$

			validate += validateXpath(blFirst);
			if ((validate.length() > 0) && blFirst)
				return validate;

			validate += validateName(blFirst);
			if ((validate.length() > 0) && blFirst)
				return validate;

			return validate;
		}

		final public static int XPATH_INTERNAL = 0;

		final public static int XPATH_UNKNOWN = 2;

		final public static int XPATH_ERROR = 3;

		/**
		 * XMLタグを定義するクラス
		 * 
		 * @author shumi
		 * 
		 */
		public class XmlTagDef {
			/**
			 * タグ名
			 */
			private String name;

			/**
			 * インデックス
			 */
			private int index = 0;

			/**
			 * indexを取得する
			 * 
			 * @return Returns the index.
			 */
			public int getIndex() {
				return index;
			}

			/**
			 * indexをセットする
			 * 
			 * @param index
			 *            The index to set.
			 */
			public void setIndex(int index) {
				this.index = index;
			}

			/**
			 * nameを取得する
			 * 
			 * @return Returns the name.
			 */
			public String getName() {
				return name;
			}

			/**
			 * nameをセットする
			 * 
			 * @param name
			 *            The name to set.
			 */
			public void setName(String name) {
				this.name = name;
			}

		}

		/**
		 * Xpath詳細を定義するクラス
		 * 
		 * @author shumi
		 * 
		 */
		public class XpathDetail {
			/**
			 * タグ名
			 */
			private String name = ""; //$NON-NLS-1$

			/**
			 * 属性フラグ
			 */
			private boolean blAttr = false;

			/**
			 * インデックス
			 */
			private int index = 0;

			/**
			 * 親タグ定義のリスト
			 */
			private List tagDefList = new ArrayList();

			/**
			 * blAttrを取得する
			 * 
			 * @return Returns the blAttr.
			 */
			public boolean isBlAttr() {
				return blAttr;
			}

			/**
			 * indexを取得する
			 * 
			 * @return Returns the index.
			 */
			public int getIndex() {
				return index;
			}

			/**
			 * nameを取得する
			 * 
			 * @return Returns the name.
			 */
			public String getName() {
				return name;
			}

			/**
			 * tagDefListを取得する
			 * 
			 * @return Returns the tagDefList.
			 */
			public List getTagDefList() {
				return tagDefList;
			}

		}

		/**
		 * Xpath詳細をパースする
		 * 
		 * @return XPath詳細を返す
		 */
		public XpathDetail parserXpath() {
			XpathDetail def = new XpathDetail();

			String[] pathList = xpath.split("/"); //$NON-NLS-1$
			for (int cnt = 0; cnt < pathList.length; cnt++) {
				String name = pathList[cnt];
				int tagIndex = 0;
				Pattern pattern = Pattern.compile("^(.*)\\[([0-9]+)\\]$"); //$NON-NLS-1$
				Matcher matcher = pattern.matcher(name);
				if (matcher.find()) {
					name = matcher.group(1);
					tagIndex = Integer.parseInt(matcher.group(2));
				}

				if (cnt == (pathList.length - 1)) {
					def.blAttr = name.startsWith("@"); //$NON-NLS-1$
					if (def.blAttr)
						def.name = name.substring(1);
					else
						def.name = name;
					def.index = tagIndex;
				} else {
					XmlTagDef tagDef = new XmlTagDef();
					def.tagDefList.add(tagDef);
					tagDef.name = name;
					tagDef.index = tagIndex;
				}
			}
			return def;

		}

		String lastXpathError = ""; //$NON-NLS-1$

		private String getIndexStr(int pos) {
			String temp = "00000" + pos;
			return temp.substring(temp.length() - 3);
		}

		private int getXpathType() {
			int xpathType = XPATH_UNKNOWN;
			String xpathPos = "";
			lastXpathError = ""; //$NON-NLS-1$

			XpathDetail def = parserXpath();
			MispSchemaComplexType ft = schema.findComplexTypeByName(schema
					.getFeatureElement().getType());
			MispSchemaComplexType ct = ft;
			for (int cnt = 0; cnt < def.tagDefList.size(); cnt++) {
				XmlTagDef tagDef = (XmlTagDef) def.tagDefList.get(cnt);
				FindElementResult result = findElement(ct, tagDef.name);
				if (result != null) {
					MispSchemaElement element = result.em;
					if (element.getMaxOccursVal() != 1) {
						if ((tagDef.index < 1)
								|| ((element.getMaxOccursVal() > 0) && (tagDef.index > element
										.getMaxOccursVal()))) {
							xpathType = XPATH_ERROR;
							lastXpathError = MispSchemaMessages
									.getString("MispSchemaCx.13"); //$NON-NLS-1$
							break;
						}
						xpathPos += getIndexStr(result.index) + "["
								+ getIndexStr(tagDef.index) + "]/";

					} else {
						if (tagDef.index != 0) {
							xpathType = XPATH_ERROR;
							lastXpathError = MispSchemaMessages
									.getString("MispSchemaCx.14"); //$NON-NLS-1$
							break;
						}
						xpathPos += getIndexStr(result.index) + "["
								+ getIndexStr(0) + "]/";
					}
					int typeGrp = schema.getElementTypeGrp(element.getType());
					if ((typeGrp == MispSchema.GRP_COMPLEX_INTERNAL)
							|| (typeGrp == MispSchema.GRP_COMPLEX_EXTERNAL)) {
						ct = schema.findComplexTypeByName(element.getType());
						if (ct == null) {
							xpathType = XPATH_ERROR;
							lastXpathError = MispSchemaMessages
									.getString("MispSchemaCx.15"); //$NON-NLS-1$
							break;
						}
					} else {
						xpathType = XPATH_ERROR;
						lastXpathError = MispSchemaMessages
								.getString("MispSchemaCx.16"); //$NON-NLS-1$
						break;
					}
				}
			}

			if (xpathType != XPATH_ERROR) {
				if (def.blAttr) {
					// Attribute
					FindAttributeResult result = findAttribute(ct, def.name);
					if (result != null) {
						xpathType = XPATH_INTERNAL;
						xpathPos += "#" + getIndexStr(result.index) + "["
								+ getIndexStr(0) + "]";
					}
				} else {
					// Elements
					FindElementResult result = findElement(ct, def.name);
					if (result != null) {
						MispSchemaElement element = result.em;
						if (element.getMaxOccursVal() != 1) {
							if ((def.index < 1)
									|| ((element.getMaxOccursVal() > 0) && (def.index > element
											.getMaxOccursVal()))) {
								xpathType = XPATH_ERROR;
								lastXpathError = MispSchemaMessages
										.getString("MispSchemaCx.13"); //$NON-NLS-1$
							}
							xpathPos += getIndexStr(result.index) + "["
									+ getIndexStr(def.index) + "]";
						} else {
							if (def.index != 0) {
								xpathType = XPATH_ERROR;
								lastXpathError = MispSchemaMessages
										.getString("MispSchemaCx.14"); //$NON-NLS-1$
							}
							xpathPos += getIndexStr(result.index) + "["
									+ getIndexStr(0) + "]";
						}

						if (xpathType != XPATH_ERROR) {
							int typeGrp = schema.getElementTypeGrp(element
									.getType());
							if ((typeGrp == MispSchema.GRP_COMPLEX_INTERNAL)
									|| (typeGrp == MispSchema.GRP_COMPLEX_EXTERNAL)) {
								xpathType = XPATH_ERROR;
							} else {
								xpathType = XPATH_INTERNAL;
							}
						}
					}
				}
			}

			if (xpathType == XPATH_UNKNOWN) {
				// if (ft.getExtension().length() <= 0)
				{
					xpathType = XPATH_ERROR;
					lastXpathError = MispSchemaMessages
							.getString("MispSchemaCx.15"); //$NON-NLS-1$
				}
			}

			this.pos = xpathPos;
			//System.out.println("NAME:" + this.xpath + " POS:" + xpathPos);
			return xpathType;
		}

		/**
		 * Xpathリファレンスを取得する
		 * 
		 * @return Xpathの種類を返す
		 */
		public Object getXpathRef() {
			Object ref = null;
			int xpathType = XPATH_UNKNOWN;

			XpathDetail def = parserXpath();
			MispSchemaComplexType ft = schema.findComplexTypeByName(schema
					.getFeatureElement().getType());
			MispSchemaComplexType ct = ft;
			for (int cnt = 0; cnt < def.tagDefList.size(); cnt++) {
				XmlTagDef tagDef = (XmlTagDef) def.tagDefList.get(cnt);
				Object[] elements = ct.findElements(tagDef.name);
				if ((elements != null) && (elements.length >= 1)) {
					MispSchemaElement element = (MispSchemaElement) elements[0];
					if (element.getMaxOccursVal() != 1) {
						if ((tagDef.index < 1)
								|| ((element.getMaxOccursVal() > 0) && (tagDef.index > element
										.getMaxOccursVal()))) {
							xpathType = XPATH_ERROR;
							break;
						}

					} else {
						if (tagDef.index != 0) {
							xpathType = XPATH_ERROR;
							break;
						}
					}
					int typeGrp = schema.getElementTypeGrp(element.getType());
					if ((typeGrp == MispSchema.GRP_COMPLEX_INTERNAL)
							|| (typeGrp == MispSchema.GRP_COMPLEX_EXTERNAL)) {
						ct = schema.findComplexTypeByName(element.getType());
						if (ct == null) {
							xpathType = XPATH_ERROR;
							break;
						}
					} else {
						xpathType = XPATH_ERROR;
						break;
					}
				}
			}

			if (xpathType != XPATH_ERROR) {
				if (def.blAttr) {
					FindAttributeResult result = findAttribute(ct, def.name);
					if (result != null) {
						ref = result.attr;
					}
				} else {
					// Elements
					FindElementResult result = findElement(ct, def.name);
					if (result != null) {
						MispSchemaElement element = result.em;
						if (element.getMaxOccursVal() != 1) {
							if ((def.index < 1)
									|| ((element.getMaxOccursVal() > 0) && (def.index > element
											.getMaxOccursVal()))) {
								xpathType = XPATH_ERROR;
							}

						} else {
							if (def.index != 0) {
								xpathType = XPATH_ERROR;
							}
						}

						if (xpathType != XPATH_ERROR) {
							int typeGrp = schema.getElementTypeGrp(element
									.getType());
							if ((typeGrp == MispSchema.GRP_COMPLEX_INTERNAL)
									|| (typeGrp == MispSchema.GRP_COMPLEX_EXTERNAL)) {
								xpathType = XPATH_ERROR;
							} else {
								xpathType = XPATH_INTERNAL;
								ref = element;
							}
						}
					}
				}
			}

			if (xpathType == XPATH_UNKNOWN) {
				if (ft.getExtension().length() <= 0)
					xpathType = XPATH_ERROR;
			}
			return (xpathType == XPATH_ERROR) ? null : ref;
		}

		class FindElementResult {
			int index;

			MispSchemaElement em;
		}

		FindElementResult findElement(MispSchemaComplexType ct, String name) {
			List elements = ct.getAttributeElementList(true, true, false, true);
			for (int cnt = 0; cnt < elements.size(); cnt++) {
				MispSchemaElement em = (MispSchemaElement) elements.get(cnt);
				if (em.getName().equals(name)) {
					FindElementResult result = new FindElementResult();
					result.em = em;
					result.index = cnt;
					return result;
				}
			}
			return null;
		}

		class FindAttributeResult {
			int index;

			MispSchemaAttribute attr;
		}

		FindAttributeResult findAttribute(MispSchemaComplexType ct, String name) {
			List elements = ct.getAttributeElementList(true, true, true, false);
			for (int cnt = 0; cnt < elements.size(); cnt++) {
				MispSchemaAttribute attr = (MispSchemaAttribute) elements
						.get(cnt);
				if (attr.getName().equals(name)) {
					FindAttributeResult result = new FindAttributeResult();
					result.attr = attr;
					result.index = cnt;
					return result;
				}
			}
			return null;
		}

		/**
		 * XMLデータのXpathの検証を行う
		 * 
		 * @param blFirst
		 *            全エラーを返すかか最初のエラーのみを返すフラグ
		 * @return エラーがある場合該当エラーを返す それ以外は空文字を返す
		 */
		public String validateXpath(boolean blFirst) {
			String validate = ""; //$NON-NLS-1$
			int type = getXpathType();
			if (type == XPATH_ERROR) {
				if (lastXpathError.length() < 0)
					validate = MispSchemaMessages.getString("MispSchemaCx.3"); //$NON-NLS-1$
				else
					validate = lastXpathError;
			}
			return validate;
		}

		/**
		 * CSV項目名の検証を行う
		 * 
		 * @param blFirst
		 *            全エラーを返すかか最初のエラーのみを返すフラグ
		 * @return エラーがある場合該当エラーを返す それ以外は空文字を返す
		 */
		public String validateName(boolean blFirst) {
			String validate = ""; //$NON-NLS-1$
			for (int cnt = 0; cnt < csvxColumnList.size(); cnt++) {
				MispSchemaCxColumn col = (MispSchemaCxColumn) csvxColumnList
						.get(cnt);
				if (this != col) {
					if (this.getName().equals(col.getName())) {
						validate += MispSchemaMessages
								.getString("MispSchemaCx.4"); //$NON-NLS-1$
					}
				}
			}
			return validate;
		}
	}

	/**
	 * schemaを取得する
	 * 
	 * @return Returns the schema.
	 */
	public MispSchema getSchema() {
		return schema;
	}

	/**
	 * CSV項目リストサイズを取得する
	 * 
	 * @return CSV項目リストサイズ
	 */
	public int getCxColumnSize() {
		return csvxColumnList.size();
	}

	/**
	 * CSV項目配列を取得する
	 * 
	 * @return CSV項目配列
	 */
	public Object[] getCxColumnArray() {
		return csvxColumnList.toArray();
	}

	/**
	 * 指定されたインデックスのCSV項目を取得する
	 * 
	 * @return CSV項目
	 */
	public MispSchemaCxColumn getCxColumn(int index) {
		return (MispSchemaCxColumn) csvxColumnList.get(index);
	}

	/**
	 * 指定された名前のCSV項目を取得する
	 * 
	 * @return CSV項目の配列
	 */
	public Object[] findCxColumns(String name) {
		List rc = new ArrayList();

		for (int cnt = 0; cnt < csvxColumnList.size(); cnt++) {
			MispSchemaCxColumn attr = (MispSchemaCxColumn) csvxColumnList
					.get(cnt);
			if (attr.name.equals(name)) {
				rc.add(attr);
			}
		}
		return rc.toArray();
	}

	/**
	 * CSV項目をリストに追加する
	 * 
	 * @param index
	 *            追加位置
	 * @return 追加されたCSV項目
	 */
	public MispSchemaCxColumn addCxColumn(int index) {
		MispSchemaCxColumn attr = new MispSchemaCxColumn();
		attr.index = csvxColumnList.size();

		if ((index >= 0) && (index < csvxColumnList.size())) {
			csvxColumnList.add(index, attr);
			updateCxColumnIndex(0);
		} else
			csvxColumnList.add(attr);
		return attr;
	}

	/**
	 * CSV項目をリストから削除する
	 * 
	 * @param index
	 *            削除するCSV項目のインデックス
	 */
	public void delCxColumn(int index) {
		if ((index >= 0) && (index < csvxColumnList.size())) {
			csvxColumnList.remove(index);
			updateCxColumnIndex(index);
		}
	}

	/**
	 * CSV項目をリストから削除する
	 * 
	 * @param attr
	 *            削除するCSV項目
	 */
	public void delCxColumn(MispSchemaCxColumn attr) {
		delCxColumn(csvxColumnList.indexOf(attr));
	}

	/**
	 * CSV項目を移動する
	 * 
	 * @param attr
	 *            移動するCSV項目
	 * @param delta
	 *            移動量
	 */
	public void moveCxColumn(MispSchemaCxColumn attr, int delta) {
		int oldIndex = csvxColumnList.indexOf(attr);

		if ((oldIndex >= 0) && (oldIndex < csvxColumnList.size())) {
			int newIndex = oldIndex + delta;
			if (newIndex < 0)
				newIndex = 0;
			if (newIndex > csvxColumnList.size())
				newIndex = csvxColumnList.size();
			if ((newIndex >= 0) && (newIndex < csvxColumnList.size())) {
				if (oldIndex != newIndex) {
					Object org = csvxColumnList.get(newIndex);
					csvxColumnList.set(newIndex, attr);
					csvxColumnList.set(oldIndex, org);
					updateCxColumnIndex(0);
				}

			}
		}
	}

	/**
	 * CSV項目リストのインデックスを更新
	 * 
	 * @param start
	 *            更新するCSV項目の開始インデックス
	 */
	private void updateCxColumnIndex(int start) {
		if (start < 0)
			start = 0;
		for (int cnt = start; cnt < csvxColumnList.size(); cnt++) {
			((MispSchemaCxColumn) csvxColumnList.get(cnt)).index = cnt;
		}
	}

	/**
	 * クローンをする
	 */
	public Object clone() {
		MispSchemaCx csvx = new MispSchemaCx();
		csvx.schema = (MispSchema) this.schema.clone();
		csvx.csvxColumnList.clear();
		MispSchemaUtil.copyMispSchemaCsvxList(csvx, this.csvxColumnList);
		return schema;
	}

	/**
	 * スキーマを設定する
	 * 
	 * @param currSchema
	 *            設定するスキーマ
	 */
	public void setSchemaSync(MispSchema currSchema) {
		this.schema = currSchema;
	}

	/**
	 * CSVヘッダーを取得する
	 * 
	 * @return ヘッダーを返す
	 */
	public String getCsvHeader() {
		String header = "#";
		for (int cnt = 0; cnt < csvxColumnList.size(); cnt++) {
			MispSchemaCxColumn col = (MispSchemaCxColumn) csvxColumnList
					.get(cnt);
			if (cnt > 0)
				header += ",";
			header += col.getName();
		}
		return header;
	}
	
	
	public List getSortedCxColumnList()
	{
		List sortedCsvxColumnList = new ArrayList();
		for (int cnt = 0; cnt < csvxColumnList.size(); cnt++) {
			MispSchemaCxColumn col = (MispSchemaCxColumn)csvxColumnList.get(cnt);
			if (col.getXpathType() == MispSchemaCxColumn.XPATH_INTERNAL)
			{
				/* Xpathとスキーマの整合性は取れています */
				sortedCsvxColumnList.add(col);
				
			}
			else
			{
				/* Xpathとスキーマの整合性は取れていません　警告メッセージを出力する */
				log.error("Xpath不正：" + col.getXpath());
				return null;
				
			}
		}
		Collections.sort(sortedCsvxColumnList, new MispSchemaCxColumnComparator());
		return  sortedCsvxColumnList;
	}
	
	class MispSchemaCxColumnComparator implements Comparator
	{

		public int compare(Object arg0, Object arg1) {
			int rc = 0;
			if ((arg0 instanceof MispSchemaCxColumn) && (arg0 != null)
					&& (arg1 instanceof MispSchemaCxColumn) && (arg1 != null)) {
				MispSchemaCxColumn element0 = (MispSchemaCxColumn) arg0;
				MispSchemaCxColumn element1 = (MispSchemaCxColumn) arg1;
				rc = element0.pos.compareTo(element1.pos);
			}
			return rc;
		}
		
	}
}
