package daruma.bus.processor.csv;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;
import daruma.bus.processor.csv.MispSchema.MispSchemaImport;
import daruma.bus.processor.csv.MispSchema.MispSchemaNamespace;
import daruma.bus.processor.csv.MispSchemaCx.MispSchemaCxColumn;

/**
 * DaRuMaスキーマユーティリティ
 * 
 * @author shumi
 * 
 */
public class MispSchemaUtil {
	/**
	 * スキーマにリストをコピーする
	 * 
	 * @param schema
	 *            スキーマ
	 * @param srcLst
	 *            ソースリスト
	 */
	public static void copyMispSchemaList(MispSchema schema, List srcLst) {
		for (int cnt = 0; cnt < srcLst.size(); cnt++) {
			Object src = srcLst.get(cnt);
			Object dst = null;

			if (src instanceof MispSchemaComplexType) {
				MispSchemaComplexType srcCt = (MispSchemaComplexType) src;
				MispSchemaComplexType dstCt;
				if (srcCt.getKind() == MispSchema.KIND_INTERNAL) {
					dstCt = schema.addComplexType(null);
				} else {
					dstCt = schema.addComplexType(schema.getImport(srcCt
							.getIm().getIndex()));
				}
				srcCt.copyTo(dstCt);
			} else if (src instanceof MispSchemaImport) {
				dst = schema.addImport();
				((MispSchemaImport) src).copyTo((MispSchemaImport) dst);
			} else if (src instanceof MispSchemaNamespace) {
				dst = schema.addNamespace();
				((MispSchemaNamespace) src).copyTo((MispSchemaNamespace) dst);
			} else if (src instanceof MispSchemaElement) {
				dst = schema.addRootElement();
				((MispSchemaElement) src).copyTo((MispSchemaElement) dst);
			} else if (src instanceof MispSchemaAttribute) {
				dst = schema.addRootAttribute();
				((MispSchemaAttribute) src).copyTo((MispSchemaAttribute) dst);
			}
		}
	}

	/**
	 * 複合型にリストをコピーする
	 * 
	 * @param ct
	 *            複合型
	 * @param srcLst
	 *            ソースリスト
	 */
	public static void copyMispSchemaComplexTypeList(MispSchemaComplexType ct,
			List srcLst, boolean withoutRef) {
		for (int cnt = 0; cnt < srcLst.size(); cnt++) {
			Object src = srcLst.get(cnt);
			Object dst = null;

			if (src instanceof MispSchemaAttribute) {
				dst = ct.addAttribute(-1);
				if (withoutRef)
					((MispSchemaAttribute) src)
							.copyToWithoutRef((MispSchemaAttribute) dst);
				else
					((MispSchemaAttribute) src)
							.copyTo((MispSchemaAttribute) dst);

			} else if (src instanceof MispSchemaElement) {
				dst = ct.addElement(-1);
				if (withoutRef)
					((MispSchemaElement) src)
							.copyToWithoutRef((MispSchemaElement) dst);
				else
					((MispSchemaElement) src).copyTo((MispSchemaElement) dst);
			}

		}
	}

	/**
	 * リストを比較する
	 * 
	 * @param lst1
	 *            リスト１
	 * @param lst2
	 *            リスト２
	 * @return 同じである場合True それ以外はFalseを返す
	 */
	public static boolean equalsList(List lst1, List lst2) {
		if (lst1 == null)
			return (lst2 == null);
		if (lst2 == null)
			return false;
		if (lst1.size() != lst2.size())
			return false;
		for (int cnt = 0; cnt < lst1.size(); cnt++) {
			Object obj1 = lst1.get(cnt);
			Object obj2 = lst2.get(cnt);
			if (obj1 == null)
				return (obj2 == null);
			if (obj2 == null)
				return false;
			if (!obj1.equals(obj2))
				return false;
		}
		return true;
	}

	/**
	 * リストを比較する
	 * 
	 * @param lst1
	 *            リスト１
	 * @param lst2
	 *            リスト２
	 * @return 同じである場合True それ以外はFalseを返す
	 */
	public static boolean equalsList2(List lst1, List lst2) {
		if (lst1 == null)
			return (lst2 == null);
		if (lst2 == null)
			return false;
		if (lst1.size() != lst2.size())
			return false;
		for (int cnti = 0; cnti < lst1.size(); cnti++) {
			Object obj1 = lst1.get(cnti);
			boolean matched = false;
			for (int cntj = 0; cntj < lst2.size(); cntj++) {
				Object obj2 = lst2.get(cntj);
				if (((obj1 == null) && ((obj2 == null))) || obj1.equals(obj2)) {
					matched = true;
					break;
				}
			}
			if (!matched)
				return false;
		}
		for (int cnti = 0; cnti < lst1.size(); cnti++) {
			Object obj1 = lst2.get(cnti);
			boolean matched = false;
			for (int cntj = 0; cntj < lst2.size(); cntj++) {
				Object obj2 = lst1.get(cntj);
				if (((obj1 == null) && ((obj2 == null))) || obj1.equals(obj2)) {
					matched = true;
					break;
				}
			}
			if (!matched)
				return false;
		}
		return true;
	}

	/**
	 * リストを検証する
	 * 
	 * @param srcList
	 *            検証するリスト
	 * @param blFirst
	 *            全エラーを返すかか最初のエラーのみを返すフラグ
	 * @return エラーがある場合該当エラーを返す それ以外は空文字を返す
	 */
	public static String validateList(List srcList, boolean blFirst) {
		String validate = ""; //$NON-NLS-1$
		if (srcList != null) {
			for (int cnt = 0; cnt < srcList.size(); cnt++) {
				Object item = srcList.get(cnt);
				if (item instanceof MispSchemaComplexType) {
					validate += ((MispSchemaComplexType) item)
							.validate(blFirst);
				} else if (item instanceof MispSchemaAttribute) {
					validate += ((MispSchemaAttribute) item).validate(blFirst);
				} else if (item instanceof MispSchemaElement) {
					validate += ((MispSchemaElement) item).validate(blFirst);
				} else if (item instanceof MispSchemaImport) {
					validate += ((MispSchemaImport) item).validate(blFirst);
				} else if (item instanceof MispSchemaNamespace) {
					validate += ((MispSchemaNamespace) item).validate(blFirst);
				}
				if ((validate.length() > 0) && blFirst)
					break;
			}
		}
		return validate;
	}

	/**
	 * スキーマをダンプする
	 * 
	 * @param schema
	 *            ダンプするスキーマ
	 * @param out
	 *            出力ストリームライター
	 * @param separator
	 *            セパレータキャラクタ
	 */
	public static void dumpFeatureType(MispSchema schema,
			OutputStreamWriter out, String separator) {
		try {
			MispSchemaElement ft = schema.getFeatureElement();
			MispSchemaComplexType ct = schema.findComplexTypeByName(ft
					.getType());
			if (ct == null) {
				out.write("FeatureType not found"); //$NON-NLS-1$
			} else {
				out.write("Name" + separator + ft.name + "\r\n"); //$NON-NLS-1$ //$NON-NLS-2$
				out.write("Type" + separator + ft.type + "\r\n"); //$NON-NLS-1$ //$NON-NLS-2$
				if (ct.getExtension().length() > 0) {
					out.write("ExtensionBase" + separator + ct.getExtension() //$NON-NLS-1$
							+ "\r\n\r\n"); //$NON-NLS-1$
				}
				dumpComplexType(schema, ct, out, true, separator, ft.name);
			}
		} catch (IOException e) {

		}
	}

	/**
	 * 複合型をダンプする
	 * 
	 * @param schema
	 *            ダンプするスキーマ
	 * @param ct
	 *            ダンプする複合型
	 * @param out
	 *            出力ストリームライター
	 * @param flgExt
	 *            回帰的ダンプフラグ
	 * @param separator
	 *            セパレータキャラクタ
	 * @param prefix
	 *            出力接頭辞
	 * @throws IOException
	 */
	private static void dumpComplexType(MispSchema schema,
			MispSchemaComplexType ct, OutputStreamWriter out, boolean flgExt,
			String separator, String prefix) throws IOException {
		for (int cnt = 0; cnt < ct.getAttributeSize(); cnt++) {
			MispSchemaAttribute attr = (MispSchemaAttribute) ct
					.getAttribute(cnt);
			out.write(prefix + "/@" + attr.getName() + separator //$NON-NLS-1$
					+ attr.getType() + "\r\n"); //$NON-NLS-1$
		}
		for (int cnt = 0; cnt < ct.getElementSize(); cnt++) {
			MispSchemaElement em = (MispSchemaElement) ct.getElement(cnt);
			switch (schema.getElementTypeGrp(em.getType())) {
			case MispSchema.GRP_COMPLEX_INTERNAL:
			case MispSchema.GRP_COMPLEX_EXTERNAL:
				if (flgExt) {
					MispSchemaComplexType childEt = schema
							.findComplexTypeByName(em.getType());
					if (childEt != null) {
						dumpComplexType(schema, childEt, out, flgExt,
								separator, prefix + "/" + em.getName()); //$NON-NLS-1$
						break;
					}
				}
			default:
				out.write(prefix + "/" + em.getName() + separator //$NON-NLS-1$
						+ em.getType() + "\r\n"); //$NON-NLS-1$
				break;
			}

		}
	}

	/**
	 * DaRuMaスキーマのXPath項目
	 * 
	 * @author shumi
	 * 
	 */
	public static class MispSchemaXpath {
		/**
		 * Xpath
		 */
		private String xpath;
		
		/**
		 * 属性フラグ
		 */
		private boolean blAttr;

		/**
		 * レベル
		 */
		private int level;

		/**
		 * 参照オブジェクト（属性或いは要素）
		 */
		private Object ref;

		/**
		 * blAttrを取得する
		 * 
		 * @return Returns the blAttr.
		 */
		public boolean isBlAttr() {
			return blAttr;
		}

		/**
		 * blAttrをセットする
		 * 
		 * @param blAttr
		 *            The blAttr to set.
		 */
		public void setBlAttr(boolean blAttr) {
			this.blAttr = blAttr;
		}

		/**
		 * levelを取得する
		 * 
		 * @return Returns the level.
		 */
		public int getLevel() {
			return level;
		}

		/**
		 * levelをセットする
		 * 
		 * @param level
		 *            The level to set.
		 */
		public void setLevel(int level) {
			this.level = level;
		}

		/**
		 * refを取得する
		 * 
		 * @return Returns the ref.
		 */
		public Object getRef() {
			return ref;
		}

		/**
		 * refをセットする
		 * 
		 * @param ref
		 *            The ref to set.
		 */
		public void setRef(Object ref) {
			this.ref = ref;
		}

		/**
		 * xpathを取得する
		 * 
		 * @return Returns the xpath.
		 */
		public String getXpath() {
			return xpath;
		}

		/**
		 * xpathをセットする
		 * 
		 * @param xpath
		 *            The xpath to set.
		 */
		public void setXpath(String xpath) {
			this.xpath = xpath;
		}

	}

	/**
	 * XPathリストを取得する
	 * 
	 * @param schema
	 *            スキーマ
	 */
	public static List getXPathList(MispSchema schema, String ns) {
		List xpathList = new ArrayList();
		MispSchemaElement ft = schema.getFeatureElement();
		MispSchemaComplexType ct = schema.findComplexTypeByName(ft.getType());
		if (ct != null) {
			getXPathList(schema, ct, xpathList, true, "", ns, 0, TYPE_ALL); //$NON-NLS-1$
		}
		return xpathList;
	}

	/**
	 * 複合型の属性と要素のリストをXPath形式で取得する
	 * 
	 * @param ct
	 *            スキーマ
	 * @param prefix
	 *            接頭辞
	 */
	public static List getXPathList(MispSchemaComplexType ct, String prefix) {
		MispSchema schema = ct.getSchema();
		List xpathList = new ArrayList();
		getXPathList(schema, ct, xpathList, true, "", prefix, 0, TYPE_ALL); //$NON-NLS-1$
		return xpathList;
	}

	/**
	 * XPathリストを取得する
	 * 
	 * @param schema
	 *            スキーマ
	 * @param prefix
	 *            接頭辞
	 */
	public static List getDeleteNullXpathList(MispSchema schema, String prefix) {
		List xpathList = new ArrayList();
		MispSchemaElement ft = schema.getFeatureElement();
		MispSchemaComplexType ct = schema.findComplexTypeByName(ft.getType());
		if (ct != null) {
			getXPathList(schema, ct, xpathList, true, "", prefix, 0, TYPE_DELNULL); //$NON-NLS-1$
		}
		return xpathList;
	}
	

	/**
	 * XPathリストを取得する
	 * 
	 * @param schema
	 *            スキーマ
	 * @param prefix
	 *            接頭辞
	 */
	public static List getAttatchFileXpathList(MispSchema schema, String prefix) {
		List xpathList = new ArrayList();
		MispSchemaElement ft = schema.getFeatureElement();
		MispSchemaComplexType ct = schema.findComplexTypeByName(ft.getType());
		if (ct != null) {
			getXPathList(schema, ct, xpathList, true, "", prefix, 0, TYPE_ATTATCHFILE); //$NON-NLS-1$
		}
		return xpathList;
	}


	private static int TYPE_ALL = 0;
	private static int TYPE_DELNULL = 1;
	private static int TYPE_ATTATCHFILE = 2;
	
	/**
	 * 複合型のXPathリストを取得する
	 * 
	 * @param schema
	 *            ダンプするスキーマ
	 * @param ct
	 *            ダンプする複合型
	 * @param out
	 *            出力ストリームライター
	 * @param flgExt
	 *            回帰的ダンプフラグ
	 * @param separator
	 *            セパレータキャラクタ
	 * @param prefix
	 *            出力接頭辞
	 * @throws IOException
	 */
	private static void getXPathList(MispSchema schema,
			MispSchemaComplexType ct, List xpathList, boolean flgExt,
			String prefix, String ns, int level, int type) {

		if (type == TYPE_ALL) {
			for (int cnt = 0; cnt < ct.getAttributeSize(); cnt++) {
				MispSchemaAttribute attr = (MispSchemaAttribute) ct
						.getAttribute(cnt);
				MispSchemaXpath xpath = new MispSchemaXpath();
				if ((ns != null) && (ns.length() > 0) && (attr.getNamePrefix().length() == 0))
					xpath.xpath = prefix + "@" + ns + ":" + attr.getName(); //$NON-NLS-1$ //$NON-NLS-2$
				else
					xpath.xpath = prefix + "@" + attr.getName(); //$NON-NLS-1$
				xpath.blAttr = true;
				xpath.level = level;
				xpath.ref = attr;
				xpathList.add(xpath);
			}
		}
		for (int cnt = 0; cnt < ct.getElementSize(); cnt++) {
			MispSchemaElement em = (MispSchemaElement) ct.getElement(cnt);
			String emName ;
			if ((ns != null) && (ns.length() > 0) && (em.getNamePrefix().length() == 0))
				emName = prefix + ns + ":" + em.getName(); //$NON-NLS-1$
			else
				emName = prefix + em.getName();

			switch (schema.getElementTypeGrp(em.getType())) {
			case MispSchema.GRP_COMPLEX_INTERNAL:
			case MispSchema.GRP_COMPLEX_EXTERNAL:
				boolean delNullAdd = false;
				if (type == TYPE_DELNULL) {
					if ((em.getMaxOccursVal() != 1)
							|| (em.getMinOccursVal() == 0)) {
						MispSchemaXpath xpath = new MispSchemaXpath();
						xpath.xpath = emName;
						xpath.blAttr = false;
						xpath.level = level;
						xpath.ref = em;
						xpathList.add(xpath);
						delNullAdd = true;
					}
				} 
				if((type == TYPE_ALL) || (type == TYPE_ATTATCHFILE) || ((type == TYPE_DELNULL) && !delNullAdd)){
					if (flgExt) {
						MispSchemaComplexType childEt = schema
								.findComplexTypeByName(em.getType());
						if (childEt != null) {
							getXPathList(schema, childEt, xpathList, flgExt,
									emName + "/", ns, level + 1, //$NON-NLS-1$
									type);
						}
					}
				}
				break;
			default:
				boolean add = false;
				if(type == TYPE_DELNULL)
				{
					add = (em.getMaxOccursVal() != 1)
							|| (em.getMinOccursVal() == 0);
				}
				else if (type == TYPE_ALL)
				{
					add =  true;
				}
				else if(type == TYPE_ATTATCHFILE)
				{
					add = em.getType().equals(Const.DFS_TYPE_ATTATCHEDFILE);
				}
				
				if (add) {
					MispSchemaXpath xpath = new MispSchemaXpath();
					xpath.xpath = emName;
					xpath.blAttr = false;
					xpath.level = level;
					xpath.ref = em;
					xpathList.add(xpath);
				}
				break;
			}

		}
	}

	/**
	 * MispSchemaCxにリストをコピーする
	 * 
	 * @param csvx
	 *            MispSchemaCx
	 * @param srcLst
	 *            ソースリスト
	 */
	public static void copyMispSchemaCsvxList(MispSchemaCx csvx, List srcLst) {
		for (int cnt = 0; cnt < srcLst.size(); cnt++) {
			Object src = srcLst.get(cnt);
			Object dst = null;

			if (src instanceof MispSchemaCxColumn) {
				dst = csvx.addCxColumn(-1);
				((MispSchemaCxColumn) src).copyTo((MispSchemaCxColumn) dst);

			}

		}
	}
	
	
	/**
	 * ＣＳＶファイルの行をカラム単位で分割して、文字列の配列を返す。
	 * 
	 * @param line
	 *            行データ
	 * @param separator
	 *            セパレータキャラクタ
	 * @param parenthesis
	 *            括弧キャラクタ
	 * @return 文字列の配列
	 * @throws CsvMessageException
	 */
	static public String[] getColumns(String line, char separator,
			char parenthesis) {

		try {
			StringBuffer buffer = new StringBuffer(line);
			List columnList = new ArrayList();
			splitLine(buffer, separator, parenthesis, 0, columnList);
			return (String[]) columnList.toArray(new String[0]);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * バッファ内の文字列で、指定したセパレータと括弧で分割される最初のポジションを返す。
	 * 
	 * @param buffer
	 *            文字列バッファ
	 * @param separator
	 *            セパレータキャラクタ
	 * @param parenthesis
	 *            括弧キャラクタ
	 * @param startOffset
	 *            開始オフセット
	 * @param curOffset
	 *            カレントオフセット
	 * @return CSVアイテムの位置
	 */
	static private int csvLineIndexOf(StringBuffer buffer, char separator,
			char parenthesis, int startOffset, int curOffset) {
		String str = String.valueOf(separator);
		int curPos = buffer.indexOf(str, curOffset);
		if (curPos == -1)
			return -1;
		else if (buffer.charAt(startOffset) != parenthesis)
			return curPos;
		else if ((buffer.charAt(startOffset) == parenthesis && curPos > 0 && buffer
				.charAt(curPos - 1) == parenthesis))
			return curPos;
		else
			return csvLineIndexOf(buffer, separator, parenthesis, startOffset,
					curPos + 1);
	}

	/**
	 * バッファ内の文字列を、指定したセパレータと括弧で分割し、リストに格納する。
	 * 
	 * @param buffer
	 *            文字列バッファ
	 * @param separator
	 *            セパレータキャラクタ
	 * @param parenthesis
	 *            括弧キャラクタ
	 * @param offset
	 *            オフセット
	 * @param columnList
	 *            カラムリスト
	 */
	static private void splitLine(StringBuffer buffer, char separator,
			char parenthesis, int offset, List columnList) {
		int curPos = csvLineIndexOf(buffer, separator, parenthesis, offset,
				offset);
		if (curPos == -1) {
			String data = buffer.substring(offset, buffer.length());
			if (data.length() > 0 && data.charAt(0) == parenthesis
					&& data.charAt(data.length() - 1) == parenthesis)
				columnList.add(data.substring(1, data.length() - 1));
			else
				columnList.add(data);
			return;
		} else {
			if (offset < curPos) {
				String data = buffer.substring(offset, curPos);
				if (data.length() > 0 && data.charAt(0) == parenthesis
						&& data.charAt(data.length() - 1) == parenthesis) {
					if (data.length() > 1)
						columnList.add(data.substring(1, data.length() - 1));
					else
						columnList.add(""); //$NON-NLS-1$
				} else
					columnList.add(data);
			} else
				columnList.add(""); //$NON-NLS-1$
			splitLine(buffer, separator, parenthesis, curPos + 1, columnList);
		}
	}

	public static String convertXpathToTagTreeText(MispSchema schema, String xpath)
	{
		String val = "";
		String[] temp= xpath.split("/");
		for(int cnt = 0; cnt < temp.length; cnt++)
		{
			int index = temp[cnt].indexOf(":");
			MispSchemaNamespace ns = null;
			String localName = temp[cnt];
			if(index > 0)
			{
				ns = schema.findNamespaceByPrefix(temp[cnt].substring(0, index));
				localName = temp[cnt].substring(index + 1);
			}
			String uri = (ns == null) ? schema.getTargetNamespace(): ns.getUri();
			if(val.length() > 0 ) val += Const.INTERNAL_TAG_SEP;
			val += uri + Const.INTERNAL_NAMESPACE_SEP + localName;
		}
		
		return val;
	}
	
	public static String makeTagTreeText(String uri, String localName)
	{
		return uri + Const.INTERNAL_NAMESPACE_SEP + localName;
	}
	
	public static String makeTagTreeText(List lstName) {
		String fullName = "";
		for (int cnt = 0; cnt < lstName.size(); cnt++) {
			if (cnt > 0)
				fullName += Const.INTERNAL_TAG_SEP;
			fullName += (String) lstName.get(cnt);
		}
		return fullName;
	}

	public static String makeTagTreeText(String[] lstName, int maxIndex) {
		String fullName = "";
		for (int cnt = 0; (cnt < lstName.length) && (cnt <= maxIndex); cnt++) {
			if (cnt > 0)
				fullName += Const.INTERNAL_TAG_SEP;
			fullName += lstName[cnt];
		}
		return fullName;
	}

	public static String[] makeTagTreeList(String treeText) {
		String[] tagList;
		if (treeText.indexOf(Const.INTERNAL_TAG_SEP) > 0)
			tagList = treeText.split(Const.INTERNAL_TAG_SEP);
		else {
			tagList = new String[1];
			tagList[0] = treeText;
		}
		return tagList;

	}
}
