package daruma.client.schemaeditor;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

import daruma.client.schemaeditor.ui.actions.CloseAction;
import daruma.client.schemaeditor.ui.actions.CsvxNewAction;
import daruma.client.schemaeditor.ui.actions.CsvxOpenAction;
import daruma.client.schemaeditor.ui.actions.ExitAction;
import daruma.client.schemaeditor.ui.actions.ExportXsdAction;
import daruma.client.schemaeditor.ui.actions.ImportXsdAction;
import daruma.client.schemaeditor.ui.actions.NewAction;
import daruma.client.schemaeditor.ui.actions.OpenAction;
import daruma.client.schemaeditor.ui.actions.RedoAction;
import daruma.client.schemaeditor.ui.actions.SaveAction;
import daruma.client.schemaeditor.ui.actions.SaveAsAction;
import daruma.client.schemaeditor.ui.actions.UndoAction;
import daruma.client.schemaeditor.ui.views.SchemaEditorView;
import daruma.client.schemaeditor.ui.views.SwtUtil;

/**
 * An action bar advisor is responsible for creating, adding, and disposing of
 * the actions added to a workbench window. Each window will be populated with
 * new actions.
 */
public class ApplicationActionBarAdvisor extends ActionBarAdvisor implements
		IMenuListener {

	// Actions - important to allocate these only in makeActions, and then use
	// them
	// in the fill methods. This ensures that the actions aren't recreated
	// when fillActionBars is called with FILL_PROXY.

	private Action exitAction;

	private IWorkbenchAction aboutAction;

	private Action openAction;

	private Action closeAction;

	private Action saveAction;

	private Action saveAsAction;

	private Action exportXsdAction;
	
	private Action importXsdAction;

	private Action newAction;

	private Action undoAction;

	private Action redoAction;

	private Action csvxNewAction;

	private Action csvxOpenAction;

	/**
	 * アプリケーションアクションバーアドバイザーを生成する
	 * 
	 * @param configurer
	 *            the action bar configurer
	 */
	public ApplicationActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.ActionBarAdvisor#makeActions(org.eclipse.ui.IWorkbenchWindow)
	 */
	protected void makeActions(final IWorkbenchWindow window) {
		aboutAction = ActionFactory.ABOUT.create(window);
		register(aboutAction);

		/**
		 * 終了Action
		 */
		exitAction = ExitAction.makeAction();
		exitAction.setText(Messages.getString("ApplicationActionBarAdvisor.0")); //$NON-NLS-1$
		exitAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.8")); //$NON-NLS-1$

		/**
		 * 新規Action
		 */
		newAction = NewAction.makeAction();
		newAction.setText(Messages.getString("ApplicationActionBarAdvisor.1")); //$NON-NLS-1$
		newAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.9")); //$NON-NLS-1$

		/**
		 * 開くAction
		 */
		openAction = OpenAction.makeAction();
		openAction.setText(Messages.getString("ApplicationActionBarAdvisor.2")); //$NON-NLS-1$
		openAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.10")); //$NON-NLS-1$

		/**
		 * 閉じるAction
		 */
		closeAction = CloseAction.makeAction();
		closeAction
				.setText(Messages.getString("ApplicationActionBarAdvisor.3")); //$NON-NLS-1$
		closeAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.11")); //$NON-NLS-1$

		/**
		 * 保存Action
		 */
		saveAction = SaveAction.makeAction();
		saveAction.setText(Messages.getString("ApplicationActionBarAdvisor.4")); //$NON-NLS-1$
		saveAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.12")); //$NON-NLS-1$

		/**
		 * 名前付けて保存Action
		 */
		saveAsAction = SaveAsAction.makeAction();
		saveAsAction.setText(Messages
				.getString("ApplicationActionBarAdvisor.5")); //$NON-NLS-1$
		saveAsAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.13")); //$NON-NLS-1$

		/**
		 * XSDエクスポートAction
		 */
		exportXsdAction = ExportXsdAction.makeAction();
		exportXsdAction.setText(Messages
				.getString("ApplicationActionBarAdvisor.6")); //$NON-NLS-1$
		exportXsdAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.14")); //$NON-NLS-1$

		/**
		 * XSDインポートAction
		 */
		importXsdAction = ImportXsdAction.makeAction();
		importXsdAction.setText(Messages.getString("ApplicationActionBarAdvisor.31")); //$NON-NLS-1$
		importXsdAction.setToolTipText(Messages.getString("ApplicationActionBarAdvisor.32")); //$NON-NLS-1$
		
		/**
		 * undoAction
		 */
		undoAction = UndoAction.makeAction();
		undoAction
				.setText(Messages.getString("ApplicationActionBarAdvisor.22")); //$NON-NLS-1$
		undoAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.23")); //$NON-NLS-1$

		/**
		 * redoAction
		 */
		redoAction = RedoAction.makeAction();
		redoAction
				.setText(Messages.getString("ApplicationActionBarAdvisor.24")); //$NON-NLS-1$
		redoAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.25")); //$NON-NLS-1$

		/**
		 * CsvxOpenAction
		 */
		csvxOpenAction = CsvxOpenAction.makeAction();
		csvxOpenAction.setText(Messages
				.getString("ApplicationActionBarAdvisor.17")); //$NON-NLS-1$
		csvxOpenAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.18")); //$NON-NLS-1$

		/**
		 * CsvxNewAction
		 */
		csvxNewAction = CsvxNewAction.makeAction();
		csvxNewAction.setText(Messages
				.getString("ApplicationActionBarAdvisor.15")); //$NON-NLS-1$
		csvxNewAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.16")); //$NON-NLS-1$

		/**
		 * CsvxOpenAction
		 */
		csvxOpenAction = CsvxOpenAction.makeAction();
		csvxOpenAction.setText(Messages
				.getString("ApplicationActionBarAdvisor.17")); //$NON-NLS-1$
		csvxOpenAction.setToolTipText(Messages
				.getString("ApplicationActionBarAdvisor.18")); //$NON-NLS-1$
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.ActionBarAdvisor#fillMenuBar(org.eclipse.jface.action.IMenuManager)
	 */
	protected void fillMenuBar(IMenuManager menuBar) {
		MenuManager fileMenu = new MenuManager(Messages
				.getString("ApplicationActionBarAdvisor.7"), //$NON-NLS-1$
				IWorkbenchActionConstants.M_FILE);
		menuBar.add(fileMenu);
		fileMenu.addMenuListener(this);
		fileMenu.add(newAction);
		fileMenu.add(openAction);
		fileMenu.add(closeAction);
		fileMenu.add(new Separator());
		fileMenu.add(saveAction);
		fileMenu.add(saveAsAction);
		fileMenu.add(new Separator());
		fileMenu.add(importXsdAction);
		fileMenu.add(exportXsdAction);
		fileMenu.add(new Separator());
		fileMenu.add(exitAction);

		MenuManager editMenu = new MenuManager(Messages
				.getString("ApplicationActionBarAdvisor.19"), //$NON-NLS-1$
				IWorkbenchActionConstants.M_EDIT);
		menuBar.add(editMenu);
		editMenu.addMenuListener(this);
		editMenu.add(undoAction);
		editMenu.add(redoAction);

		MenuManager toolMenu = new MenuManager(Messages
				.getString("ApplicationActionBarAdvisor.20"), //$NON-NLS-1$
				"tool"); //$NON-NLS-1$
		menuBar.add(toolMenu);
		toolMenu.addMenuListener(this);
		toolMenu.add(csvxNewAction);
		toolMenu.add(csvxOpenAction);

		MenuManager helpMenu = new MenuManager(Messages
				.getString("ApplicationActionBarAdvisor.21"), //$NON-NLS-1$
				IWorkbenchActionConstants.M_HELP);
		menuBar.add(helpMenu);
		helpMenu.add(aboutAction);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void menuAboutToShow(IMenuManager manager) {
		SchemaEditorView view = (SchemaEditorView) SwtUtil
				.findView(SchemaEditorView.ID);
		boolean hasView = (view != null);
		boolean enabled;

		view.setFocus();
		newAction.setEnabled(hasView);
		openAction.setEnabled(hasView);
		closeAction.setEnabled(hasView && view.isOpened());
		saveAction.setEnabled(hasView && view.isNeedSave());
		saveAsAction.setEnabled(hasView);
		exportXsdAction.setEnabled(hasView);
		importXsdAction.setEnabled(hasView);
		exitAction.setEnabled(true);

		if (hasView) {
			enabled = view.isCanEditUndoRedo(false);
			undoAction.setEnabled(enabled);
			undoAction
					.setText(Messages
							.getString("ApplicationActionBarAdvisor.27") //$NON-NLS-1$
							+ (enabled ? (" - " + view.getEditUndoRedoOpName(false)) : "")); //$NON-NLS-1$ //$NON-NLS-2$

			enabled = view.isCanEditUndoRedo(true);
			redoAction.setEnabled(enabled);
			redoAction
					.setText(Messages
							.getString("ApplicationActionBarAdvisor.30") //$NON-NLS-1$
							+ (enabled ? (" - " + view.getEditUndoRedoOpName(true)) : "")); //$NON-NLS-1$ //$NON-NLS-2$

		}

		manager.update();

	}

}
