package daruma.client.schemaeditor;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import daruma.client.schemaeditor.ui.views.SchemaEditorView;
import daruma.client.schemaeditor.ui.views.SwtUtil;

/**
 * The workbench window advisor object is created in response to a workbench window being created (one per window), and is used to configure the window. 
 * @author shumi
 *
 */
public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {
	/**
	 * ロガー
	 */
	static private Log log = LogFactory
			.getLog(ApplicationWorkbenchWindowAdvisor.class);

	/**
	 * カレントアドバイザ
	 */
	private static ApplicationWorkbenchWindowAdvisor currAdvisor = null;

	/**
	 * アプリケーションワークベンチウィンドアドバイザを生成する
	 * @param configurer an object for configuring the workbench window
	 */
	public ApplicationWorkbenchWindowAdvisor(
			IWorkbenchWindowConfigurer configurer) {
		super(configurer);
		currAdvisor = this;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.WorkbenchWindowAdvisor#createActionBarAdvisor(org.eclipse.ui.application.IActionBarConfigurer)
	 *      manager)
	 */
	public ActionBarAdvisor createActionBarAdvisor(
			IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.WorkbenchWindowAdvisor#preWindowOpen()
	 *      manager)
	 */
	public void preWindowOpen() {
		log.info("workbench opened"); //$NON-NLS-1$
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(800, 600));
		configurer.setShowCoolBar(false);
		configurer.setShowStatusLine(false);
		setWorkbenchWindowTitle(""); //$NON-NLS-1$
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.WorkbenchWindowAdvisor#preWindowShellClose()
	 *      manager)
	 */

	public boolean preWindowShellClose() {
		onExit("workbench is closed");//$NON-NLS-1$
		return true;
	}

	/**
	 * ワークベンチウィンドのタイトルを設定する
	 * 
	 * @param title 設定するタイトル文字列
	 */

	public static void setWorkbenchWindowTitle(String title) {
		if (currAdvisor != null) {
			if ((title != null) && (title.length() > 0)) {
				title = Messages
						.getString("ApplicationWorkbenchWindowAdvisor.3") + title; //$NON-NLS-1$
			}
			currAdvisor
					.getWindowConfigurer()
					.setTitle(
							Messages
									.getString("ApplicationWorkbenchWindowAdvisor.4") + title); //$NON-NLS-1$
		}
	}

	/**
	 * アプリケーション終了時の処理
	 */
	public static void onExit(String reason) {
		log.info("Exit because " + reason);
		SchemaEditorView view = (SchemaEditorView) SwtUtil
				.findView(SchemaEditorView.ID);
		if (view != null) {
			view.closeFile();
		}

	}

}
