package daruma.client.schemaeditor.ui.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;

import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchemaCx;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;
import daruma.bus.processor.csv.MispSchemaCx.MispSchemaCxColumn;

import daruma.client.schemaeditor.Messages;

public class CsvxEditorDialog extends GenericDialog {
	public Shell parentShell;

	protected CsvxEditorDialog(Shell parentShell) {
		super(parentShell, Messages.getString("CsvxEditorDialog.0"), 720, 440); //$NON-NLS-1$
		super.setOkText(Messages.getString("CsvxEditorDialog.1")); //$NON-NLS-1$
		boolean windows = System.getProperty("os.name").startsWith("Windows"); //$NON-NLS-1$ //$NON-NLS-2$
		toolTipNoneTblText = windows ? "" : " "; //$NON-NLS-1$ //$NON-NLS-2$
		this.parentShell = parentShell;
	}

	/**
	 * MispSchemaCx
	 */
	MispSchemaCx currCx;

	/**
	 * 親コンポジット
	 */
	private Composite parent;

	/**
	 * ツリービュー
	 */
	private TreeViewer treeComplexDetail;

	/**
	 * ツリービューの展開レベル
	 */
	private static int DFT_TREE_LEVEL = 2;

	/**
	 * XPathツリーリスト
	 */
	private Table tblCsvx;

	/**
	 * XPathツリーリスト
	 */
	private TableViewer viewCsvx;

	/**
	 * 名前空間追加アクション
	 */
	private Action actionAdd;

	/**
	 * 名前空間削除アクション
	 */
	private Action actionDel;

	/**
	 * 上へ移動
	 */
	private Action actionMoveUp;

	/**
	 * 下へ移動
	 */
	private Action actionMoveDown;

	/**
	 * テーブルスタイル
	 */
	private static int TABLE_STYLE = SWT.FULL_SELECTION | SWT.BORDER;

	/**
	 * ツールチップない場合のテキスト
	 */
	public String toolTipNoneTblText = ""; //$NON-NLS-1$

	/**
	 * フォーカスを設定する
	 * 
	 */
	public void setFocus() {

	}

	/**
	 * ビューのアクションを生成する
	 * 
	 */
	private void makeActions() {
		Action[] actions = new Action[4];
		for (int cnt = 0; cnt < actions.length; cnt++) {
			actions[cnt] = new Action() {
				public void run() {
					handleActions(this);
				}
			};
		}
		actionAdd = actions[0];
		actionAdd.setText(Messages.getString("CsvxEditorDialog.2")); //$NON-NLS-1$
		actionAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));
		actionDel = actions[1];
		actionDel.setText(Messages.getString("CsvxEditorDialog.3")); //$NON-NLS-1$
		actionDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionMoveUp = actions[2];
		actionMoveUp.setText(Messages.getString("CsvxEditorDialog.4")); //$NON-NLS-1$
		actionMoveUp.setImageDescriptor(SwtUtil
				.loadImageDescriptor("icons/arrow_up.gif")); //$NON-NLS-1$

		actionMoveDown = actions[3];
		actionMoveDown.setText(Messages.getString("CsvxEditorDialog.6")); //$NON-NLS-1$
		actionMoveDown.setImageDescriptor(SwtUtil
				.loadImageDescriptor("icons/arrow_down.gif")); //$NON-NLS-1$
	}

	/**
	 * ツリービューで選択されている属性／要素を取得する
	 * 
	 * @return 属性か要素
	 */
	private TreeObject getCurrentTreeObj() {
		ISelection selection = treeComplexDetail.getSelection();
		Object item = ((IStructuredSelection) selection).getFirstElement();
		if (item instanceof TreeObject) {
			TreeObject treeObj = (TreeObject) item;
			Object data = treeObj.getData();

			if ((data instanceof MispSchema.MispSchemaAttribute)
					|| (data instanceof MispSchema.MispSchemaElement)
					|| (data instanceof MispSchema.MispSchemaComplexType))
				return treeObj;
		}
		return null;
	}

	/**
	 * アクションを処理する
	 * 
	 * @param action
	 *            アクション
	 */
	private void handleActions(Action action) {
		if (action == actionAdd) {
			TreeObject treeObject = getCurrentTreeObj();
			if (treeObject != null) {
				addCsvx(treeObject);
			}
		} else if (action == actionDel) {
			int index = tblCsvx.getSelectionIndex();
			if (index >= 0) {
				currCx.delCxColumn(index);
			}
		} else if ((action == actionMoveUp) || (action == actionMoveDown)) {
			int index = tblCsvx.getSelectionIndex();
			MispSchemaCxColumn column = currCx.getCxColumn(index);
			if (index >= 0) {
				currCx.moveCxColumn(column, (action == actionMoveUp) ? -1 : 1);
			}
		} else
			return;

		viewCsvx.refresh();
	}

	private String addCsvx(TreeObject treeObject) {
		TreeObject temp = treeObject;
		String xpathPrefix = ""; //$NON-NLS-1$
		String namePrefix = ""; //$NON-NLS-1$
		while ((temp = temp.getParent()) != null) {
			Object data = temp.getData();
			if (data instanceof MispSchemaElement) {
				MispSchemaElement element = (MispSchemaElement) data;
				MispSchemaComplexType ct;
				ct = element.findComplexType();
				if (ct == null)
					break;
				if (element.getMaxOccursVal() == 1) {
					xpathPrefix = element.getName() + "/" + xpathPrefix; //$NON-NLS-1$
					namePrefix = element.getName() + "_" + xpathPrefix; //$NON-NLS-1$

				} else {
					int index = getAddToIndex(element, "", false); //$NON-NLS-1$
					xpathPrefix = element.getName() + "[" + index + "]/" //$NON-NLS-1$ //$NON-NLS-2$
							+ xpathPrefix;
					namePrefix = element.getName() + "[" + index + "]_" //$NON-NLS-1$ //$NON-NLS-2$
							+ xpathPrefix;
				}
			} else
				break;
		}

		List newCxColumnList = new ArrayList();
		addCsvx2(newCxColumnList, xpathPrefix, namePrefix, treeObject.getData());
		for (int cnti = 0; cnti < newCxColumnList.size(); cnti++) {
			NewCsvxInfo newInfo = (NewCsvxInfo) newCxColumnList.get(cnti);
			MispSchemaCxColumn cx = currCx.addCxColumn(-1);
			cx.setXpath(newInfo.xpath);
			newInfo.isDup = false;
			for (int cntj = 0; cntj < newCxColumnList.size(); cntj++) {
				NewCsvxInfo newInfo2 = (NewCsvxInfo) newCxColumnList.get(cntj);
				if (newInfo2.name.equals(newInfo.name))
					newInfo.isDup = true;
			}

			cx.setName(newInfo.isDup ? newInfo.fullName : newInfo.name);
		}
		return xpathPrefix;
	}

	class NewCsvxInfo {
		String xpath = ""; //$NON-NLS-1$

		String name = ""; //$NON-NLS-1$

		String fullName = ""; //$NON-NLS-1$

		boolean isDup = false;
	}

	private int getAddToIndex(MispSchemaElement element, String xpathPrefix,
			boolean blOccurs) {
		GetIndexDialog dialog = new GetIndexDialog(parentShell);
		dialog.setData(element, xpathPrefix, blOccurs);
		dialog.setIndex("1"); //$NON-NLS-1$
		while (true) {
			int index = -1;
			dialog.open();
			try {
				index = Integer.parseInt(dialog.getIndex());
				if ((index >= 1)
						&& ((index <= element.getMaxOccursVal()) || (element
								.getMaxOccursVal() < 0)))
					return index;
			} catch (Exception e) {

			}
		}
	}

	private void addCsvx2(List newCxColumnList, String xpathPrefix,
			String namePrefix, Object data) {
		if (data instanceof MispSchemaComplexType) {
			MispSchemaComplexType ct = (MispSchemaComplexType) data;
			List attrEmList = ct
					.getAttributeElementList(true, true, true, true);
			for (int cnt = 0; cnt < attrEmList.size(); cnt++) {
				addCsvx2(newCxColumnList, xpathPrefix, namePrefix, attrEmList
						.get(cnt));
			}
		} else if (data instanceof MispSchemaElement) {
			MispSchemaElement element = (MispSchemaElement) data;
			int grp = currCx.getSchema().getElementTypeGrp(element.getType());
			if ((grp == MispSchema.GRP_COMPLEX_INTERNAL)
					|| (grp == MispSchema.GRP_COMPLEX_EXTERNAL)) {
				MispSchemaComplexType ct = element.findComplexType();
				if (ct != null) {
					if (element.getMaxOccursVal() == 1) {
						addCsvx2(newCxColumnList, xpathPrefix
								+ element.getName() + "/", namePrefix //$NON-NLS-1$
								+ element.getName() + "_", ct); //$NON-NLS-1$
					} else {
						int index = getAddToIndex(element, xpathPrefix, true);
						for (int cnt = 0; cnt++ < index;) {
							addCsvx2(newCxColumnList, xpathPrefix
									+ element.getName() + "[" + cnt //$NON-NLS-1$
									+ "]" + "/", namePrefix //$NON-NLS-1$ //$NON-NLS-2$
									+ element.getName() + "[" + cnt //$NON-NLS-1$
									+ "]" + "_", ct); //$NON-NLS-1$ //$NON-NLS-2$
						}
					}
				}
			} else {
				if (element.getMaxOccursVal() == 1) {
					NewCsvxInfo newInfo = new NewCsvxInfo();
					newInfo.xpath = xpathPrefix + element.getName();
					newInfo.fullName = namePrefix + element.getName();
					newInfo.name = element.getName();
					newCxColumnList.add(newInfo);
				} else {
					int index = getAddToIndex(element, xpathPrefix, true);
					for (int cnt = 0; cnt++ < index;) {
						NewCsvxInfo newInfo = new NewCsvxInfo();
						newInfo.xpath = xpathPrefix + element.getName() + "[" //$NON-NLS-1$
								+ cnt + "]"; //$NON-NLS-1$
						newInfo.fullName = namePrefix + element.getName() + "[" //$NON-NLS-1$
								+ cnt + "]"; //$NON-NLS-1$
						newInfo.name = element.getName() + cnt;
						newCxColumnList.add(newInfo);
					}
				}
			}
		} else if (data instanceof MispSchemaAttribute) {
			MispSchemaAttribute attr = (MispSchemaAttribute) data;
			NewCsvxInfo newInfo = new NewCsvxInfo();
			newInfo.xpath = xpathPrefix + "@" + attr.getName(); //$NON-NLS-1$
			newInfo.fullName = namePrefix + "@" + attr.getName(); //$NON-NLS-1$
			newInfo.name = "@" + attr.getName(); //$NON-NLS-1$
			newCxColumnList.add(newInfo);
		}
	}

	/**
	 * ダイアログの画面を生成する
	 * 
	 * @param composite
	 *            コンポジット
	 */
	public void createPropertyDialogArea(Composite composite) {
		int colNum = 3;

		makeActions();

		this.parent = composite;
		parent.setLayout(SwtUtil.makeGridLayout(colNum));

		new Label(parent, SWT.NULL).setText(Messages
				.getString("CsvxEditorDialog.10")); //$NON-NLS-1$
		new Label(parent, SWT.NULL).setText(Messages
				.getString("CsvxEditorDialog.11")); //$NON-NLS-1$
		Button btnHeader = new Button(parent, SWT.NULL);
		btnHeader.setText(Messages.getString("CsvxEditorDialog.46")); //$NON-NLS-1$
		btnHeader.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				TextMessageDialog dialog = new TextMessageDialog(parentShell,
						Messages.getString("CsvxEditorDialog.47")); //$NON-NLS-1$
				dialog.setMsg(currCx.getCsvHeader());
				dialog.open();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		Composite compComplexDetail = new Composite(composite, SWT.NULL);
		compComplexDetail.setLayoutData(SwtUtil.makeVarGridData(320, 100, 1));
		compComplexDetail.setLayout(new FillLayout());

		treeComplexDetail = new TreeViewer(compComplexDetail, SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER);
		treeComplexDetail
				.setContentProvider(new SchemaEditorViewProvider.ComplexTypeDetailContentProvider());
		treeComplexDetail
				.setLabelProvider(new SchemaEditorViewProvider.ComplexTypeDetailLabelProvider(
						false));
		makeTreeViewerContextMenu(treeComplexDetail);

		tblCsvx = new Table(parent, TABLE_STYLE);
		tblCsvx.setLayoutData(SwtUtil.makeVarGridData(400, 100, 2));
		{
			String[] cols = {
					"", Messages.getString("CsvxEditorDialog.17"), Messages.getString("CsvxEditorDialog.15") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			int[] widths = { 40, 180, 180 };
			SwtUtil.setupTable(tblCsvx, cols, widths);
		}
		viewCsvx = new TableViewer(tblCsvx);
		viewCsvx
				.setContentProvider(new CsvxEditorDialogProvider.CxContentProvider());
		viewCsvx.setLabelProvider(new CsvxEditorDialogProvider.CxLabelProvider(
				this));

		{
			String[] properties = new String[] { "", "xpath", "name", }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			viewCsvx.setColumnProperties(properties);
			CellEditor[] editors = new CellEditor[] { null,
					new TextCellEditor(tblCsvx), new TextCellEditor(tblCsvx), };
			viewCsvx.setCellEditors(editors);
			viewCsvx
					.setCellModifier(new CsvxEditorDialogProvider.CxCellModifier(
							this, viewCsvx));
		}
		{
			Action[] actions = new Action[] { actionDel, actionMoveUp,
					actionMoveDown };
			int[] enableTypes = new int[] {
					TableViewerMenuListener.ENABLE_TYPE_SELECTED,
					TableViewerMenuListener.ENABLE_TYPE_NOT_FIRST,
					TableViewerMenuListener.ENABLE_TYPE_NOT_LAST };
			makeTableViewerContextMenu(null, viewCsvx, actions, enableTypes);
		}
		tblCsvx
				.addMouseMoveListener(new TableMouseMoveListener(this, viewCsvx));

		loadMispSchemaCxToGui();
	}

	/**
	 * テーブルビューのコンテキストメニューを作成する
	 * 
	 * @param parentViewer
	 *            親テーブルビュー
	 * @param viewer
	 *            テーブルビュー
	 * @param actions
	 *            アクション配列
	 * @param enableTypes
	 *            アクション有効フラグ配列
	 */
	private void makeTableViewerContextMenu(TableViewer parentViewer,
			TableViewer viewer, Action[] actions, int[] enableTypes) {
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TableViewerMenuListener(parentViewer,
				viewer, actions, enableTypes));
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
	}

	/**
	 * CSV項目を設定する
	 * 
	 * @param cx
	 *            設定するCSV項目
	 */
	public void setMispSchemaCx(MispSchemaCx cx) {
		this.currCx = cx;
	}

	private void loadMispSchemaCxToGui() {
		MispSchema schema = currCx.getSchema();
		MispSchemaComplexType ct = schema.findComplexTypeByName(schema
				.getFeatureElement().getType());
		treeComplexDetail.setInput(ct);
		SwtUtil.expandToLevel(treeComplexDetail, null, DFT_TREE_LEVEL);
		viewCsvx.setInput(currCx);

	}

	/**
	 * テーブルビューを更新する
	 * 
	 * @param viewer
	 *            更新するテーブルビュー
	 * @param element
	 *            更新するエレメント
	 */
	public void updateTableViewer(TableViewer viewer, Object element) {
		if (element == null)
			viewer.refresh();
		else
			viewer.refresh(element);
	}

	/**
	 * ダイアログを閉じる
	 * 
	 * @param flgCancel
	 *            キャンセルフラグ
	 */
	public void dialogClose(boolean flgCancel) {
	}

	public void widgetSelected(SelectionEvent e) {
	}

	public void widgetDefaultSelected(SelectionEvent e) {
	}

	/**
	 * currCxを取得する
	 * 
	 * @return Returns the currCx.
	 */
	public MispSchemaCx getCurrCx() {
		return currCx;
	}

	/**
	 * ツリービューのコンテキストメニューを作成する
	 * 
	 * @param parentViewer
	 *            親テーブルビュー
	 * @param viewer
	 *            テーブルビュー
	 * @param actions
	 *            アクション配列
	 * @param enableTypes
	 *            アクション有効フラグ配列
	 */
	private void makeTreeViewerContextMenu(TreeViewer viewer) {
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TreeViewerMenuListener());
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
	}

	/**
	 * ツリービューメニューリスナー
	 */
	private class TreeViewerMenuListener implements IMenuListener {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
		 */
		public void menuAboutToShow(IMenuManager manager) {
			ISelection selection = treeComplexDetail.getSelection();
			Object item = ((IStructuredSelection) selection).getFirstElement();

			if (item instanceof TreeObject) {
				// TreeObject treeObj = (TreeObject) item;
				// Object data = treeObj.getData();
				// boolean isComplexType = (data instanceof
				// MispSchema.MispSchemaComplexType);
				// boolean isElement = (data instanceof
				// MispSchema.MispSchemaElement);
				// boolean isAttr = (data instanceof
				// MispSchema.MispSchemaAttribute);
				actionAdd.setEnabled(true);
				manager.add(actionAdd);
			}
			manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		}
	}

}
