package daruma.client.schemaeditor.ui.views;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Item;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import daruma.bus.processor.csv.MispSchemaCx;
import daruma.bus.processor.csv.MispSchemaCx.MispSchemaCxColumn;
import daruma.client.schemaeditor.Messages;

/**
 * スキーマエディタビューの表示内容プロバイダ
 * 
 * @author shumi
 * 
 */
public class CsvxEditorDialogProvider {
	/**
	 * 無効アイコン
	 */
	public static Image invalidImg = PlatformUI.getWorkbench()
			.getSharedImages().getImage(ISharedImages.IMG_OBJS_WARN_TSK);

	/**
	 * 名前空間セルModifier
	 * 
	 * @author shumi
	 * 
	 */
	public static class CxCellModifier implements ICellModifier {
		/**
		 * テーブルビュー
		 */
		CsvxEditorDialog dlg;

		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param viewer
		 *            テーブルビュー
		 */
		public CxCellModifier(CsvxEditorDialog dlg, TableViewer viewer) {
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			return property.length() > 0;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			MispSchemaCxColumn column = (MispSchemaCxColumn) element;
			if (property.equals("xpath")) //$NON-NLS-1$
				value = column.getXpath();
			if (property.equals("name")) //$NON-NLS-1$
				value = column.getName();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			MispSchemaCxColumn column = (MispSchemaCxColumn) element;
			if (property.equals("xpath")) //$NON-NLS-1$
			{
				column.setXpath((String) value);
				viewer.refresh(element);
			}
			if (property.equals("name")) //$NON-NLS-1$
			{
				column.setName((String) value);
				viewer.refresh();
			}

		}
	}

	/**
	 * 名前空間コンテントプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class CxContentProvider implements IStructuredContentProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if ((inputElement != null)
					&& (inputElement instanceof MispSchemaCx)) {
				MispSchemaCx cx = (MispSchemaCx) inputElement;
				return cx.getCxColumnArray();
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}
	}

	/**
	 * 名前空間ラベルプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class CxLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		CsvxEditorDialog dlg;

		public CxLabelProvider(CsvxEditorDialog dlg) {
			this.dlg = dlg;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}
	}
	
	/**
	 * カラムの検証結果を取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return 検証結果
	 */
	public static String getColValidate(Object element, int columnIndex) {
		String validate = ""; //$NON-NLS-1$
		if (element instanceof MispSchemaCxColumn) {
			MispSchemaCxColumn column = (MispSchemaCxColumn) element;
			switch (columnIndex) {
			case 0:
				validate = column.validate(true);
				break;
			case 1:
				//validate = column.validateXpath(true);
				break;
			case 2:
				//validate = column.validateName(true);
				break;
			}
		}
		return validate;
	}
	
	/**
	 * カラムのイメージを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return イメージ
	 */
	public static Image getColImage(Object element, int columnIndex) {
		return ((columnIndex == 0) && (getColValidate(element, columnIndex).length() > 0)) ? SchemaEditorView.invalidImg
				: SchemaEditorView.validImg;
	}
	
	
	/**
	 * カラムのテキストを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return テキスト
	 */
	public static String getColText(Object element, int columnIndex) {
		if (element instanceof MispSchemaCxColumn) {
			MispSchemaCxColumn column = (MispSchemaCxColumn) element;
			if (columnIndex == 0)
				return "" + (column.getIndex() + 1); //$NON-NLS-1$
			else if (columnIndex == 1)
				return column.getXpath();
			else if (columnIndex == 2)
				return column.getName();
		}
		return ""; //$NON-NLS-1$
	}
	
	/**
	 * 
	 * カラムのチップテキストを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return チップテキスト
	 */
	public static String getColToolTipText(Object element, int columnIndex) {
		String tip = ""; //$NON-NLS-1$
		if (element instanceof MispSchemaCxColumn) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("CsvxEditorDialogProvider.2"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("CsvxEditorDialogProvider.3"); //$NON-NLS-1$
				break;
			}
		} 
		return tip;
	}

}
