package daruma.client.schemaeditor.ui.views;

import java.io.File;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchemaAccess;
import daruma.bus.processor.csv.MispSchemaCx;
import daruma.bus.processor.csv.MispSchemaCxAccess;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;
import daruma.client.schemaeditor.Application;
import daruma.client.schemaeditor.ApplicationWorkbenchWindowAdvisor;
import daruma.client.schemaeditor.Messages;
import daruma.client.schemaeditor.ui.views.SchemaEditorViewProvider.NamespaceLabelProvider;

/**
 * スキーマエディタビュー
 * 
 * @author shumi
 * 
 */
public class SchemaEditorView extends ViewPart {
	/**
	 * ロガー
	 */
	static private Log log = LogFactory.getLog(SchemaEditorView.class);

	/**
	 * ビューID
	 */
	public static final String ID = SchemaEditorView.class.getName();

	/**
	 * 無効アイコン
	 */
	public static Image invalidImg = PlatformUI.getWorkbench()
			.getSharedImages().getImage(ISharedImages.IMG_OBJS_WARN_TSK);

	/**
	 * 有効アイコン
	 */
	public static Image validImg = null;

	/**
	 * プロパティファイル
	 */
	static private final String PROPS_FILE = "etc/schemaeditor.properties"; //$NON-NLS-1$

	/**
	 * プロパティファイル
	 */
	static private final String NEW_SCHEMA_TEMPLATE = "etc/new.template"; //$NON-NLS-1$

	/**
	 * ツリービューの展開レベル
	 */
	private static int DFT_TREE_LEVEL = 8;

	/**
	 * テーブルスタイル
	 */
	private static int TABLE_STYLE = SWT.FULL_SELECTION | SWT.BORDER;

	/**
	 * 親コンポジット
	 */
	private Composite parent;

	/**
	 * IDテキストボックス
	 */
	private Text txtID;

	/**
	 * IDラベル
	 */
	private Label lblID;

	/**
	 * ターゲット名前空間テキストボックス
	 */
	private Text txtTargetNamespace;

	/**
	 * 注釈テキストボックス
	 */
	private Text txtAnnotation;

	/**
	 * ターゲット名前空間ラベル
	 */
	private Label lblTargetNamespace;

	/**
	 * フィーチャタイプ名テキストボックス
	 */
	private Text txtFeatureTypeName;

	/**
	 * フィーチャタイプ名ラベル
	 */
	private Label lblFeatureTypeName;

	/**
	 * フィーチャタイプデータ型コンボボックス
	 */
	private Combo cmbFeatureTypeType;

	private Label lblMinOccurs;

	private Text txtMinOccurs;

	private Label lblMaxOccurs;

	private Combo cmbMaxOccurs;

	private Button btnIndexed;

	/**
	 * フィーチャタイプデータ型ラベル
	 */
	private Label lblFeatureTypeType;

	/**
	 * インポートスキーマテーブルビュー
	 */
	private TableViewer viewImport;

	/**
	 * 名前空間テーブルビュー
	 */
	private TableViewer viewNamespace;

	/**
	 * 複合型テーブルビュー
	 */
	private TableViewer viewComplexType;

	/**
	 * 名前空間タブアイテム
	 */
	private TabItem tabNamespace;

	/**
	 * インポートスキーマタブアイテム
	 */
	private TabItem tabImport;

	/**
	 * 複合型タブアイテム
	 */
	private TabItem tabComplexType;
	
	private TabFolder  tabFolder;

	/**
	 * カレントファイルパス
	 */
	private String currFile;

	/**
	 * CSV形式スキーマ
	 */
	private MispSchemaAccess access = new MispSchemaAccess();

	/**
	 * XML形式Cx
	 */
	MispSchemaCxAccess cxAccess = new MispSchemaCxAccess();

	/**
	 * カレントスキーマ
	 */
	private MispSchema currSchema;

	/**
	 * カレントツリーオブジェ宇土
	 */
	private TreeObject currTreeObj;

	/**
	 * オリジナルスキーマ
	 */
	private MispSchema orgSchema;

	private boolean initFlg = false;

	/**
	 * ロード中フラグ
	 */
	private int loading;

	/**
	 * テキストボックスリスナー
	 */
	private TextModifyListener textModifyListener;

	/**
	 * テキストボックスリスナー
	 */
	private TextFocusListener textFocusListener;

	/**
	 * コンボボックスリスナー
	 */
	private ComboSelectionListener comboSelectionListener;

	/**
	 * ボタンリスナー
	 */
	private ButtonSelectionListener buttonSelectionListener;
	

	/**
	 * 名前空間追加アクション
	 */
	private Action actionNsAdd;

	/**
	 * 名前空間削除アクション
	 */
	private Action actionNsDel;

	/**
	 * インポートスキーマ追加アクション
	 */
	private Action actionImAdd;

	/**
	 * インポートスキーマ追加アクション
	 */
	private Action actionImAdd2;

	/**
	 * インポートスキーマリロードアクション
	 */
	private Action actionImReLoad;

	/**
	 * インポートスキーマリロードアクション
	 */
	private Action actionImReLocation;

	/**
	 * インポートスキーマ削除アクション
	 */
	private Action actionImDel;

	/**
	 * 複合型追加アクション
	 */
	private Action actionCtAdd;

	/**
	 * 複合型削除アクション
	 */
	private Action actionCtDel;

	/**
	 * 属性追加アクション
	 */
	private Action actionAttrAdd;

	/**
	 * 属性削除アクション
	 */
	private Action actionAttrDel;

	/**
	 * 要素追加アクション
	 */
	private Action actionEmAdd;

	/**
	 * 要素削除アクション
	 */
	private Action actionEmDel;

	/**
	 * 子属性追加アクション
	 */
	private Action actionCtAttrAdd;

	/**
	 * 子要素追加アクション
	 */
	private Action actionCtEmAdd;

	/**
	 * 上へ移動
	 */
	private Action actionMoveUp;

	/**
	 * 下へ移動
	 */
	private Action actionMoveDown;

	/**
	 * ツールチップない場合のテキスト
	 */
	private String toolTipNoneText = ""; //$NON-NLS-1$

	/**
	 * ツールチップない場合のテキスト
	 */
	public String toolTipNoneTblText = ""; //$NON-NLS-1$

	private boolean windows;

	/**
	 * ツリービュー
	 */
	private TreeViewer treeComplexDetail;

	/**
	 * 要素／属性名ラベル
	 */
	private Label lblDetailName;

	/**
	 * 要素／属性名タイプ
	 */
	private Label lblDetailType;

	/**
	 * 要素／属性名テキストボックス
	 */
	private Text txtDetailName;

	/**
	 * 要素／属性名コンボ
	 */
	private Combo cmbDetailType;

	/**
	 * プロパティ
	 */
	private SchemaEditorViewProp prop;

	/**
	 * スキーマ編集操作クラス
	 */
	private SchemaEditorViewOperation operation;
	
	/**
	 * スキーマエディタをビューを生成する
	 * 
	 */
	public SchemaEditorView() {
		prop = SchemaEditorViewProp.loadProp(PROPS_FILE);
		access.setDfsOutputVersion(prop.isCsvOutputVersion());
		operation = new SchemaEditorViewOperation(this, prop);
		windows = System.getProperty("os.name").startsWith("Windows"); //$NON-NLS-1$ //$NON-NLS-2$
		toolTipNoneText = ""; //$NON-NLS-1$
		toolTipNoneTblText = windows ? "" : " "; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * 画面を作成する
	 * @param ButtonSelectionListener 
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		int colNum = 8;

		/* アクションを作成する */
		makeActions();

		this.parent = parent;
		parent.setLayout(SwtUtil.makeGridLayout(colNum));

		textModifyListener = new TextModifyListener();
		textFocusListener = new TextFocusListener();
		comboSelectionListener = new ComboSelectionListener();
		buttonSelectionListener = new ButtonSelectionListener();
		
		/* スキーマID */
		new Label(parent, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.1")); //$NON-NLS-1$
		lblID = new Label(parent, SWT.NULL);
		lblID.setText("  "); //$NON-NLS-1$
		txtID = new Text(parent, SWT.BORDER);
		txtID.setLayoutData(SwtUtil.makeHorVarGridData(200, 0, colNum - 3));
		txtID.addFocusListener(textFocusListener);
		txtID.addModifyListener(textModifyListener);
		Group grpAnnotation = new Group(parent, SWT.NULL);
		grpAnnotation.setText(Messages.getString("SchemaEditorView.153")); //$NON-NLS-1$
		grpAnnotation.setLayoutData(SwtUtil.makeHorVarGridData(200, 70, 1, 3));
		grpAnnotation.setLayout(SwtUtil.makeGridLayout(1));

		/* ターゲット名前空間 */
		new Label(parent, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.2")); //$NON-NLS-1$
		lblTargetNamespace = new Label(parent, SWT.NULL);
		lblTargetNamespace.setText("  "); //$NON-NLS-1$
		txtTargetNamespace = new Text(parent, SWT.BORDER);
		txtTargetNamespace.setLayoutData(SwtUtil.makeHorVarGridData(200, 0,
				colNum - 3));
		txtTargetNamespace.addFocusListener(textFocusListener);
		txtTargetNamespace.addModifyListener(textModifyListener);

		txtAnnotation = new Text(grpAnnotation, SWT.BORDER | SWT.MULTI
				| SWT.V_SCROLL);
		txtAnnotation.setLayoutData(SwtUtil.makeMaximumGridData());
		txtAnnotation.addFocusListener(textFocusListener);
		txtAnnotation.addModifyListener(textModifyListener);

		/* フィーチャタイプ名 */
		new Label(parent, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.3")); //$NON-NLS-1$
		lblFeatureTypeName = new Label(parent, SWT.NULL);
		lblFeatureTypeName.setText("  "); //$NON-NLS-1$
		txtFeatureTypeName = new Text(parent, SWT.BORDER);
		txtFeatureTypeName.setLayoutData(SwtUtil.makeFixedGridData(150, 0, 1));
		txtFeatureTypeName.addFocusListener(textFocusListener);
		txtFeatureTypeName.addModifyListener(textModifyListener);

		/* フィーチャタイプ型 */
		new Label(parent, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.5")); //$NON-NLS-1$
		lblFeatureTypeType = new Label(parent, SWT.NULL);
		lblFeatureTypeType.setText("  "); //$NON-NLS-1$
		cmbFeatureTypeType = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
		cmbFeatureTypeType.setLayoutData(SwtUtil.makeFixedGridData(150, 0, 1));
		cmbFeatureTypeType.addFocusListener(textFocusListener);
		cmbFeatureTypeType.addModifyListener(textModifyListener);
		cmbFeatureTypeType.addSelectionListener(comboSelectionListener);

		if (true) {
			tabFolder = new TabFolder(parent, SWT.NULL);
			tabFolder.setLayoutData(SwtUtil.makeVarGridData(500, 100, colNum));

			/* 名前空間タブ */
			tabNamespace = new TabItem(tabFolder, SWT.NULL);
			Composite comp1 = new Composite(tabFolder, SWT.NULL);
			tabNamespace.setText(Messages.getString("SchemaEditorView.7")); //$NON-NLS-1$
			tabNamespace.setControl(comp1);
			setupNamespaceTable(comp1);

			/* インポートスキーマタブ */
			tabImport = new TabItem(tabFolder, SWT.NULL);
			Composite comp2 = new Composite(tabFolder, SWT.NULL);
			tabImport.setText(Messages.getString("SchemaEditorView.8")); //$NON-NLS-1$
			tabImport.setControl(comp2);
			setupImportTable(comp2);

			/* 複合型タブ */
			tabComplexType = new TabItem(tabFolder, SWT.NULL);
			Composite comp3 = new Composite(tabFolder, SWT.NULL);
			tabComplexType.setText(Messages.getString("SchemaEditorView.9")); //$NON-NLS-1$
			tabComplexType.setControl(comp3);
			setupComplexTypeTable(comp3);

			tabFolder.setSelection(2);
		} else {
			/* @deprecated */
			Group grpNamespace = new Group(parent, SWT.NULL);
			grpNamespace.setText(Messages.getString("SchemaEditorView.10")); //$NON-NLS-1$
			grpNamespace.setLayoutData(SwtUtil
					.makeVarGridData(500, 100, colNum));
			setupNamespaceTable(grpNamespace);

			Group grpImport = new Group(parent, SWT.NULL);
			grpImport.setText(Messages.getString("SchemaEditorView.11")); //$NON-NLS-1$
			grpImport.setLayoutData(SwtUtil.makeVarGridData(500, 100, colNum));
			setupImportTable(grpImport);

			Group grpComplexType = new Group(parent, SWT.NULL);
			grpComplexType.setText(Messages.getString("SchemaEditorView.12")); //$NON-NLS-1$
			grpComplexType.setLayoutData(SwtUtil.makeVarGridData(500, 250,
					colNum));
			setupComplexTypeTable(grpComplexType);
		}

		if (!windows) {
			/* Linux GTKではツールチップが正しく動かない場合があります */
			viewNamespace.getTable().setToolTipText(toolTipNoneTblText);
			viewImport.getTable().setToolTipText(toolTipNoneTblText);
			viewComplexType.getTable().setToolTipText(toolTipNoneTblText);
			treeComplexDetail.getTree().setToolTipText(toolTipNoneTblText);
		}

		makeDrugAndDrop();

		/* 起動後初期状態は新規ファイルを編集 */
		validImg = SwtUtil.makeImg(invalidImg.getBounds().width, invalidImg
				.getBounds().height);
		openFile(""); //$NON-NLS-1$
		validImg = null;
		openFile(""); //$NON-NLS-1$
		if ((Application.firstFile != null)
				&& (Application.firstFile.length() > 0))
			openFile(Application.firstFile);
		initFlg = true;
	}

	private void makeDrugAndDrop() {
		/**
		 * ドラッグ＆ドロップ機能
		 */
		final FileTransfer fileTransfer = FileTransfer.getInstance();
		Transfer[] types = new Transfer[] { fileTransfer };
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		DropTarget source = new DropTarget(parent, operations);
		source.setTransfer(types);
		source.addDropListener(new DropTargetListener() {

			public void dragEnter(DropTargetEvent event) {
			}

			public void dragOperationChanged(DropTargetEvent event) {
			}

			public void dragOver(DropTargetEvent event) {
			}

			public void drop(DropTargetEvent event) {
				if (fileTransfer.isSupportedType(event.currentDataType)) {
					String[] files = (String[]) event.data;
					if ((files.length > 0)
							&& files[0]
									.endsWith(MispSchemaAccess.DFS_DFT_FILE_EXT)) {
						if (MessageDialog.openQuestion(null, Messages.getString("SchemaEditorView.165"), Messages.getString("SchemaEditorView.166") //$NON-NLS-1$ //$NON-NLS-2$
								+ files[0] + Messages.getString("SchemaEditorView.167"))) { //$NON-NLS-1$
							openFile(files[0]);
						}
					}
				}
			}

			public void dragLeave(DropTargetEvent event) {
				// TODO Auto-generated method stub

			}

			public void dropAccept(DropTargetEvent event) {
				// TODO Auto-generated method stub

			}

		});

	}

	/**
	 * ビューのアクションを生成する
	 * 
	 */
	private void makeActions() {
		Action[] actions = new Action[17];
		for (int cnt = 0; cnt < actions.length; cnt++) {
			actions[cnt] = new Action() {
				public void run() {
					handleActions(this);
				}
			};
		}
		actionNsAdd = actions[0];
		actionNsAdd.setText(Messages.getString("SchemaEditorView.62")); //$NON-NLS-1$
		actionNsAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));
		actionNsDel = actions[1];
		actionNsDel.setText(Messages.getString("SchemaEditorView.63")); //$NON-NLS-1$
		actionNsDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionImAdd = actions[2];
		actionImAdd.setText(Messages.getString("SchemaEditorView.64")); //$NON-NLS-1$
		actionImAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));

		actionImAdd2 = actions[3];
		actionImAdd2.setText(Messages.getString("SchemaEditorView.158")); //$NON-NLS-1$
		actionImAdd2.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_OBJ_FILE));

		actionImDel = actions[4];
		actionImDel.setText(Messages.getString("SchemaEditorView.65")); //$NON-NLS-1$
		actionImDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionCtAdd = actions[5];
		actionCtAdd.setText(Messages.getString("SchemaEditorView.66")); //$NON-NLS-1$
		actionCtAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));

		actionCtDel = actions[6];
		actionCtDel.setText(Messages.getString("SchemaEditorView.67")); //$NON-NLS-1$
		actionCtDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionAttrAdd = actions[7];
		actionAttrAdd.setText(Messages.getString("SchemaEditorView.68")); //$NON-NLS-1$
		actionAttrAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));
		actionAttrDel = actions[8];
		actionAttrDel.setText(Messages.getString("SchemaEditorView.69")); //$NON-NLS-1$
		actionAttrDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionEmAdd = actions[9];
		actionEmAdd.setText(Messages.getString("SchemaEditorView.70")); //$NON-NLS-1$
		actionEmAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));
		actionEmDel = actions[10];
		actionEmDel.setText(Messages.getString("SchemaEditorView.71")); //$NON-NLS-1$
		actionEmDel.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_DELETE));

		actionCtAttrAdd = actions[11];
		actionCtAttrAdd.setText(Messages.getString("SchemaEditorView.143")); //$NON-NLS-1$
		actionCtAttrAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));
		actionCtEmAdd = actions[12];
		actionCtEmAdd.setText(Messages.getString("SchemaEditorView.144")); //$NON-NLS-1$
		actionCtEmAdd.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_NEW_WIZARD));

		actionMoveUp = actions[13];
		actionMoveUp.setText(Messages.getString("SchemaEditorView.145")); //$NON-NLS-1$
		actionMoveUp.setImageDescriptor(SwtUtil
				.loadImageDescriptor("icons/arrow_up.gif")); //$NON-NLS-1$
		actionMoveDown = actions[14];
		actionMoveDown.setText(Messages.getString("SchemaEditorView.146")); //$NON-NLS-1$
		actionMoveDown.setImageDescriptor(SwtUtil
				.loadImageDescriptor("icons/arrow_down.gif")); //$NON-NLS-1$

		actionImReLoad = actions[15];
		actionImReLoad.setText(Messages.getString("SchemaEditorView.159")); //$NON-NLS-1$
		actionImReLoad.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_UP));

		actionImReLocation = actions[16];
		actionImReLocation.setText(Messages.getString("SchemaEditorView.160")); //$NON-NLS-1$
		actionImReLocation.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_OBJ_FILE));
	}

	/**
	 * アクションを処理する
	 * 
	 * @param action
	 *            アクション
	 */
	private void handleActions(Action action) {
		MispSchemaComplexType currCt = getCurrComplexType(currSchema);
		TreeObject treeObj = getCurrentAttributeTreeObj();
		Object currAttrEm = (treeObj == null) ? null : treeObj.getData();

		int op = SchemaEditorViewOperation.OP_NONE;

		if (action == actionNsAdd) {
			/* 名前空間追加 */
			op = operation.opNamespace(currSchema, null,
					SchemaEditorViewOperation.OP_NS_ADD, null);
			if (op != SchemaEditorViewOperation.OP_NONE)
				updateTableViewer(viewNamespace, null);
		} else if (action == actionNsDel) {
			/* 名前空間削除 */
			int index = viewNamespace.getTable().getSelectionIndex();
			if (index >= 0) {
				op = operation.opNamespace(currSchema, currSchema
						.getNamespace(index),
						SchemaEditorViewOperation.OP_NS_DEL, null);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTableViewer(viewNamespace, null);
			}
		} else if (action == actionImAdd) {
			/* インポートスキーマ追加 */
			op = operation.opImport(currSchema, null,
					SchemaEditorViewOperation.OP_IM_ADD, null);
			if (op != SchemaEditorViewOperation.OP_NONE)
				updateTableViewer(viewImport, null);
		} else if (action == actionImAdd2) {
			/* インポートスキーマ追加 */
			op = operation.opImport(currSchema, null,
					SchemaEditorViewOperation.OP_IM_ADD2, null);
			if (op != SchemaEditorViewOperation.OP_NONE) {
				refreshAll();
			}
		} else if (action == actionImReLoad) {
			int index = viewImport.getTable().getSelectionIndex();
			if (index >= 0) {
				/* インポートスキーマリロード */
				op = operation.opImport(currSchema,
						currSchema.getImport(index),
						SchemaEditorViewOperation.OP_IM_RELOAD, null);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				refreshAll();
			}
		} else if (action == actionImReLocation) {
			int index = viewImport.getTable().getSelectionIndex();
			if (index >= 0) {
				/* インポートスキーマリロケーション */
				op = operation.opImport(currSchema,
						currSchema.getImport(index),
						SchemaEditorViewOperation.OP_IM_RELOCATION, null);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				refreshAll();
			}
		} else if (action == actionImDel) {
			/* インポートスキーマ削除 */
			int index = viewImport.getTable().getSelectionIndex();
			if (index >= 0) {
				op = operation.opImport(currSchema,
						currSchema.getImport(index),
						SchemaEditorViewOperation.OP_IM_DEL, null);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTableViewer(viewImport, null);
			}
		} else if (action == actionCtAdd) {
			/* 複合型追加 */
			op = operation.opComplexType(currSchema, null,
					SchemaEditorViewOperation.OP_CT_ADD, null);
			if (op != SchemaEditorViewOperation.OP_NONE)
				updateTableViewer(viewComplexType, null);
		} else if (action == actionCtDel) {
			/* 複合型削除 */
			int index = viewComplexType.getTable().getSelectionIndex();
			if (index >= 0) {
				op = operation.opComplexType(currSchema, currSchema
						.getComplexType(index),
						SchemaEditorViewOperation.OP_CT_DEL, null);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTableViewer(viewComplexType, null);
			}
		} else if (currCt == null) {

		} else if (currAttrEm == null) {
			if (action == actionEmAdd) {
				/* 要素追加（複合型トップ） */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ET_ADD, new Integer(
								prop.isAddNewBefore() ? -1 : 0));
			} else if (action == actionAttrAdd) {
				/* 属性追加（複合型トップ） */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ATTR_ADD, new Integer(
								prop.isAddNewBefore() ? -1 : 0));
			}
			if (op != SchemaEditorViewOperation.OP_NONE)
				updateTreeParent(treeComplexDetail, treeObj);
		} else if (currAttrEm instanceof MispSchemaAttribute) {
			MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
			currCt = attr.getParentCt();
			if (action == actionAttrDel) {
				/* 属性削除 */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ATTR_DEL,
						(MispSchemaAttribute) currAttrEm);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			} else if (action == actionAttrAdd) {
				/* 前に属性追加 */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ATTR_ADD, new Integer(
								attr.getIndex()
										+ (prop.isAddNewBefore() ? 0 : 1)));
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			} else if ((action == actionMoveUp) || (action == actionMoveDown)) {
				op = operation
						.opAttribute(
								currSchema,
								attr,
								(action == actionMoveUp) ? SchemaEditorViewOperation.OP_ATTR_MOVEUP
										: SchemaEditorViewOperation.OP_ATTR_MOVEDOWN,
								null);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			}
		} else if (currAttrEm instanceof MispSchemaElement) {
			MispSchemaElement em = (MispSchemaElement) currAttrEm;
			currCt = em.getParentCt();
			if (action == actionEmDel) {
				/* 要素削除 */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ET_DEL,
						(MispSchemaElement) currAttrEm);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			} else if (action == actionEmAdd) {
				/* 前に要素追加 */
				op = operation.opComplexType(currSchema, currCt,
						SchemaEditorViewOperation.OP_CT_ET_ADD, new Integer(em
								.getIndex()
								+ (prop.isAddNewBefore() ? 0 : 1)));
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			} else if ((action == actionCtEmAdd) || (action == actionCtAttrAdd)) {
				currCt = ((MispSchemaElement) currAttrEm).findComplexType();
				if (currCt == null) {
					if (MessageDialog
							.openQuestion(
									this.getSite().getShell(),
									Messages.getString("SchemaEditorView.147"), Messages.getString("SchemaEditorView.148"))) { //$NON-NLS-1$ //$NON-NLS-2$
						/* 複合型追加 */
						op = operation.opElement(currSchema,
								(MispSchemaElement) currAttrEm,
								SchemaEditorViewOperation.OP_EM_NEW_CT, null);
						if (op != SchemaEditorViewOperation.OP_NONE) {
							currCt = ((MispSchemaElement) currAttrEm)
									.findComplexType();
							viewComplexType.refresh();
						}
					}
				}
				if (currCt != null) {
					if (action == actionCtEmAdd) {
						/* 子要素追加 */
						op = operation.opComplexType(currSchema, currCt,
								SchemaEditorViewOperation.OP_CT_ET_ADD,
								new Integer(prop.isAddNewBefore() ? -1 : 0));
					} else {
						/* 子属性追加 */
						op = operation.opComplexType(currSchema, currCt,
								SchemaEditorViewOperation.OP_CT_ATTR_ADD,
								new Integer(prop.isAddNewBefore() ? -1 : 0));
					}
					if (op != SchemaEditorViewOperation.OP_NONE)
						updateTreeObject(treeComplexDetail, treeObj);
				}
			} else if ((action == actionMoveUp) || (action == actionMoveDown)) {
				op = operation
						.opElement(
								currSchema,
								em,
								(action == actionMoveUp) ? SchemaEditorViewOperation.OP_EM_MOVEUP
										: SchemaEditorViewOperation.OP_EM_MOVEDOWN,
								null);
				if (op != SchemaEditorViewOperation.OP_NONE)
					updateTreeParent(treeComplexDetail, treeObj);
			}
		}
	}

	/**
	 * テーブルで選択されている複合型を取得する
	 * 
	 * @param schema
	 *            スキーマ
	 * @return 複合型
	 */
	private MispSchemaComplexType getCurrComplexType(MispSchema schema) {
		MispSchemaComplexType ct = null;
		int ctIndex = viewComplexType.getTable().getSelectionIndex();
		if ((ctIndex >= 0) && (ctIndex < schema.getComplexTypeSize())) {
			ct = schema.getComplexType(ctIndex);
		}
		return ct;
	}

	/**
	 * ツリービューで選択されている属性／要素を取得する
	 * 
	 * @return 属性か要素
	 */
	private TreeObject getCurrentAttributeTreeObj() {
		ISelection selection = treeComplexDetail.getSelection();
		Object item = ((IStructuredSelection) selection).getFirstElement();
		if (item instanceof TreeObject) {
			TreeObject treeObj = (TreeObject) item;
			Object data = treeObj.getData();

			if ((data instanceof MispSchemaAttribute)
					|| (data instanceof MispSchemaElement))
				return treeObj;
		}
		return null;
	}

	/**
	 * テーブルビューのコンテキストメニューを作成する
	 * 
	 * @param parentViewer
	 *            親テーブルビュー
	 * @param viewer
	 *            テーブルビュー
	 * @param actions
	 *            アクション配列
	 * @param enableTypes
	 *            アクション有効フラグ配列
	 */
	private void makeTableViewerContextMenu(TableViewer parentViewer,
			TableViewer viewer, Action[] actions, int[] enableTypes) {
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TableViewerMenuListener(parentViewer,
				viewer, actions, enableTypes));
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	/**
	 * ツリービューのコンテキストメニューを作成する
	 * 
	 * @param parentViewer
	 *            親テーブルビュー
	 * @param viewer
	 *            テーブルビュー
	 * @param actions
	 *            アクション配列
	 * @param enableTypes
	 *            アクション有効フラグ配列
	 */
	private void makeTreeViewerContextMenu(TableViewer parentViewer,
			TreeViewer viewer) {
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TreeViewerMenuListener());
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	/**
	 * 名前空間テーブルをセットアップする
	 * 
	 * @param container
	 *            コンテナ
	 */
	private void setupNamespaceTable(Composite container) {
		int grpColNum = 2;
		container.setLayout(SwtUtil.makeGridLayout(grpColNum));

		Table tblNamespace = new Table(container, TABLE_STYLE);

		tblNamespace
				.setLayoutData(SwtUtil.makeVarGridData(500, 100, grpColNum));
		{
			String[] cols = {
					Messages.getString("SchemaEditorView.0"), Messages.getString("SchemaEditorView.14"), Messages.getString("SchemaEditorView.15") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			int[] widths = { 40, 200, 400 };
			SwtUtil.setupTable(tblNamespace, cols, widths);
		}
		viewNamespace = new TableViewer(tblNamespace);
		viewNamespace
				.setContentProvider(new SchemaEditorViewProvider.NamespaceContentProvider());
		viewNamespace.setLabelProvider(new NamespaceLabelProvider());
		{
			String[] properties = new String[] { "", "prefix", "uri", }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			viewNamespace.setColumnProperties(properties);
			CellEditor[] editors = new CellEditor[] { null,
					new TextCellEditor(tblNamespace),
					new TextCellEditor(tblNamespace), };
			viewNamespace.setCellEditors(editors);
			viewNamespace
					.setCellModifier(new SchemaEditorViewProvider.NamespaceCellModifier(
							this, viewNamespace));
		}
		{
			Action[] actions = new Action[] { actionNsAdd, actionNsDel };
			int[] enableTypes = new int[] {
					TableViewerMenuListener.ENABLE_TYPE_ALWAYS,
					TableViewerMenuListener.ENABLE_TYPE_SELECTED };
			makeTableViewerContextMenu(null, viewNamespace, actions,
					enableTypes);
		}
		tblNamespace.addMouseMoveListener(new TableMouseMoveListener(this,
				viewNamespace));
	}

	/**
	 * インポートスキーマテーブルをセットアップする
	 * 
	 * @param container
	 *            コンテナ
	 */
	private void setupImportTable(Composite container) {
		int grpColNum = 2;
		container.setLayout(SwtUtil.makeGridLayout(grpColNum));
		Table tblImport = new Table(container, TABLE_STYLE);
		tblImport.setLayoutData(SwtUtil.makeVarGridData(500, 100, grpColNum));
		{
			String[] cols = {
					Messages.getString("SchemaEditorView.54"), Messages.getString("SchemaEditorView.18"), Messages.getString("SchemaEditorView.19") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			int[] widths = { 40, 350, 380 };
			SwtUtil.setupTable(tblImport, cols, widths);
		}
		viewImport = new TableViewer(tblImport);
		viewImport
				.setContentProvider(new SchemaEditorViewProvider.ImportContentProvider());
		viewImport
				.setLabelProvider(new SchemaEditorViewProvider.ImportLabelProvider());
		{
			String[] properties = new String[] { "", "uri", "location", }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			viewImport.setColumnProperties(properties);
			CellEditor[] editors = new CellEditor[] { null,
					new TextCellEditor(tblImport),
					new TextCellEditor(tblImport), };
			viewImport.setCellEditors(editors);
			viewImport
					.setCellModifier(new SchemaEditorViewProvider.ImportCellModifier(
							this, viewImport));
		}
		{
			Action[] actions = new Action[] { actionImAdd, actionImDel, null,
					actionImAdd2, null, actionImReLoad, actionImReLocation };
			int[] enableTypes = new int[] {
					TableViewerMenuListener.ENABLE_TYPE_ALWAYS,
					TableViewerMenuListener.ENABLE_TYPE_SELECTED,
					TableViewerMenuListener.ENABLE_TYPE_ALWAYS,
					TableViewerMenuListener.ENABLE_TYPE_ALWAYS,
					TableViewerMenuListener.ENABLE_TYPE_SELECTED,
					TableViewerMenuListener.ENABLE_TYPE_SELECTED };
			makeTableViewerContextMenu(null, viewImport, actions, enableTypes);
		}
		tblImport.addMouseMoveListener(new TableMouseMoveListener(this,
				viewImport));

	}

	/**
	 * 複合型テーブルをセットアップする
	 * 
	 * @param container
	 *            コンテナ
	 */
	private void setupComplexTypeTable(Composite container) {
		int grpColNum = 2;
		container.setLayout(SwtUtil.makeGridLayout(grpColNum));

		Composite compComplexTypeDetail = new Composite(container, SWT.NULL);
		compComplexTypeDetail.setLayoutData(SwtUtil.makeVarGridData(400, 0,
				grpColNum - 1));

		setupComplexTypeDetail(compComplexTypeDetail);

		Table tblComplexType = new Table(container, TABLE_STYLE);
		tblComplexType.setLayoutData(SwtUtil.makeVarGridData(300, 0, 1, 1));
		{
			String[] cols = {
					Messages.getString("SchemaEditorView.53"), Messages.getString("SchemaEditorView.22"), Messages.getString("SchemaEditorView.23") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			int[] widths = { 40, 140, 140 };
			SwtUtil.setupTable(tblComplexType, cols, widths);
		}
		viewComplexType = new TableViewer(tblComplexType);
		viewComplexType
				.setContentProvider(new SchemaEditorViewProvider.ComplexTypeContentProvider());
		viewComplexType
				.setLabelProvider(new SchemaEditorViewProvider.ComplexTypeLabelProvider());
		viewComplexType
				.addSelectionChangedListener(new ComplexTypeSelectionChangeListener());
		{
			String[] properties = new String[] { "", "name", "extension" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			viewComplexType.setColumnProperties(properties);
			CellEditor[] editors = new CellEditor[] { null,
					new TextCellEditor(tblComplexType),
					new TextCellEditor(tblComplexType),

			};
			viewComplexType.setCellEditors(editors);
			viewComplexType
					.setCellModifier(new SchemaEditorViewProvider.ComplexTypeCellModifier(
							this, viewComplexType));
		}
		{
			Action[] actions = new Action[] { actionCtAdd, actionCtDel };
			int[] enableTypes = new int[] {
					TableViewerMenuListener.ENABLE_TYPE_ALWAYS,
					TableViewerMenuListener.ENABLE_TYPE_SELECTED };
			makeTableViewerContextMenu(null, viewComplexType, actions,
					enableTypes);
		}
		tblComplexType.addMouseMoveListener(new TableMouseMoveListener(this,
				viewComplexType));
	}

	/**
	 * 複合型詳細をセットアップする
	 * 
	 * @param container
	 *            コンテナ
	 */
	private void setupComplexTypeDetail(Composite container) {
		int detColNum = 7;
		container.setLayout(SwtUtil.makeGridLayout(detColNum));

		Composite compComplexDetail = new Composite(container, SWT.NULL);
		compComplexDetail.setLayoutData(SwtUtil.makeVarGridData(200, 200,
				detColNum));
		compComplexDetail.setLayout(new FillLayout());
		treeComplexDetail = new TreeViewer(compComplexDetail, SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER);
		treeComplexDetail
				.setContentProvider(new SchemaEditorViewProvider.ComplexTypeDetailContentProvider());
		treeComplexDetail
				.setLabelProvider(new SchemaEditorViewProvider.ComplexTypeDetailLabelProvider(
						true));

		treeComplexDetail
				.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						loadAttributeElementDetailToGui(currSchema);
					}
				});
		treeComplexDetail.getTree().addMouseMoveListener(
				new TreeMouseMoveListener(this, treeComplexDetail));
		makeTreeViewerContextMenu(viewComplexType, treeComplexDetail);

		new Label(container, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.139")); //$NON-NLS-1$
		lblDetailName = new Label(container, SWT.NULL);
		lblDetailName.setText("  "); //$NON-NLS-1$
		txtDetailName = new Text(container, SWT.BORDER);
		txtDetailName.setLayoutData(SwtUtil.makeHorVarGridData(100, -1,
				detColNum - 2));
		txtDetailName.addFocusListener(textFocusListener);
		txtDetailName.addModifyListener(textModifyListener);

		new Label(container, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.140")); //$NON-NLS-1$
		lblDetailType = new Label(container, SWT.NULL);
		lblDetailType.setText("  "); //$NON-NLS-1$
		cmbDetailType = new Combo(container, SWT.DROP_DOWN);
		cmbDetailType.setLayoutData(SwtUtil.makeHorVarGridData(100, -1,
				detColNum - 2));
		cmbDetailType.addFocusListener(textFocusListener);
		cmbDetailType.addModifyListener(textModifyListener);
		cmbDetailType.addSelectionListener(comboSelectionListener);

		new Label(container, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.154")); //$NON-NLS-1$
		lblMinOccurs = new Label(container, SWT.NULL);
		lblMinOccurs.setText("  "); //$NON-NLS-1$
		txtMinOccurs = new Text(container, SWT.BORDER);
		txtMinOccurs.setLayoutData(SwtUtil.makeHorVarGridData(30, -1, 0));
		txtMinOccurs.addFocusListener(textFocusListener);
		txtMinOccurs.addModifyListener(textModifyListener);
		new Label(container, SWT.NULL).setText(Messages
				.getString("SchemaEditorView.155")); //$NON-NLS-1$
		lblMaxOccurs = new Label(container, SWT.NULL);
		lblMaxOccurs.setText("  "); //$NON-NLS-1$
		cmbMaxOccurs = new Combo(container, SWT.BORDER);
		cmbMaxOccurs.setLayoutData(SwtUtil.makeHorVarGridData(30, -1, 0));
		cmbMaxOccurs.addFocusListener(textFocusListener);
		cmbMaxOccurs.addModifyListener(textModifyListener);
		cmbMaxOccurs.add("unbounded"); //$NON-NLS-1$
		btnIndexed = new Button(container, SWT.CHECK);
		btnIndexed.setLayoutData(SwtUtil.makeHorVarGridData(100, -1, 0));
		btnIndexed.setText(Messages.getString("SchemaEditorView.156")); //$NON-NLS-1$
		btnIndexed.addSelectionListener(buttonSelectionListener);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		tabFolder.setFocus();
	}

	/**
	 * CSV形式ファイルをオープンする
	 * 
	 * @param file
	 *            ファイル名
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean openFile(String file) {
		boolean rc = false;

		closeFile(true);
		if (file.length() > 0) {
			currSchema = access.loadSchemaFromCsv(file, prop.getCsvCharSet(),
					MispSchemaAccess.DFS_DFT_SEPARATOR,
					MispSchemaAccess.DFS_DFT_PARENTHESIS,
					MispSchemaAccess.DFS_DFT_COMMENT);
			if (currSchema == null) {
				if (initFlg) {
					MessageDialog
							.openError(
									this.getSite().getShell(),
									Messages.getString("SchemaEditorView.101"), Messages.getString("SchemaEditorView.94")); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		} else {
			currSchema = operation
					.makeNewSchemaFromTemplate(NEW_SCHEMA_TEMPLATE);
		}
		if (currSchema != null) {
			orgSchema = (MispSchema) currSchema.clone();
			operation.reset(currSchema);
			loadSchemaToGui(currSchema);
			currFile = file;
			parent.setEnabled(true);
			setWorkbenchWindowTitle(false);
			rc = true;
		}
		if (!rc)
			closeFile(false);
		return rc;
	}

	/**
	 * CSVXファイルをオープンする
	 * 
	 * @param csvxFile
	 *            ファイル名
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean openCsvxFile(String csvxFile) {
		CsvxEditorDialog dialog = new CsvxEditorDialog(this.getViewSite()
				.getShell());
		MispSchemaCx cx = null;

		if (csvxFile.length() > 0) {
			cx = cxAccess.loadSchemaCxFromXml(csvxFile, prop.getCsvCharSet());
			if (!cx.getSchema().equals(currSchema)) {
				if (MessageDialog
						.openQuestion(
								this.getSite().getShell(),
								Messages.getString("SchemaEditorView.151"), Messages.getString("SchemaEditorView.152"))) //$NON-NLS-1$ //$NON-NLS-2$
				{
					cx.setSchemaSync(currSchema);
				}
			}
		} else
			cx = new MispSchemaCx(currSchema);

		if (cx != null) {
			dialog.setMispSchemaCx(cx);
			int rc = dialog.open();
			if (rc == IDialogConstants.OK_ID) {
				if (csvxFile.length() <= 0) {
					String file = chooseFile(MispSchemaCxAccess.DFT_FILE_EXT);
					if (file != null)
						csvxFile = file;
				}
				if (csvxFile.length() > 0) {
					if (cxAccess.saveSchemaCxToXml(cx, csvxFile, prop
							.getCxCharSet(), prop.getCxEncoding())) {
						return true;
					} else {
						log.error("File save error"); //$NON-NLS-1$
						MessageDialog.openError(this.getSite().getShell(),
								Messages.getString("SchemaEditorView.43"), //$NON-NLS-1$
								Messages.getString("SchemaEditorView.44")); //$NON-NLS-1$			}
					}
				}
			}
		}
		return false;
	}

	/**
	 * Redo/Undoを行う
	 * 
	 * @param flgRedo
	 *            redoフラグ
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean editUndoRedo(boolean flgRedo) {
		MispSchema schema = operation.editUndoRedo(flgRedo);
		if (schema != null) {
			currSchema = schema;
			loadSchemaToGui(currSchema);
		}
		return false;
	}

	/**
	 * Redo/Undoを行うことできるかどうかを返すフラグ
	 * 
	 * @param flgRedo
	 *            redoフラグ
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean isCanEditUndoRedo(boolean flgRedo) {
		return operation.isCanEditUndoRedo(flgRedo);
	}

	/**
	 * Redo/Undoを行うオペレーションを返す
	 * 
	 * @param flgRedo
	 *            redoフラグ
	 * @return 成功した場合オペレーション名, それ以外は空白を返す
	 */
	public String getEditUndoRedoOpName(boolean flgRedo) {
		return operation.getEditUndoRedoOpName(flgRedo);
	}

	/**
	 * ファイルをクローズする
	 * 
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean closeFile() {
		return closeFile(true);
	}

	/**
	 * ファイルをクローズする
	 * 
	 * @param blSave
	 *            保存フラグ
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	private boolean closeFile(boolean blSave) {
		boolean rc = false;

		if (isNeedSave() && blSave) {
			if (MessageDialog.openQuestion(this.getSite().getShell(), Messages
					.getString("SchemaEditorView.41"), //$NON-NLS-1$
					Messages.getString("SchemaEditorView.42"))) //$NON-NLS-1$
			{
				if (!saveFile()) {
					return false;
				}

			}
		}
		currFile = null;
		currSchema = null;
		orgSchema = null;
		loading = 0;
		clearSchemaGui();
		parent.setEnabled(false);
		setWorkbenchWindowTitle(false);
		rc = true;

		return rc;
	}

	/**
	 * ファイルを保存する
	 * 
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean saveFile() {
		boolean rc = false;

		if (currFile.length() <= 0) {
			String file = chooseFile(MispSchemaAccess.DFS_DFT_FILE_EXT);
			if (file != null)
				currFile = file;
		}
		if (currFile.length() > 0) {
			if (access.saveSchemaToCsv(currSchema, currFile, prop
					.getCsvCharSet(), MispSchemaAccess.DFS_DFT_SEPARATOR,
					MispSchemaAccess.DFS_DFT_PARENTHESIS,
					MispSchemaAccess.DFS_DFT_COMMENT)) {
				orgSchema = (MispSchema) currSchema.clone();
				checkModified();
				refreshAll();
				rc = true;
				if (currSchema.hasAbsolutePathImport()) {
					MessageDialog
							.openInformation(
									this.getSite().getShell(),
									Messages.getString("SchemaEditorView.161"), Messages.getString("SchemaEditorView.162")); //$NON-NLS-1$ //$NON-NLS-2$
				}

			} else {
				log.error("File save error"); //$NON-NLS-1$
				MessageDialog.openError(this.getSite().getShell(), Messages
						.getString("SchemaEditorView.43"), //$NON-NLS-1$
						Messages.getString("SchemaEditorView.44")); //$NON-NLS-1$			}
			}
		}
		return rc;
	}

	/**
	 * ファイルがオープンされているかどうかを取得する
	 * 
	 * @return オープンされている場合True, それ以外はFalseを返す
	 */
	public boolean isOpened() {
		return currFile != null;
	}

	/**
	 * ファイルを保存する必要あるかどうかを取得する
	 * 
	 * @return 保存する必要がある場合True, それ以外はFalseを返す
	 */
	public boolean isNeedSave() {
		if (currFile != null) {
			return !currSchema.equals(orgSchema);
		}
		return false;
	}

	/**
	 * スキーマに変更があったかどうかをチェックする
	 * 
	 */
	private void checkModified() {
		boolean modified = false;
		if (currFile != null)
			modified = !currSchema.equals(orgSchema);
		setWorkbenchWindowTitle(modified);
	}

	/**
	 * ワークベンチウインドのタイトルを設定する
	 * 
	 * @param modified
	 *            変更ありフラグ
	 */
	private void setWorkbenchWindowTitle(boolean modified) {
		String title;
		if (currFile == null) {
			title = ""; //$NON-NLS-1$
		} else if (currFile.length() > 0) {
			title = currFile;
		} else {
			title = Messages.getString("SchemaEditorView.46"); //$NON-NLS-1$
		}
		if (modified)
			title += Messages.getString("SchemaEditorView.47"); //$NON-NLS-1$
		ApplicationWorkbenchWindowAdvisor.setWorkbenchWindowTitle(title);
	}

	/**
	 * XML形式スキーマをエキスポートす
	 * 
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean exportXsd() {
		boolean rc = false;

		String file = chooseFile(MispSchemaAccess.XSD_DFT_FILE_EXT);
		if (file != null) {
			if (access.saveSchemaToXml(currSchema, file, prop.getXsdCharSet(),
					prop.getXsdEncoding())) {
				rc = true;
			} else {
				log.error("File save as error"); //$NON-NLS-1$
				MessageDialog.openError(this.getSite().getShell(), Messages
						.getString("SchemaEditorView.80"), //$NON-NLS-1$
						Messages.getString("SchemaEditorView.81")); //$NON-NLS-1$
			}
		}
		return rc;
	}

	/**
	 * XML形式スキーマをエキスポートす
	 * 
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean importXsd(String xsdFile) {
		boolean rc = false;

		MispSchema importSchema = access.loadSchemaFromXml(xsdFile, prop
				.getCsvCharSet());
		if (importSchema == null) {
			MessageDialog
					.openError(
							this.getSite().getShell(),
							Messages.getString("SchemaEditorView.101"), Messages.getString("SchemaEditorView.164")); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_IMPORT_XSD,
					importSchema);
			refreshAll();
			rc = true;
		}
		return rc;
	}

	/**
	 * 名前をつけて CSV形式でスキーマを保存する
	 * 
	 * @return 成功した場合True, それ以外はFalseを返す
	 */
	public boolean saveAsFile() {
		boolean rc = false;

		String file = chooseFile(MispSchemaAccess.DFS_DFT_FILE_EXT);
		if (file != null) {
			if (access.saveSchemaToCsv(currSchema, file, prop.getCsvCharSet(),
					MispSchemaAccess.DFS_DFT_SEPARATOR,
					MispSchemaAccess.DFS_DFT_PARENTHESIS,
					MispSchemaAccess.DFS_DFT_COMMENT)) {
				currFile = file;
				setWorkbenchWindowTitle(false);
				orgSchema = (MispSchema) currSchema.clone();
				checkModified();
				refreshAll();
				rc = true;
				if (currSchema.hasAbsolutePathImport()) {
					MessageDialog
							.openInformation(
									this.getSite().getShell(),
									Messages.getString("SchemaEditorView.161"), Messages.getString("SchemaEditorView.162")); //$NON-NLS-1$ //$NON-NLS-2$
				}
			} else {
				log.error("File save as error"); //$NON-NLS-1$
				MessageDialog.openError(this.getSite().getShell(), Messages
						.getString("SchemaEditorView.43"), //$NON-NLS-1$
						Messages.getString("SchemaEditorView.44")); //$NON-NLS-1$			}
			}
		}
		return rc;
	}

	/**
	 * ファイルダイアログを表示して ファイルを選択する
	 * 
	 * @param ext
	 *            拡張子
	 * @return 選択した場合はファイル名 それ以外はNULLを返す
	 */
	public String chooseFile(String ext) {
		FileDialog openDialog = new FileDialog(this.getSite().getShell(),
				SWT.SAVE);
		openDialog.setFilterExtensions(new String[] { "*." + ext, "*.*" }); //$NON-NLS-1$ //$NON-NLS-2$
		while (true) {
			String file = openDialog.open();
			if (file != null) {
				if (!file.endsWith("." + ext)) //$NON-NLS-1$
					file = file + "." + ext; //$NON-NLS-1$
				File fl = new File(file);
				if (fl.exists()) {
					if (!MessageDialog
							.openQuestion(
									this.getSite().getShell(),
									Messages.getString("SchemaEditorView.87"), Messages.getString("SchemaEditorView.88"))) { //$NON-NLS-1$ //$NON-NLS-2$
						continue;
					}
				}
			}
			log.debug("Filename:" + file); //$NON-NLS-1$
			return file;
		}

	}

	/**
	 * GUI画面をクリアする
	 * 
	 */
	private void clearSchemaGui() {
		loadSchemaToGui(null);
	}

	/**
	 * スキーマをGUI画面にロードする
	 * 
	 * @param schema
	 *            スキーマ
	 */
	private void loadSchemaToGui(MispSchema schema) {
		loading++;
		if (schema != null)
			clearSchemaGui();
		loadItemToGui(schema, ITEM_ID, FLG_ALL, null);
		loadItemToGui(schema, ITEM_ANNOTATION, FLG_ALL, null);
		loadItemToGui(schema, ITEM_TNS, FLG_ALL, null);
		loadItemToGui(schema, ITEM_FT_NAME, FLG_ALL, null);
		loadItemToGui(schema, ITEM_FT_TYPE, FLG_ALL, null);

		viewNamespace.setInput(schema);
		if (viewNamespace.getTable().getItemCount() > 0)
			viewNamespace.getTable().setSelection(0);
		viewImport.setInput(schema);
		if (viewImport.getTable().getItemCount() > 0)
			viewImport.getTable().setSelection(0);
		viewComplexType.setInput(schema);
		if (viewComplexType.getTable().getItemCount() > 0)
			viewComplexType.getTable().setSelection(0);

		setTabItemValidate(schema);
		loadComplexTypeDetailToGui(schema);
		loading--;
	}

	/**
	 * 複合型詳細GUI画面をクリアする
	 * 
	 */
	private void clearComplexTypeDetailGui() {
		loadComplexTypeDetailToGui(null);
	}

	/**
	 * 複合型詳細をGUI画面にロードする
	 * 
	 * @param schema
	 *            スキーマ
	 * 
	 */
	private void loadComplexTypeDetailToGui(MispSchema schema) {
		loading++;

		MispSchemaComplexType ct = null;
		if (schema != null) {
			clearComplexTypeDetailGui();
			ct = getCurrComplexType(schema);
		}
		treeComplexDetail.setInput(ct);
		SwtUtil.expandToLevel(treeComplexDetail, null, DFT_TREE_LEVEL);
		loadAttributeElementDetailToGui(schema);

		loading--;
	}

	/**
	 * 複合型詳細GUI画面をクリアする
	 * 
	 */
	private void clearAttributeElementDetailToGui() {
		loadAttributeElementDetailToGui(null);
	}

	/**
	 * 属性要素詳細をGUI画面にロードする
	 * 
	 * @param schema
	 *            スキーマ
	 * 
	 */
	private void loadAttributeElementDetailToGui(MispSchema schema) {
		loading++;
		if (schema != null) {
			clearAttributeElementDetailToGui();
		}

		TreeObject treeObj = getCurrentAttributeTreeObj();
		Object currAttrEm = (treeObj == null) ? null : treeObj.getData();

		loadItemToGui(schema, ITEM_EM_ATTR_NAME, FLG_ALL, null);
		loadItemToGui(schema, ITEM_EM_ATTR_TYPE, FLG_ALL, null);
		loadItemToGui(schema, ITEM_EM_ATTR_MIN_OCCURS, FLG_ALL, null);
		loadItemToGui(schema, ITEM_EM_ATTR_MAX_OCCURS, FLG_ALL, null);
		loadItemToGui(schema, ITEM_EM_ATTR_INDEXED, FLG_ALL, null);

		boolean editable = (treeObj == null) ? false : treeObj.isEditable();
		txtDetailName.setEnabled(editable);
		cmbDetailType.setEnabled(editable);
		btnIndexed.setEnabled(editable);

		if (!(currAttrEm instanceof MispSchemaElement))
			editable = false;
		txtMinOccurs.setEnabled(editable);
		cmbMaxOccurs.setEnabled(editable);

		currTreeObj = treeObj;
		loading--;
	}

	final private static int ITEM_ID = 0;

	final private static int ITEM_TNS = 1;

	final private static int ITEM_FT_NAME = 2;

	final private static int ITEM_FT_TYPE = 3;

	final private static int ITEM_EM_ATTR_NAME = 4;

	final private static int ITEM_EM_ATTR_TYPE = 5;

	final private static int ITEM_EM_ATTR_MIN_OCCURS = 6;

	final private static int ITEM_EM_ATTR_MAX_OCCURS = 7;

	final private static int ITEM_EM_ATTR_INDEXED = 8;

	final private static int ITEM_ANNOTATION = 9;

	final private static int FLG_VALUE = 0x1;

	final private static int FLG_VALIDATE = 0x2;

	final private static int FLG_ALL = FLG_VALUE | FLG_VALIDATE;

	public void loadItemToGui(MispSchema schema, int item, int flg, Object param) {
		loading++;

		MispSchemaElement fem = null;
		if (schema != null)
			fem = schema.getFeatureElement();
		TreeObject treeObj = getCurrentAttributeTreeObj();
		Object currAttrEm = (treeObj == null) ? null : treeObj.getData();

		String validate;
		Image img;

		switch (item) {
		case ITEM_ID:
			if ((flg & FLG_VALUE) != 0) {
				txtID.setText((schema == null) ? "" : schema.getId()); //$NON-NLS-1$
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = (schema != null) ? schema.validateId(true) : ""; //$NON-NLS-1$
				img = (validate.length() > 0) ? invalidImg : validImg;
				;
				lblID.setImage(img);
				lblID.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$
				txtID.setToolTipText(Messages.getString("SchemaEditorView.55")); //$NON-NLS-1$		
			}
			break;
		case ITEM_ANNOTATION:
			if ((flg & FLG_VALUE) != 0) {
				txtAnnotation.setText((schema == null) ? "" : schema //$NON-NLS-1$
						.getAnnotation());
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = (schema != null) ? schema.validateAnnotation(true)
						: ""; //$NON-NLS-1$
				img = (validate.length() > 0) ? invalidImg : validImg;
				;
				txtAnnotation.setToolTipText(Messages
						.getString("SchemaEditorView.157")); //$NON-NLS-1$
			}
			break;
		case ITEM_TNS:
			if ((flg & FLG_VALUE) != 0) {
				txtTargetNamespace.setText((schema == null) ? "" : schema //$NON-NLS-1$
						.getTargetNamespace());
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = (schema != null) ? schema
						.validateTargetNamespace(true) : ""; //$NON-NLS-1$
				img = (validate.length() > 0) ? invalidImg : validImg;
				;
				lblTargetNamespace.setImage(img);
				lblTargetNamespace
						.setToolTipText((validate.length() > 0) ? validate
								: toolTipNoneText); //$NON-NLS-1$
				txtTargetNamespace.setToolTipText(Messages
						.getString("SchemaEditorView.56")); //$NON-NLS-1$
			}
			break;
		case ITEM_FT_NAME:
			if ((flg & FLG_VALUE) != 0) {
				txtFeatureTypeName.setText((schema == null) ? "" : schema //$NON-NLS-1$
						.getFeatureElement().getName());
			}
			if ((flg & FLG_VALIDATE) != 0) {

				validate = (fem != null) ? fem.validateName(true) : ""; //$NON-NLS-1$
				img = (validate.length() > 0) ? invalidImg : validImg;
				;
				lblFeatureTypeName.setImage(img);
				lblFeatureTypeName
						.setToolTipText((validate.length() > 0) ? validate
								: toolTipNoneText); //$NON-NLS-1$
				txtFeatureTypeName.setToolTipText(Messages
						.getString("SchemaEditorView.57")); //$NON-NLS-1$
			}
			break;
		case ITEM_FT_TYPE:
			if ((flg & FLG_VALUE) != 0) {
				cmbFeatureTypeType.setItems((schema == null) ? new String[0]
						: schema.getValidFeatureElementTypes());
				cmbFeatureTypeType.setText((schema == null) ? "" : schema //$NON-NLS-1$
						.getFeatureElement().getType());
			}
			if ((flg & FLG_VALIDATE) != 0) {

				validate = (fem != null) ? fem.validateType(true) : ""; //$NON-NLS-1$
				img = (validate.length() > 0) ? invalidImg : validImg;
				;
				lblFeatureTypeType.setImage(img);
				lblFeatureTypeType
						.setToolTipText((validate.length() > 0) ? validate
								: toolTipNoneText); //$NON-NLS-1$
				cmbFeatureTypeType.setToolTipText(Messages
						.getString("SchemaEditorView.58")); //$NON-NLS-1$
			}
			break;
		case ITEM_EM_ATTR_NAME:
			if ((flg & FLG_VALUE) != 0) {
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					txtDetailName.setText(attr.getName());
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					txtDetailName.setText(et.getName());
				} else {
					txtDetailName.setText(""); //$NON-NLS-1$
				}
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = ""; //$NON-NLS-1$
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					validate = attr.validateName(true);
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					validate = et.validateName(true);
				}
				img = (validate.length() > 0) ? invalidImg : validImg;
				lblDetailName.setImage(img);
				lblDetailName.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$

			}
			break;

		case ITEM_EM_ATTR_INDEXED:
			if ((flg & FLG_VALUE) != 0) {
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					btnIndexed.setSelection(attr.isIndexedFlg());
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					btnIndexed.setSelection(et.isIndexedFlg());
				} else {
					btnIndexed.setSelection(false);
				}
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = ""; //$NON-NLS-1$
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					validate = attr.validateName(true);
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					validate = et.validateName(true);
				}
				img = (validate.length() > 0) ? invalidImg : validImg;
				lblDetailName.setImage(img);
				lblDetailName.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$

			}
			break;

		case ITEM_EM_ATTR_MIN_OCCURS:
			if ((flg & FLG_VALUE) != 0) {
				if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					txtMinOccurs.setText(et.getMinOccurs());
				} else {
					txtMinOccurs.setText(""); //$NON-NLS-1$
				}
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = ""; //$NON-NLS-1$
				if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					validate = et.validateMinOccurs(true);
				}
				img = (validate.length() > 0) ? invalidImg : validImg;
				lblMinOccurs.setImage(img);
				lblMinOccurs.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$

			}
			break;

		case ITEM_EM_ATTR_MAX_OCCURS:
			if ((flg & FLG_VALUE) != 0) {
				if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					cmbMaxOccurs.setText(et.getMaxOccurs());
				} else {
					cmbMaxOccurs.setText(""); //$NON-NLS-1$
				}
			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = ""; //$NON-NLS-1$
				if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					validate = et.validateMaxOccurs(true);
				}
				img = (validate.length() > 0) ? invalidImg : validImg;
				lblMaxOccurs.setImage(img);
				lblMaxOccurs.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$
			}
			break;
		case ITEM_EM_ATTR_TYPE:
			if ((flg & FLG_VALUE) != 0) {
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					cmbDetailType.setItems((schema == null) ? new String[0]
							: schema.getValidAttributeTypes());
					cmbDetailType.setText((schema == null) ? "" : attr //$NON-NLS-1$
							.getType());
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					cmbDetailType.setItems((schema == null) ? new String[0]
							: schema.getValidElementTypes(et));
					cmbDetailType.setText((schema == null) ? "" : et //$NON-NLS-1$
							.getType());
				} else {
					cmbDetailType.setText(""); //$NON-NLS-1$
					cmbDetailType.removeAll();
				}

			}
			if ((flg & FLG_VALIDATE) != 0) {
				validate = ""; //$NON-NLS-1$
				if (currAttrEm instanceof MispSchemaAttribute) {
					MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
					validate = attr.validateType(true);
				} else if (currAttrEm instanceof MispSchemaElement) {
					MispSchemaElement et = (MispSchemaElement) currAttrEm;
					validate = et.validateType(true);
				}
				img = (validate.length() > 0) ? invalidImg : validImg;
				lblDetailType.setImage(img);
				lblDetailType.setToolTipText((validate.length() > 0) ? validate
						: toolTipNoneText); //$NON-NLS-1$

			}
			break;
		}

		loading--;
	}

	/**
	 * テーブルビューにある複合型を更新する
	 * 
	 * @param viewer
	 *            更新するテーブルビュー
	 * @param treeObj
	 *            更新するエレメント
	 */
	public void updateTreeParent(TreeViewer viewer, TreeObject treeObj) {
		TreeObject treeParent = (treeObj == null) ? null : treeObj.getParent();
		updateTreeObject(viewer, treeParent);
	}

	/**
	 * テーブルビューにある複合型を更新する
	 * 
	 * @param viewer
	 *            更新するテーブルビュー
	 * @param treeObj
	 *            更新するエレメント
	 */
	public void updateTreeObject(TreeViewer viewer, TreeObject treeObj) {

		if (treeObj == null) {
			viewer.refresh();
			SwtUtil.expandToLevel(viewer, null, DFT_TREE_LEVEL);
		} else {
			viewer.refresh(treeObj);
			SwtUtil.expandToLevel(viewer, treeObj, DFT_TREE_LEVEL);
		}
		setTabItemValidate(currSchema);
		viewComplexType.refresh();
		checkModified();

	}

	/**
	 * テーブルビューを更新する
	 * 
	 * @param viewer
	 *            更新するテーブルビュー
	 * @param element
	 *            更新するエレメント
	 */
	public void updateTableViewer(TableViewer viewer, Object element) {
		if (element == null)
			viewer.refresh();
		else {
			viewer.refresh(element);
		}

		if (viewer == viewNamespace) {
			viewImport.refresh();
			viewComplexType.refresh();
		}

		if (viewer == viewImport) {
			viewComplexType.refresh();
		}

		if (viewer == viewComplexType) {
			loadItemToGui(currSchema, ITEM_FT_TYPE, FLG_ALL, null);
			loadComplexTypeDetailToGui(currSchema);
			loadAttributeElementDetailToGui(currSchema);

		}

		setTabItemValidate(currSchema);
		checkModified();
	}

	/**
	 * スキーマのテーブル入力の有効性をチェックする
	 * 
	 * @param schema
	 *            スキーマ
	 */
	private void setTabItemValidate(MispSchema schema) {
		String validate;
		Image img;

		validate = (schema != null) ? schema.validateNamespaceList(true) : ""; //$NON-NLS-1$
		img = (validate.length() > 0) ? invalidImg : validImg;
		;
		tabNamespace.setImage(img);
		tabNamespace.setToolTipText((validate.length() > 0) ? validate
				: Messages.getString("SchemaEditorView.59")); //$NON-NLS-1$

		validate = (schema != null) ? schema.validateImportList(true) : ""; //$NON-NLS-1$
		img = (validate.length() > 0) ? invalidImg : validImg;
		;
		tabImport.setImage(img);
		tabImport.setToolTipText((validate.length() > 0) ? validate : Messages
				.getString("SchemaEditorView.60")); //$NON-NLS-1$

		validate = (schema != null) ? schema.validateComplexTypeList(true) : ""; //$NON-NLS-1$
		img = (validate.length() > 0) ? invalidImg : validImg;
		;
		tabComplexType.setImage(img);
		tabComplexType.setToolTipText((validate.length() > 0) ? validate
				: Messages.getString("SchemaEditorView.61")); //$NON-NLS-1$
	}

	private static int UPDATE_MODE_SELECT = 0;
	private static int UPDATE_MODE_MODIFY_TEXT = 1;
	private static int UPDATE_MODE_LOST_FOCUS = 2;

	private void doEvent(Object widget, int mode) {
		int op = SchemaEditorViewOperation.OP_NONE;
		TreeObject treeObj = currTreeObj;
		Object currAttrEm = (treeObj == null) ? null : treeObj.getData();

		if(mode == UPDATE_MODE_MODIFY_TEXT) return;
		//if(mode == UPDATE_MODE_LOST_FOCUS) return;
		
		if (widget == txtID) {
			op = operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_ID, txtID.getText());
			loadItemToGui(currSchema, ITEM_ID, FLG_VALIDATE, null);
		} else if (widget == txtAnnotation) {
			op = operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_ANNOTATION,
					txtAnnotation.getText());
			loadItemToGui(currSchema, ITEM_ANNOTATION, FLG_VALIDATE, null);
		} else if (widget == txtTargetNamespace) {
			op = operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_TNS, txtTargetNamespace
							.getText());
			loadItemToGui(currSchema, ITEM_TNS, FLG_VALIDATE, null);
		} else if (widget == txtFeatureTypeName) {
			op = operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_FT_NAME,
					txtFeatureTypeName.getText());
			loadItemToGui(currSchema, ITEM_FT_NAME, FLG_VALIDATE, null);
		} else if (widget == cmbFeatureTypeType) {
			op = operation.opSchema(currSchema,
					SchemaEditorViewOperation.OP_SCHEMA_FT_TYPE,
					cmbFeatureTypeType.getText());
			loadItemToGui(currSchema, ITEM_FT_TYPE, FLG_VALIDATE, null);
			MispSchemaComplexType ct = currSchema.getFeatureElement()
					.findComplexType();
			if (ct != null) {
				if ((ct.getIndex() >= 0)
						&& (ct.getIndex() < viewComplexType.getTable()
								.getItemCount())) {
					viewComplexType.refresh();
					viewComplexType.getTable().setSelection(ct.getIndex());
					loadComplexTypeDetailToGui(currSchema);
				}
			}
		} else if (currAttrEm == null) {
		} else if (widget == cmbDetailType) {
			String type = cmbDetailType.getText();
			if (currAttrEm instanceof MispSchemaAttribute) {
				op = operation.opAttribute(currSchema,
						(MispSchemaAttribute) currAttrEm,
						SchemaEditorViewOperation.OP_ATTR_TYPE, type);
			} else if (currAttrEm instanceof MispSchemaElement) {
				op = operation.opElement(currSchema,
						(MispSchemaElement) currAttrEm,
						SchemaEditorViewOperation.OP_EM_TYPE, type);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				updateTreeObject(treeComplexDetail, treeObj);
				loadItemToGui(currSchema, ITEM_EM_ATTR_TYPE, FLG_VALIDATE, null);

			}
		} else if (widget == txtMinOccurs) {
			String minOccurs = txtMinOccurs.getText();
			if (currAttrEm instanceof MispSchemaElement) {
				op = operation.opElement(currSchema,
						(MispSchemaElement) currAttrEm,
						SchemaEditorViewOperation.OP_EM_MIN_OCCURS, minOccurs);
				loadItemToGui(currSchema, ITEM_EM_ATTR_MIN_OCCURS,
						FLG_VALIDATE, null);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				updateTreeObject(treeComplexDetail, treeObj);
				loadItemToGui(currSchema, ITEM_EM_ATTR_MIN_OCCURS,
						FLG_VALIDATE, null);
			}
		} else if (widget == cmbMaxOccurs) {
			String maxOccurs = cmbMaxOccurs.getText();
			if (currAttrEm instanceof MispSchemaElement) {
				op = operation.opElement(currSchema,
						(MispSchemaElement) currAttrEm,
						SchemaEditorViewOperation.OP_EM_MAX_OCCURS, maxOccurs);
				loadItemToGui(currSchema, ITEM_EM_ATTR_MAX_OCCURS,
						FLG_VALIDATE, null);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				updateTreeObject(treeComplexDetail, treeObj);
				loadItemToGui(currSchema, ITEM_EM_ATTR_MAX_OCCURS,
						FLG_VALIDATE, null);
			}
		} else if (widget == btnIndexed) {
			Boolean checked = new Boolean(btnIndexed.getSelection());
			if (currAttrEm instanceof MispSchemaAttribute) {
				op = operation.opAttribute(currSchema,
						(MispSchemaAttribute) currAttrEm,
						SchemaEditorViewOperation.OP_ATTR_INDEXED, checked);
			} else if (currAttrEm instanceof MispSchemaElement) {
				op = operation.opElement(currSchema,
						(MispSchemaElement) currAttrEm,
						SchemaEditorViewOperation.OP_EM_INDEXED, checked);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				updateTreeObject(treeComplexDetail, treeObj);
				loadItemToGui(currSchema, ITEM_EM_ATTR_INDEXED, FLG_VALIDATE,
						null);
			}
		} else if (widget == txtDetailName) {
			String name = txtDetailName.getText();

			if (currAttrEm instanceof MispSchemaAttribute) {
				MispSchemaAttribute attr = (MispSchemaAttribute) currAttrEm;
				Object[] attrs = attr.getParentCt().findAttributes(name);
				if ((attrs.length > 1)
						|| ((attrs.length == 1) && (attrs[0] != attr))) {
					MessageDialog.openError(getSite().getShell(), Messages
							.getString("SchemaEditorView.141"), //$NON-NLS-1$
							Messages.getString("SchemaEditorView.142")); //$NON-NLS-1$
					loadItemToGui(currSchema, ITEM_EM_ATTR_NAME, FLG_ALL, null);
				} else {
					op = operation.opAttribute(currSchema,
							(MispSchemaAttribute) currAttrEm,
							SchemaEditorViewOperation.OP_ATTR_NAME, name);
				}

			} else if (currAttrEm instanceof MispSchemaElement) {
				MispSchemaElement em = (MispSchemaElement) currAttrEm;
				Object[] ems = em.getParentCt().findElements(name);
				if ((ems.length > 1) || ((ems.length == 1) && (ems[0] != em))) {

					MessageDialog.openError(getSite().getShell(), Messages
							.getString("SchemaEditorView.137"), //$NON-NLS-1$
							Messages.getString("SchemaEditorView.138")); //$NON-NLS-1$
					loadItemToGui(currSchema, ITEM_EM_ATTR_NAME, FLG_ALL, null);
				} else {
					op = operation.opElement(currSchema,
							(MispSchemaElement) currAttrEm,
							SchemaEditorViewOperation.OP_EM_NAME, name);
				}
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				updateTreeObject(treeComplexDetail, treeObj);
				loadItemToGui(currSchema, ITEM_EM_ATTR_NAME, FLG_VALIDATE, null);
			}
		}

		if (op != SchemaEditorViewOperation.OP_NONE) {
			checkModified();
		}
	}

	/**
	 * 複合型変更リスナー
	 * 
	 * @author shumi
	 * 
	 */
	class ComplexTypeSelectionChangeListener implements
			ISelectionChangedListener {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
		 */
		public void selectionChanged(SelectionChangedEvent event) {
			loadComplexTypeDetailToGui(currSchema);
		}
	}

	/**
	 * ツリービューメニューリスナー
	 */
	private class TreeViewerMenuListener implements IMenuListener {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
		 */
		public void menuAboutToShow(IMenuManager manager) {
			ISelection selection = treeComplexDetail.getSelection();
			Object item = ((IStructuredSelection) selection).getFirstElement();

			if (item instanceof TreeObject) {
				TreeObject treeObj = (TreeObject) item;
				Object data = treeObj.getData();

				if (treeObj.isEditable()) {
					boolean canUp = false;
					boolean canDown = false;
					boolean isComplexType = (data instanceof MispSchemaComplexType);
					boolean isElement = (data instanceof MispSchemaElement);
					if (isElement) {
						MispSchemaElement em = (MispSchemaElement) data;
						canUp = em.getIndex() > 0;
						canDown = em.getIndex() < (em.getParentCt()
								.getElementSize() - 1);
					}
					boolean isAttr = (data instanceof MispSchemaAttribute);
					if (isAttr) {
						MispSchemaAttribute attr = (MispSchemaAttribute) data;
						canUp = attr.getIndex() > 0;
						canDown = attr.getIndex() < (attr.getParentCt()
								.getAttributeSize() - 1);
					}

					if (isComplexType | isElement) {
						manager.add(new Separator());
						actionEmAdd.setEnabled(isComplexType | isElement);
						manager.add(actionEmAdd);
						actionEmDel.setEnabled(isElement);
						manager.add(actionEmDel);
					}

					if (isComplexType | isAttr) {
						manager.add(new Separator());

						actionAttrAdd.setEnabled(isComplexType | isAttr);
						manager.add(actionAttrAdd);
						actionAttrDel.setEnabled(isAttr);
						manager.add(actionAttrDel);
					}

					if (isElement | isAttr) {
						manager.add(new Separator());

						actionMoveUp.setEnabled(canUp);
						manager.add(actionMoveUp);
						actionMoveDown.setEnabled(canDown);
						manager.add(actionMoveDown);
					}

					if (isElement) {
						manager.add(new Separator());

						actionCtEmAdd.setEnabled(isElement);
						manager.add(actionCtEmAdd);
						actionCtAttrAdd.setEnabled(isElement);
						manager.add(actionCtAttrAdd);
					}
				}
			}
			manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		}
	}

	/**
	 * テキストボックス変更リスナー
	 * 
	 * @author shumi
	 * 
	 */
	class ButtonSelectionListener implements SelectionListener {
		public void widgetSelected(SelectionEvent e) {
			if ((currSchema != null) && (loading <= 0)) {
				if (e.widget == btnIndexed)
					doEvent(e.widget, UPDATE_MODE_SELECT);
			}

		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
	}

	/**
	 * テキストボックス変更リスナー
	 * 
	 * @author shumi
	 * 
	 */
	class TextModifyListener implements ModifyListener {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
		 */
		public void modifyText(ModifyEvent e) {
			if ((currSchema != null) && (loading <= 0)) {
				doEvent(e.widget, UPDATE_MODE_MODIFY_TEXT);
			}
		}
	}

	/**
	 * テキストボックス変更リスナー
	 * 
	 * @author shumi
	 * 
	 */
	class ComboSelectionListener implements SelectionListener {

		public void widgetSelected(SelectionEvent e) {
			if ((currSchema != null) && (loading <= 0)) {
				doEvent(e.widget, UPDATE_MODE_SELECT);
			}
		}

		public void widgetDefaultSelected(SelectionEvent e) {

		}

	}

	/**
	 * テキストボックス変更リスナー
	 * 
	 * @author shumi
	 * 
	 */
	class TextFocusListener implements FocusListener {

		/**
		 * 
		 * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
		 */
		public void focusGained(FocusEvent e) {
		}

		/**
		 * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
		 */
		public void focusLost(FocusEvent e) {
			if ((currSchema != null) && (loading <= 0)) {
				doEvent(e.widget, UPDATE_MODE_LOST_FOCUS);
			}
		}
	}

	/**
	 * operationを取得する
	 * 
	 * @return Returns the operation.
	 */
	public SchemaEditorViewOperation getOperation() {
		return operation;
	}

	/**
	 * currSchemaを取得する
	 * 
	 * @return Returns the currSchema.
	 */
	public MispSchema getCurrSchema() {
		return currSchema;
	}

	public void refreshAll() {
		loadSchemaToGui(currSchema);
	}

	/**
	 * accessを取得する
	 * 
	 * @return Returns the access.
	 */
	public MispSchemaAccess getAccess() {
		return access;
	}
}