package daruma.client.schemaeditor.ui.views;

import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.Path;

import daruma.bus.processor.csv.MispSchemaBuiltIn;
import daruma.client.schemaeditor.SchemaEditorPlugin;

/**
 * スキーマエディタビューのプロパティクラス
 * 
 * @author shumi
 * 
 */
public class SchemaEditorViewProp {
	/**
	 * ロガー
	 */
	static private Log log = LogFactory.getLog(SchemaEditorViewProp.class);

	/**
	 * ビルトイン要素データ型のプロパティ名
	 */
	private final static String PROPS_BUILTIN_ELEMENT_DATATYPE = "schemaeditor.element.builtindatatype"; //$NON-NLS-1$

	/**
	 * ビルトイン属性データ型のプロパティ名
	 */
	private final static String PROPS_BUILTIN_ATTRIBUTE_DATATYPE = "schemaeditor.attribute.builtindatatype"; //$NON-NLS-1$

	/**
	 * CSV形式ファイルのキャラクタセットのプロパティ名
	 */
	private final static String PROPS_CSV_CHARSET = "schemaeditor.csv.charset"; //$NON-NLS-1$

	/**
	 * XSD形式ファイルのキャラクタセットのプロパティ名
	 */
	private final static String PROPS_XSD_CHARSET = "schemaeditor.xsd.charset"; //$NON-NLS-1$

	/**
	 * XSD形式ファイルのXMLエンコードのプロパティ名
	 */
	private final static String PROPS_XSD_ENCODING = "schemaeditor.xsd.encoding"; //$NON-NLS-1$


	/**
	 * XSD形式ファイルのキャラクタセットのプロパティ名
	 */
	private final static String PROPS_CX_CHARSET = "schemaeditor.cx.charset"; //$NON-NLS-1$

	/**
	 * XSD形式ファイルのXMLエンコードのプロパティ名
	 */
	private final static String PROPS_CX_ENCODING = "schemaeditor.cx.encoding"; //$NON-NLS-1$

	
	/**
	 * 新規属性のデフォルト型のプロパティ名
	 */
	private final static String PROPS_DFT_ATTR_TYPE = "schemaeditor.attribute.defaulttype"; //$NON-NLS-1$

	/**
	 * 新規要素のデフォルト型のプロパティ名
	 */
	private final static String PROPS_DFT_ELEMENT_TYPE = "schemaeditor.element.defaulttype"; //$NON-NLS-1$

	/**
	 * 新規追加時　前に追加するフラグのプロパティ名
	 */
	private final static String PROPS_ADDNEW_BEFORE = "schemaeditor.addnew.before";
	
	/**
	 * CSVスキーマバージョン出力フラグのプロパティ名
	 */
	private final static String PROPS_CSV_OUTPUTVERSION = "schemaeditor.csv.outputversion";
	
	/**
	 * CSVキャラクタセット
	 */
	private String csvCharSet = "Windows-31J"; //$NON-NLS-1$

	/**
	 * XMLキャラクタセット
	 */
	private String xsdCharSet = "UTF-8"; //$NON-NLS-1$

	/**
	 * XMLエンコード
	 */
	private String xsdEncoding = "UTF-8"; //$NON-NLS-1$
	
	/**
	 * CXキャラクタセット
	 */
	private String cxCharSet = "UTF-8"; //$NON-NLS-1$

	/**
	 * CXエンコード
	 */
	private String cxEncoding = "UTF-8"; //$NON-NLS-1$

	/**
	 * 新規属性のデフォルト型のプロパティ名
	 */
	private String dftAttrType = "string"; //$NON-NLS-1$

	/**
	 * 新規要素のデフォルト型のプロパティ名
	 */
	private String dftElementType = "string"; //$NON-NLS-1$
	
	/**
	 * 新規追加時　前に追加するフラグ
	 */
	private boolean addNewBefore = true;
	
	/**
	 * CSVスキーマバージョン出力フラグ
	 */
	private boolean csvOutputVersion = true;
	
	private SchemaEditorViewProp() {
	}

	/**
	 * プロパテｌをロードする
	 * 
	 */
	public static SchemaEditorViewProp loadProp(String path) {
		SchemaEditorViewProp prop = new SchemaEditorViewProp();
		try {
			URL url = SchemaEditorPlugin.getDefault().find(new Path(path));

			InputStream fis = url.openStream();
			Properties props = new Properties();
			props.load(fis);

			String builtinElementDataType = props.getProperty(
					PROPS_BUILTIN_ELEMENT_DATATYPE,
					MispSchemaBuiltIn.dftBuiltinElementType);

			String builtinAttributeDataType = props.getProperty(
					PROPS_BUILTIN_ATTRIBUTE_DATATYPE,
					MispSchemaBuiltIn.dftBuiltinAttributeType);

			prop.csvCharSet = props.getProperty(PROPS_CSV_CHARSET,
					prop.csvCharSet);
			prop.xsdCharSet = props.getProperty(PROPS_XSD_CHARSET,
					prop.xsdCharSet);
			prop.xsdEncoding = props.getProperty(PROPS_XSD_ENCODING,
					prop.xsdEncoding);
			
			prop.cxCharSet = props.getProperty(PROPS_CX_CHARSET,
					prop.cxCharSet);
			prop.cxEncoding = props.getProperty(PROPS_CX_ENCODING,
					prop.cxEncoding);
			
			prop.dftAttrType = props.getProperty(PROPS_DFT_ATTR_TYPE,
					prop.dftAttrType);
			prop.dftElementType = props.getProperty(PROPS_DFT_ELEMENT_TYPE,
					prop.dftElementType);
			prop.addNewBefore = props.getProperty(PROPS_ADDNEW_BEFORE, "true").toLowerCase().equals("true");
			prop.csvOutputVersion = props.getProperty(PROPS_CSV_OUTPUTVERSION, "true").toLowerCase().equals("true");
			MispSchemaBuiltIn.setBuiltinElementType(builtinElementDataType);
			MispSchemaBuiltIn.setBuiltinAttributeType(builtinAttributeDataType);

		} catch (Exception ex) {
			log.error("Fail to open property file:" + path); //$NON-NLS-1$
		}
		return prop;
	}

	/**
	 * csvCharSetを取得する
	 * 
	 * @return Returns the csvCharSet.
	 */
	public String getCsvCharSet() {
		return csvCharSet;
	}

	/**
	 * dftAttrTypeを取得する
	 * 
	 * @return Returns the dftAttrType.
	 */
	public String getDftAttrType() {
		return dftAttrType;
	}

	/**
	 * dftElementTypeを取得する
	 * 
	 * @return Returns the dftElementType.
	 */
	public String getDftElementType() {
		return dftElementType;
	}

	/**
	 * xsdCharSetを取得する
	 * 
	 * @return Returns the xsdCharSet.
	 */
	public String getXsdCharSet() {
		return xsdCharSet;
	}

	
	
	
	/**
	 * cxCharSetを取得する
	 * @return Returns the cxCharSet.
	 */
	public String getCxCharSet() {
		return cxCharSet;
	}

	/**
	 * cxEncodingを取得する
	 * @return Returns the cxEncoding.
	 */
	public String getCxEncoding() {
		return cxEncoding;
	}

	/**
	 * xsdEncodingを取得する
	 * 
	 * @return Returns the xsdEncoding.
	 */
	public String getXsdEncoding() {
		return xsdEncoding;
	}

	/**
	 * addNewBeforeを取得する
	 * @return Returns the addNewBefore.
	 */
	public boolean isAddNewBefore() {
		return addNewBefore;
	}

	/**
	 * csvOutputVersionを取得する
	 * @return Returns the csvOutputVersion.
	 */
	public boolean isCsvOutputVersion() {
		return csvOutputVersion;
	}
	
	
}
