package daruma.client.schemaeditor.ui.views;

import java.util.List;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Item;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;
import daruma.bus.processor.csv.MispSchema.MispSchemaImport;
import daruma.bus.processor.csv.MispSchema.MispSchemaNamespace;
import daruma.client.schemaeditor.Messages;

/**
 * スキーマエディタビューの表示内容プロバイダ
 * 
 * @author shumi
 * 
 */
public class SchemaEditorViewProvider {

	/**
	 * 属性アイコン
	 */
	public static Image attributeImg = SwtUtil.loadImg("icons/attribute.gif"); //$NON-NLS-1$

	/**
	 * 属性アイコン
	 */
	public static Image attributeImg2 = SwtUtil.loadImg("icons/attribute2.gif"); //$NON-NLS-1$

	/**
	 * 要素アイコン
	 */
	public static Image elementImg = SwtUtil.loadImg("icons/element.gif"); //$NON-NLS-1$

	/**
	 * 要素アイコン
	 */
	public static Image elementImg2 = SwtUtil.loadImg("icons/element2.gif"); //$NON-NLS-1$

	/**
	 * ルートアイコン
	 */
	public static Image rootImg = PlatformUI.getWorkbench().getSharedImages()
			.getImage(ISharedImages.IMG_DEF_VIEW);

	/**
	 * 名前空間セルModifier
	 * 
	 * @author shumi
	 * 
	 */
	public static class NamespaceCellModifier implements ICellModifier {
		/**
		 * ビュー
		 */
		SchemaEditorView view;

		/**
		 * テーブルビュー
		 */
		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param view
		 *            ビュー
		 * @param viewer
		 *            テーブルビュー
		 */
		public NamespaceCellModifier(SchemaEditorView view, TableViewer viewer) {
			this.view = view;
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			return tblObj.isEditable() && (property.length() > 0);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			TableObject tblObj = (TableObject) element;
			MispSchemaNamespace ns = (MispSchemaNamespace) tblObj.getData();
			if (property.equals("prefix")) //$NON-NLS-1$
				value = ns.getPrefix();
			if (property.equals("uri")) //$NON-NLS-1$
				value = ns.getUri();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			int op = SchemaEditorViewOperation.OP_NONE;
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			MispSchemaNamespace ns = (MispSchemaNamespace) tblObj.getData();
			if (property.equals("prefix")) //$NON-NLS-1$
			{
				op = view.getOperation().opNamespace(view.getCurrSchema(), ns,
						SchemaEditorViewOperation.OP_NS_PREFIX, value);
			}
			if (property.equals("uri")) //$NON-NLS-1$
			{
				op = view.getOperation().opNamespace(view.getCurrSchema(), ns,
						SchemaEditorViewOperation.OP_NS_URI, value);
			}
			if (op != SchemaEditorViewOperation.OP_NONE) {
				if (op == SchemaEditorViewOperation.OP_NS_PREFIX) {
					view.refreshAll();
				} else {
					view.updateTableViewer(viewer, element);
				}
			}
		}
	}

	/**
	 * 名前空間コンテントプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class NamespaceContentProvider implements
			IStructuredContentProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if ((inputElement != null) && (inputElement instanceof MispSchema)) {
				MispSchema schema = (MispSchema) inputElement;
				return TableObject.toArray(schema.getNamespaceArray());
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}
	}

	/**
	 * 名前空間ラベルプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class NamespaceLabelProvider extends LabelProvider implements
			ITableLabelProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}
	}

	/**
	 * インポートスキーマセルModifier
	 * 
	 * @author shumi
	 * 
	 */
	public static class ImportCellModifier implements ICellModifier {
		/**
		 * ビュー
		 */
		SchemaEditorView view;

		/**
		 * テーブルビュー
		 */
		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param view
		 *            ビュー
		 * @param viewer
		 *            テーブルビュー
		 */
		public ImportCellModifier(SchemaEditorView view, TableViewer viewer) {
			this.view = view;
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			return tblObj.isEditable() && (property.length() > 0);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			TableObject tblObj = (TableObject) element;
			MispSchemaImport im = (MispSchemaImport) tblObj.getData();
			if (property.equals("uri")) //$NON-NLS-1$
				value = im.getUri();
			if (property.equals("location")) //$NON-NLS-1$
				value = im.getLocation();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			int op = SchemaEditorViewOperation.OP_NONE;
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			MispSchemaImport im = (MispSchemaImport) tblObj.getData();
			if (property.equals("uri")) //$NON-NLS-1$
			{
				op = view.getOperation().opImport(view.getCurrSchema(), im,
						SchemaEditorViewOperation.OP_IM_URI, value);
			}
			if (property.equals("location")) //$NON-NLS-1$
			{
				op = view.getOperation().opImport(view.getCurrSchema(), im,
						SchemaEditorViewOperation.OP_IM_LOCATION, value);
			}
			if (op != SchemaEditorViewOperation.OP_NONE)
				view.updateTableViewer(viewer, element);
		}
	}

	/**
	 * インポートスキーマコンテントプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class ImportContentProvider implements
			IStructuredContentProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if ((inputElement != null) && (inputElement instanceof MispSchema)) {
				MispSchema schema = (MispSchema) inputElement;
				return TableObject.toArray(schema.getImportArray());
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}
	}

	/**
	 * インポートスキーマラベルプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class ImportLabelProvider extends LabelProvider implements
			ITableLabelProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}
	}

	/**
	 * 複合型セルModifier
	 * 
	 * @author shumi
	 * 
	 */

	public static class ComplexTypeCellModifier implements ICellModifier {

		/**
		 * ビュー
		 */
		SchemaEditorView view;

		/**
		 * テーブルビュー
		 */
		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param view
		 *            ビュー
		 * @param viewer
		 *            テーブルビュー
		 */
		public ComplexTypeCellModifier(SchemaEditorView view, TableViewer viewer) {
			this.view = view;
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			return tblObj.isEditable() && (property.length() > 0);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			TableObject tblObj = (TableObject) element;
			MispSchemaComplexType ct = (MispSchemaComplexType) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
				value = ct.getName();
			if (property.equals("extension")) //$NON-NLS-1$
				value = ct.getExtension();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			int op = SchemaEditorViewOperation.OP_NONE;
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			MispSchemaComplexType ct = (MispSchemaComplexType) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
			{
				op = view.getOperation().opComplexType(view.getCurrSchema(),
						ct, SchemaEditorViewOperation.OP_CT_NAME, value);

			}
			if (property.equals("extension")) //$NON-NLS-1$
			{
				op = view.getOperation().opComplexType(view.getCurrSchema(),
						ct, SchemaEditorViewOperation.OP_CT_EXTENSION, value);
			}
			if (op != SchemaEditorViewOperation.OP_NONE)
				view.updateTableViewer(viewer, element);
		}
	}

	/**
	 * 複合型コンテントプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class ComplexTypeContentProvider implements
			IStructuredContentProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if ((inputElement != null) && (inputElement instanceof MispSchema)) {
				MispSchema schema = (MispSchema) inputElement;
				Object[] internals = schema.getComplexTypeArray(true, false);
				Object[] externals = schema.getComplexTypeArray(false, true);
				Object[] alls = new Object[internals.length + externals.length];
				int index = 0;
				for (int cnt = 0; cnt < internals.length;)
					alls[index++] = internals[cnt++];
				for (int cnt = 0; cnt < externals.length;)
					alls[index++] = externals[cnt++];

				return TableObject.toArray(alls);
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}

	}

	/**
	 * 複合型ラベルプロバイダ
	 * 
	 * @author shumi
	 * 
	 */
	public static class ComplexTypeLabelProvider extends LabelProvider
			implements ITableLabelProvider {

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}
	}

	/**
	 * 属性セルModifier
	 * 
	 * @author shumi
	 * @deprecated
	 */
	public static class AttributeCellModifier implements ICellModifier {
		/**
		 * ビュー
		 */
		SchemaEditorView view;

		/**
		 * テーブルビュー
		 */
		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param view
		 *            ビュー
		 * @param viewer
		 *            テーブルビュー
		 */

		public AttributeCellModifier(SchemaEditorView view, TableViewer viewer) {
			this.view = view;
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			return tblObj.isEditable() && (property.length() > 0);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			TableObject tblObj = (TableObject) element;
			MispSchemaAttribute attr = (MispSchemaAttribute) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
				value = attr.getName();
			if (property.equals("type")) //$NON-NLS-1$
				value = attr.getType();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			int op = SchemaEditorViewOperation.OP_NONE;
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			MispSchemaAttribute attr = (MispSchemaAttribute) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
			{
				op = view.getOperation().opAttribute(view.getCurrSchema(),
						attr, SchemaEditorViewOperation.OP_ATTR_NAME, value);
			}
			if (property.equals("type")) //$NON-NLS-1$
			{
				op = view.getOperation().opAttribute(view.getCurrSchema(),
						attr, SchemaEditorViewOperation.OP_ATTR_TYPE, value);
			}
			if (op != SchemaEditorViewOperation.OP_NONE)
				view.updateTableViewer(viewer, element);
		}
	}

	/**
	 * 属性コンテントプロバイダ
	 * 
	 * @author shumi
	 * @deprecated
	 */
	public static class AttributeContentProvider implements
			IStructuredContentProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */

		public Object[] getElements(Object inputElement) {
			if ((inputElement != null)
					&& (inputElement instanceof MispSchemaComplexType)) {
				MispSchemaComplexType schema = (MispSchemaComplexType) inputElement;
				return TableObject.toArray(schema.getAttributeArray());
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}
	}

	/**
	 * 属性ラベルプロバイダ
	 * 
	 * @author shumi
	 * @deprecated
	 */
	public static class AttributeLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}

	}

	/**
	 * 要素セルModifier
	 * 
	 * @author shumi
	 * @deprecated
	 * 
	 */
	public static class ElementCellModifier implements ICellModifier {

		/**
		 * ビュー
		 */
		SchemaEditorView view;

		/**
		 * テーブルビュー
		 */
		TableViewer viewer;

		/**
		 * Modifierを生成する
		 * 
		 * @param view
		 *            ビュー
		 * @param viewer
		 *            テーブルビュー
		 */

		public ElementCellModifier(SchemaEditorView view, TableViewer viewer) {
			this.view = view;
			this.viewer = viewer;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			return tblObj.isEditable() && (property.length() > 0);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			Object value = ""; //$NON-NLS-1$
			TableObject tblObj = (TableObject) element;
			MispSchemaElement em = (MispSchemaElement) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
				value = em.getName();
			if (property.equals("type")) //$NON-NLS-1$
				value = em.getType();
			return value;
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			int op = SchemaEditorViewOperation.OP_NONE;

			if (element instanceof Item) {
				element = ((Item) element).getData();
			}
			TableObject tblObj = (TableObject) element;
			MispSchemaElement em = (MispSchemaElement) tblObj.getData();
			if (property.equals("name")) //$NON-NLS-1$
			{
				op = view.getOperation().opElement(view.getCurrSchema(), em,
						SchemaEditorViewOperation.OP_EM_NAME, value);
			}
			if (property.equals("type")) //$NON-NLS-1$
			{
				op = view.getOperation().opElement(view.getCurrSchema(), em,
						SchemaEditorViewOperation.OP_EM_TYPE, value);
			}
			if (op != SchemaEditorViewOperation.OP_NONE)
				view.updateTableViewer(viewer, element);
		}
	}

	/**
	 * 要素コンテントプロバイダ
	 * 
	 * @author shumi
	 * @deprecated
	 */
	public static class ElementContentProvider implements
			IStructuredContentProvider {

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if ((inputElement != null)
					&& (inputElement instanceof MispSchemaComplexType)) {
				MispSchemaComplexType schema = (MispSchemaComplexType) inputElement;
				return TableObject.toArray(schema.getElementArray());
			}
			return new Object[0];
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {

		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}
	}

	/**
	 * 要素ラベルプロバイダ
	 * 
	 * @author shumi
	 * @deprecated
	 */
	public static class ElementLabelProvider extends LabelProvider implements
			ITableLabelProvider {
		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return getColImage(element, columnIndex);
		}

		/**
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return getColText(element, columnIndex);
		}
	}

	/**
	 * カラムのイメージを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return イメージ
	 */
	public static Image getColImage(Object element, int columnIndex) {
		return (getColValidate(element, columnIndex).length() > 0) ? SchemaEditorView.invalidImg
				: SchemaEditorView.validImg;
	}

	/**
	 * カラムの検証結果を取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return 検証結果
	 */
	public static String getColValidate(Object element, int columnIndex) {
		String validate = ""; //$NON-NLS-1$
		TableObject tblObj = (TableObject) element;
		Object data = tblObj.getData();
		if (data instanceof MispSchemaNamespace) {
			MispSchemaNamespace ns = (MispSchemaNamespace) data;
			switch (columnIndex) {
			case 0:
				// validate = ns.validate(true);
				break;
			case 1:
				validate = ns.validatePrefix(true);
				break;
			case 2:
				validate = ns.validateUri(true);
				break;
			}
		} else if (data instanceof MispSchemaImport) {
			MispSchemaImport im = (MispSchemaImport) data;
			switch (columnIndex) {
			case 0:
				// validate = im.validate(true);
				break;
			case 1:
				validate = im.validateUri(true);
				break;
			case 2:
				validate = im.validateLocation(true);
				break;
			}

		} else if (data instanceof MispSchemaComplexType) {
			MispSchemaComplexType ct = (MispSchemaComplexType) data;
			switch (columnIndex) {
			case 0:
				validate = ct.validate(true);
				break;
			case 1:
				validate = ct.validateName(true);
				break;
			case 2:
				validate = ct.validateExtension(true);
				break;
			}

		} else if (data instanceof MispSchemaAttribute) {
			MispSchemaAttribute attr = (MispSchemaAttribute) data;
			switch (columnIndex) {
			case 0:
				// validate = attr.validate(true);
				break;
			case 1:
				validate = attr.validateName(true);
				break;
			case 2:
				validate = attr.validateType(true);
				break;
			}

		} else if (data instanceof MispSchemaElement) {
			MispSchemaElement em = (MispSchemaElement) data;
			switch (columnIndex) {
			case 0:
				// validate = em.validate(true);
				break;
			case 1:
				validate = em.validateName(true);
				break;
			case 2:
				validate = em.validateType(true);
				break;
			}
		}

		return validate;
	}

	/**
	 * カラムのテキストを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return テキスト
	 */
	public static String getColText(Object element, int columnIndex) {
		String text = ""; //$NON-NLS-1$
		TableObject tblObj = (TableObject) element;
		Object data = tblObj.getData();
		if (data instanceof MispSchemaNamespace) {
			MispSchemaNamespace ns = (MispSchemaNamespace) data;
			switch (columnIndex) {
			case 0:
				text = "" + (ns.getIndex() + 1); //$NON-NLS-1$
				break;
			case 1:
				text = ns.getPrefix();
				break;
			case 2:
				text = ns.getUri();
				break;
			}
		} else if (data instanceof MispSchemaImport) {
			MispSchemaImport im = (MispSchemaImport) data;
			switch (columnIndex) {
			case 0:
				text = "" + (im.getIndex() + 1); //$NON-NLS-1$
				break;
			case 1:
				text = im.getUri();
				break;
			case 2:
				text = im.getLocation();
				break;
			}
		} else if (data instanceof MispSchemaComplexType) {
			MispSchemaComplexType ct = (MispSchemaComplexType) data;
			switch (columnIndex) {
			case 0:
				text = "" + (ct.getIndex() + 1); //$NON-NLS-1$
				break;
			case 1:
				text = ct.getName();
				break;
			case 2:
				text = ct.getExtension();
				break;
			}
		} else if (data instanceof MispSchemaAttribute) {
			MispSchemaAttribute attr = (MispSchemaAttribute) data;
			switch (columnIndex) {
			case 0:
				text = "" + (attr.getIndex() + 1); //$NON-NLS-1$
				break;
			case 1:
				text = attr.getName();
				break;
			case 2:
				text = attr.getType();
				break;
			}
		} else if (data instanceof MispSchemaElement) {
			MispSchemaElement em = (MispSchemaElement) data;
			switch (columnIndex) {
			case 0:
				text = "" + (em.getIndex() + 1); //$NON-NLS-1$
				break;
			case 1:
				text = em.getName();
				break;
			case 2:
				text = em.getType();
				break;
			}
		}
		return text;
	}

	/**
	 * 
	 * カラムのチップテキストを取得する
	 * 
	 * @param element
	 *            エレメント
	 * @param columnIndex
	 *            カラムインデックス
	 * @return チップテキスト
	 */
	public static String getColToolTipText(Object element, int columnIndex) {
		String tip = ""; //$NON-NLS-1$
		TableObject tblObj = (TableObject) element;
		Object data = tblObj.getData();
		if (data instanceof MispSchemaNamespace) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("SchemaEditorViewProvider.33"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("SchemaEditorViewProvider.34"); //$NON-NLS-1$
				break;
			}
		} else if (data instanceof MispSchemaImport) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("SchemaEditorViewProvider.35"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("SchemaEditorViewProvider.36"); //$NON-NLS-1$
				break;
			}

		} else if (data instanceof MispSchemaComplexType) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("SchemaEditorViewProvider.37"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("SchemaEditorViewProvider.38"); //$NON-NLS-1$
				break;
			}
		} else if (data instanceof MispSchemaAttribute) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("SchemaEditorViewProvider.39"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("SchemaEditorViewProvider.40"); //$NON-NLS-1$
				break;
			}

		} else if (data instanceof MispSchemaElement) {
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				tip = Messages.getString("SchemaEditorViewProvider.41"); //$NON-NLS-1$
				break;
			case 2:
				tip = Messages.getString("SchemaEditorViewProvider.42"); //$NON-NLS-1$
				break;
			}
		}
		return tip;
	}

	public static class ComplexTypeDetailContentProvider implements
			ITreeContentProvider {

		public Object[] getChildren(Object item) {
			if (item instanceof TreeObject) {
				TreeObject treeObj = (TreeObject) item;
				Object data = treeObj.getData();
				MispSchemaComplexType ct = null;
				if (data instanceof MispSchema.MispSchemaComplexType) {
					ct = (MispSchema.MispSchemaComplexType) data;
				} else if (data instanceof MispSchema.MispSchemaElement) {
					MispSchema.MispSchemaElement et = (MispSchema.MispSchemaElement) data;
					ct = et.findComplexType();
				}
				if (ct != null) {
					Object[] arrays = new Object[4];
					List extAttrList = ct.getAttributeElementList(true, false,
							true, false);
					arrays[0] = extAttrList;
					List attrList = ct.getAttributeElementList(false, true,
							true, false);
					arrays[1] = attrList;
					List extEmList = ct.getAttributeElementList(true, false,
							false, true);
					arrays[2] = extEmList;
					List emList = ct.getAttributeElementList(false, true,
							false, true);
					arrays[3] = emList;
					boolean[] editables = new boolean[4];
					editables[0] = false;
					editables[1] = true;
					editables[2] = false;
					editables[3] = true;

					return treeObj.toArray2((TreeObject) item, arrays,
							editables);
				}

			}
			return new Object[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object item) {
			if (item instanceof TreeObject) {
				TreeObject treeObj = (TreeObject) item;
				Object data = treeObj.getData();
				if (data instanceof MispSchema.MispSchemaComplexType) {
					MispSchema.MispSchemaComplexType ct = (MispSchema.MispSchemaComplexType) data;
					return ct.getAttributeElementSize(true, true, true, true) > 0;
				} else if (data instanceof MispSchema.MispSchemaElement) {
					MispSchema.MispSchemaElement et = (MispSchema.MispSchemaElement) data;
					MispSchema.MispSchemaComplexType ct = et.findComplexType();
					return (ct != null) && (ct.getAttributeElementSize(true, true, true, true) > 0);
				}
			}
			return false;
		}

		public Object[] getElements(Object inputElement) {
			if (inputElement != null) {
				if (inputElement instanceof MispSchemaComplexType) {
					MispSchemaComplexType ct = (MispSchemaComplexType) inputElement;
					TreeObject treeObj = new TreeObject(ct);
					treeObj
							.setEditable(ct.getKind() == MispSchema.KIND_INTERNAL);
					return new Object[] { treeObj };
				}
				if (inputElement instanceof TreeObject) {
					return getChildren(inputElement);
				}
			}
			return new Object[0];
		}

		public void dispose() {

		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	public static class ComplexTypeDetailLabelProvider extends LabelProvider {
		boolean blComplexDetail;

		public ComplexTypeDetailLabelProvider(boolean blComplexDetail) {
			this.blComplexDetail = blComplexDetail;
		}

		public String getText(Object item) {
			if (item instanceof TreeObject) {
				TreeObject treeObj = (TreeObject) item;
				Object data = treeObj.getData();

				if (data instanceof MispSchema.MispSchemaComplexType) {
					String text = ((MispSchema.MispSchemaComplexType) data)
							.getName();
					if (blComplexDetail) {
						text += Messages
								.getString("SchemaEditorViewProvider.25");
						if (((MispSchemaComplexType) data).getKind() != MispSchema.KIND_INTERNAL) {
							text += "("
									+ ((MispSchemaComplexType) data)
											.getFilePath() + ")";
						}
					}
					return (text); //$NON-NLS-1$ //$NON-NLS-2$
				}
				return data.toString();
			}
			return item.toString();
		}

		public Image getImage(Object item) {
			Image image = null;
			String validate = ""; //$NON-NLS-1$

			if (item instanceof TreeObject) {
				TreeObject treeObj = (TreeObject) item;
				Object data = treeObj.getData();

				if (data instanceof MispSchemaComplexType) {
					// validate = ((MispSchema.MispSchemaComplexType)
					// data).validate(true);
					image = (validate.length() > 0) ? SchemaEditorView.invalidImg
							: rootImg;
				} else if (data instanceof MispSchema.MispSchemaAttribute) {
					validate = ((MispSchema.MispSchemaAttribute) data)
							.validate(true);
					image = (validate.length() > 0) ? SchemaEditorView.invalidImg
							: (treeObj.isEditable() ? attributeImg
									: attributeImg2);
				} else if (data instanceof MispSchema.MispSchemaElement) {
					validate = ((MispSchema.MispSchemaElement) data)
							.validate(true);
					image = (validate.length() > 0) ? SchemaEditorView.invalidImg
							: (treeObj.isEditable() ? elementImg : elementImg2);
				}
			}
			return image;
		}
	}
}
