package daruma.client.schemaeditor.ui.views;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

/**
 * テーブルマウス移動リスナー
 * 
 * @author shumi
 * 
 */
public class TableMouseMoveListener implements MouseMoveListener {
	/**
	 * 前回行番号
	 */
	int prevRow = -1;

	/**
	 * 前回カラム番号
	 */
	int prevCol = -1;

	/**
	 * ビュー
	 */
	Object view;

	/**
	 * テーブルビュー
	 */
	TableViewer viewer;

	/**
	 * リスナーを生成する
	 * 
	 * @param view
	 *            ビュー
	 * @param viewer
	 *            テーブルビュー
	 */
	public TableMouseMoveListener(Object view, TableViewer viewer) {
		this.view = view;
		this.viewer = viewer;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.events.MouseMoveListener#mouseMove(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseMove(MouseEvent e) {
		if (e.widget != null) {
			if (e.widget instanceof Table) {
				Table table = (Table) e.widget;
				int row = getRow(table, e.y);
				int col = getCol(table, e.x);
				String text = "";
				if ((row >= 0) && (col >= 0)) {
					Object element = viewer.getElementAt(row);
					if (element != null) {
						if (view instanceof SchemaEditorView) {
							text = SchemaEditorViewProvider.getColValidate(
									element, col);
							if (text.length() <= 0) {
								text = SchemaEditorViewProvider
										.getColToolTipText(element, col);
							}
						} else if (view instanceof CsvxEditorDialog) {
							text = CsvxEditorDialogProvider.getColValidate(
									element, col);
							if (text.length() <= 0) {
								text = CsvxEditorDialogProvider
										.getColToolTipText(element, col);
							}
						}
					}
				}
				if ((row != prevRow) || (col != prevCol)) {
					if (view instanceof SchemaEditorView) {
						table.setToolTipText(text.length() > 0 ? text
								: ((SchemaEditorView) view).toolTipNoneTblText);
					}else if (view instanceof CsvxEditorDialog) {
						table.setToolTipText(text.length() > 0 ? text
								: ((CsvxEditorDialog) view).toolTipNoneTblText);
					}
				}
				prevRow = row;
				prevCol = col;
			}
		}
	}

	/**
	 * 指定された座標の行番号を取得する
	 * 
	 * @param table
	 *            テーブル
	 * @param y
	 *            ｙ座標
	 * @return 行番号
	 */
	public int getRow(Table table, int y) {
		for (int cnt = 0; cnt < table.getItemCount(); cnt++) {
			TableItem item = table.getItem(cnt);
			Rectangle rect = item.getBounds(0);
			int boundY1 = rect.y;
			int boundY2 = boundY1 + rect.height;
			if ((boundY1 < y) && (y < boundY2)) {
				return cnt;
			}
		}
		return -1;
	}

	/**
	 * 指定された座標のカラム番号を取得する
	 * 
	 * @param table
	 *            テーブル
	 * @param x
	 *            x座標
	 * @return カラム番号
	 */
	public int getCol(Table table, int x) {
		if (table.getItemCount() > 0) {
			TableItem item = table.getItem(0);
			for (int cnt = 0; cnt < table.getColumnCount(); cnt++) {
				Rectangle rect = item.getBounds(cnt);
				int boundX1 = rect.x;
				int boundX2 = boundX1 + rect.width;
				if ((boundX1 < x) && (x < boundX2)) {
					return cnt;
				}
			}
		}
		return -1;
	}

}
