package daruma.client.schemaeditor.ui.views;

import org.eclipse.core.runtime.IAdaptable;

import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;
import daruma.bus.processor.csv.MispSchema.MispSchemaImport;
import daruma.bus.processor.csv.MispSchema.MispSchemaNamespace;


/**
 * ツリーのノードオブジェクト
 * 詳しいことはEclipseドキュメントを参照
 *
 */
public class TableObject implements IAdaptable{
	private Object data;
	
	/**
	 * 指定されたデータのツリーオブジェクトを生成する
	 * @param data
	 */
	public TableObject(Object data) {
		this.data = data;
	}
	
	/**
	 * ツリーで表示する名前を取得する
	 * @return　名前を返す
	 */
	public String getName() {
		
		if(data == null)
		{
			return "NULL";
		}
		else
		{
			return data.toString();
		}
	}

	/**
	 * オブジェクトを記述する文字列を取得
	 * @return　文字列を返す 
	 */
	public String toString() {
		return getName();
	}

	public Object getAdapter(Class key) {
		return null;
	}

	/**
	 * データを取得
	 * @return データを返す
	 */
	public Object getData() {
		return data;
	}
	
	/**
	 * データを設定する
	 * @param data
	 *             データ
	 */
	public void setData(Object data) {
		this.data = data;
	}
	
	public static Object[] toArray(Object[] srcArray)
	{
		Object[] dstArray = new Object[srcArray.length];
		for(int cnt = 0; cnt < srcArray.length; cnt++)
		{
			
			TableObject child = new TableObject(srcArray[cnt]);
			dstArray[cnt] =  child;
		}
		return dstArray;
	}
	
	public boolean equals(Object obj)
	{
		if(obj == null) return false;
		if(!(obj instanceof TableObject)) return false;
		return this.data == (((TableObject)obj).getData());
	}
	
	public boolean isEditable()
	{
		boolean rc = false;
		if(data != null)
		{
			if(data instanceof MispSchemaNamespace)
			{
				rc = true;				
			}
			else if(data instanceof MispSchemaImport)
			{
				rc = true;
			}
			else if(data instanceof MispSchemaComplexType)
			{
				rc = ((MispSchemaComplexType)data).getKind() == MispSchema.KIND_INTERNAL;
			}
			else if(data instanceof MispSchemaAttribute)
			{
				rc = true;
			}
			else if(data instanceof MispSchemaElement)
			{
				rc = true;
			}
		}
		return rc;

	}
}	
