package daruma.client.schemaeditor.ui.views;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.ui.IWorkbenchActionConstants;

/**
 * テーブルビューメニューリスナー
 * 
 * @author shumi
 * 
 */
public class TableViewerMenuListener implements IMenuListener {
	/**
	 * ビュー
	 */
	TableViewer viewer;

	/**
	 * 親ビュー
	 */
	TableViewer parentViewer;

	/**
	 * アクション配列
	 */
	Action[] actions;

	/**
	 * アクション有効フラグ配列
	 */
	int[] enableTypes;

	/**
	 * テーブルビューメニューリスナーを生成する
	 * 
	 * @param parentViewer
	 *            親ビュー
	 * @param viewer
	 *            ビュー
	 * @param actions
	 *            アクション配列
	 * @param enableTypes
	 *            アクション有効フラグ配列
	 */
	public TableViewerMenuListener(TableViewer parentViewer,
			TableViewer viewer, Action[] actions, int[] enableTypes) {
		this.parentViewer = parentViewer;
		this.viewer = viewer;
		this.actions = actions;
		this.enableTypes = enableTypes;
	}

	public static int ENABLE_TYPE_ALWAYS = 0;
	
	public static int ENABLE_TYPE_SELECTED = 1;
	
	public static int ENABLE_TYPE_PARENT_SELECTED = 2;
	
	public static int ENABLE_TYPE_NOT_FIRST = 3;
	
	public static int ENABLE_TYPE_NOT_LAST = 4;
	
	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void menuAboutToShow(IMenuManager manager) {
		ISelection selection = viewer.getSelection();
		Object item = ((IStructuredSelection) selection).getFirstElement();
		boolean editable = true;
		if ((item instanceof TableObject) && !((TableObject) item).isEditable()) {
			editable = false;
		}

		boolean flgParentSelected = (parentViewer != null)
				&& (parentViewer.getTable().getSelectionIndex() >= 0);
		int index = viewer.getTable().getSelectionIndex();
		int size = viewer.getTable().getItemCount();
		boolean flgSelected = (index >= 0);
		for (int cnt = 0; cnt < actions.length; cnt++) {
			boolean enabled = true;
			if ((enableTypes != null) && (cnt < enableTypes.length)) {
				if (enableTypes[cnt] == 1)
					enabled = flgSelected && editable;
				else if (enableTypes[cnt] == 2)
					enabled = flgParentSelected && editable;
				else if (enableTypes[cnt] == 3)
					enabled = flgSelected && (index > 0) && editable;
				else if (enableTypes[cnt] == 4)
					enabled = flgSelected && (index < (size - 1)) && editable;
				else
					enabled = editable;
			}
			Action action = actions[cnt];
			if (action == null) {
				manager.add(new Separator());
			} else {
				action.setEnabled(enabled);
				manager.add(actions[cnt]);
			}
		}
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

}
