package daruma.client.schemaeditor.ui.views;

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import daruma.bus.processor.csv.MispSchema;

/**
 * テーブルマウス移動リスナー
 * 
 * @author shumi
 * 
 */
public class TreeMouseMoveListener implements MouseMoveListener {
	/**
	 * 前回行番号
	 */
	TreeItem prevItem;

	/**
	 * ビュー
	 */
	SchemaEditorView view;

	/**
	 * テーブルビュー
	 */
	TreeViewer viewer;

	/**
	 * リスナーを生成する
	 * 
	 * @param view
	 *            ビュー
	 * @param viewer
	 *            テーブルビュー
	 */
	public TreeMouseMoveListener(SchemaEditorView view, TreeViewer viewer) {
		this.view = view;
		this.viewer = viewer;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.events.MouseMoveListener#mouseMove(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseMove(MouseEvent e) {
		if (e.widget != null) {
			if (e.widget instanceof Tree) {
				Tree tree = (Tree) e.widget;
				TreeItem item = tree.getItem(new Point(e.x, e.y));
				if (item != null) {
					Object obj = item.getData();

					if (obj instanceof TreeObject) {
						TreeObject treeObj = (TreeObject) obj;
						Object data = treeObj.getData();

						String validate = "";
						if (data instanceof MispSchema.MispSchemaAttribute) {
							validate = ((MispSchema.MispSchemaAttribute) data)
									.validate(true);
						}
						if (data instanceof MispSchema.MispSchemaElement) {
							validate = ((MispSchema.MispSchemaElement) data)
									.validate(true);
						}
						if (prevItem != item) {
							tree
									.setToolTipText(validate.length() > 0 ? validate
											: view.toolTipNoneTblText);
						}
					}
				}
				prevItem = item;
			}
		}
	}

}
