package daruma.client.schemaeditor.ui.views;

import java.util.List;

import org.eclipse.core.runtime.IAdaptable;

import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchema.MispSchemaAttribute;
import daruma.bus.processor.csv.MispSchema.MispSchemaComplexType;
import daruma.bus.processor.csv.MispSchema.MispSchemaElement;


/**
 * ツリーのノードオブジェクト
 * 詳しいことはEclipseドキュメントを参照
 *
 */
public class TreeObject implements IAdaptable{
	private TreeObject parent = null;
	private Object data;
	private boolean editable = true;
	
	/**
	 * 指定されたデータのツリーオブジェクトを生成する
	 * @param data
	 */
	public TreeObject(Object data) {
		this.parent = null;
		this.data = data;
	}
	
	/**
	 * 指定されたデータのツリーオブジェクトを生成する
	 * @param data
	 */
	public TreeObject(TreeObject parent, Object data) {
		this.parent = parent;
		this.data = data;
	}

	/**
	 * ツリーで表示する名前を取得する
	 * @return　名前を返す
	 */
	public String getName() {
		
		if(data == null)
		{
			return "NULL";
		}
		else
		{
			return data.toString();
		}
	}

	/**
	 * 親を設定する
	 * @param parent　親
	 */
	public void setParent(TreeObject parent) {
		this.parent = parent;
	}

	/**
	 * 親を取得する
	 * @return 親を返す
	 */
	public TreeObject getParent() {
		return parent;
	}
	
	public boolean hasParent()
	{
		return parent != null;
	}

	/**
	 * オブジェクトを記述する文字列を取得
	 * @return　文字列を返す 
	 */
	public String toString() {
		return getName();
	}

	public Object getAdapter(Class key) {
		return null;
	}

	/**
	 * データを取得
	 * @return データを返す
	 */
	public Object getData() {
		return data;
	}
	
	

	/**
	 * editableを取得する
	 * @return Returns the editable.
	 */
	public boolean isEditable() {
		boolean rc = editable;
		if (data instanceof MispSchemaAttribute) {
			MispSchemaAttribute attr = (MispSchemaAttribute) data;
			MispSchemaComplexType ct = attr.getParentCt();
			rc = editable && (ct != null)
					&& (ct.getKind() == MispSchema.KIND_INTERNAL);
		} else if (data instanceof MispSchemaElement) {
			MispSchemaElement em = (MispSchemaElement) data;
			MispSchemaComplexType ct = em.getParentCt();
			rc = editable && (ct != null)
					&& (ct.getKind() == MispSchema.KIND_INTERNAL);
		}
		return rc;
	}

	/**
	 * editableをセットする
	 * @param editable The editable to set.
	 */
	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	/**
	 * データを設定する
	 * @param data
	 *             データ
	 */
	public void setData(Object data) {
		this.data = data;
	}
	
	public Object[] toArray(TreeObject  parent, Object[] srcArray)
	{
		Object[] dstArray = new Object[srcArray.length];
		for(int cnt = 0; cnt < srcArray.length; cnt++)
		{
			
			TreeObject child = new TreeObject(parent, srcArray[cnt]);
			child.parent = this;
			child.editable = parent.editable;
			dstArray[cnt] =  child;
		}
		return dstArray;
	}
	
	public Object[] toArray2(TreeObject  parent, Object[] srcListArray, boolean[] editableArray)
	{
		int length = 0;
		for(int cnti = 0; cnti < srcListArray.length; cnti++)
		{
			length += ((List)srcListArray[cnti]).size();
		}
		
		Object[] dstArray = new Object[length];
		int index = 0;
		for(int cnti = 0; cnti < srcListArray.length; cnti++)
		{
			List srcList = (List)srcListArray[cnti];
			boolean editable = parent.editable && editableArray[cnti];
			for(int cntj = 0; cntj < srcList.size(); cntj++)
			{
				TreeObject child = new TreeObject(parent, srcList.get(cntj));
				child.parent = this;
				child.editable = editable;
				dstArray[index++] =  child;
			}
		}
		return dstArray;
	}
	
}
