package application.runtime;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * アプリケーションの抽象クラス
 * 
 * @author masanori.matsuoka@netconscious.jp
 * 
 */
abstract public class Application {

    static private Log log = LogFactory.getLog(Application.class);

    /**
     * シャットダウン時のイベントハンドラ
     * 
     */
    abstract protected void onShutdown();

    /**
     * ファイナライズ時(GC)のイベントハンドラ
     * 
     */
    abstract protected void onFinalize();

    /**
     * アプリケーションの初期化時のイベントハンドラ
     */
    abstract protected boolean onInitilaize();

    /**
     * スタート時のイベントハンドラ
     * 
     * @param parameterMap
     *            コマンドライン引数のパラメータ
     */
    abstract protected int onStart(String[] parameters);

    public void shutdown() {
        onShutdown();
    }

    /**
     * finalizer
     */
    protected void finalize() throws Throwable {
        onFinalize();
        super.finalize();
    }

    /**
     * アプリケーションの起動クラス
     * 
     * @param parameters
     *            引数
     */
    public int main(String[] parameters) {

        log.info("Main Application is invoked.");

        if (!onInitilaize()) {
            shutdown();
            return -1;
        }

        // スタートハンドラの呼び出し
        return this.onStart(parameters);

    }

}
