package application.runtime;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class BasisRuntime {

    static private Log log = LogFactory.getLog(BasisRuntime.class);
    

    static public boolean openContext(boolean openSystemContext, boolean openComponentContext, boolean openUserContext) {
        // load system context

        if (openSystemContext) {
            log.info("SystemContext is initializing...");
            if (SystemContext.openSystemContext()) {
                log.info("SystemContext was initilaized.");
            }
            else {
                log.error("SystemContext was not initialized.");
                return false;
            }

        }

        // load user context
        if (openComponentContext) {
            log.info("ComponentContext is initializing...");
            if (SystemContext.openComponentContext()) {
                log.info("ComponentContext was initilaized.");
                SystemContext.startComponents(true);
            }
            else {
                log.warn("ComponentContext was not initialized.");
            }

        }

        // load user context
        if (openUserContext) {
            log.info("UserContext is initializing...");
            if (SystemContext.openUserContext()) {
                log.info("UserContext was initilaized.");
            }
            else {
                log.warn("UserContext was not initialized.");
            }
        }

        return true;
    }

    static public void closeContext() {
        SystemContext.shutdownComponents();
        SystemContext.close();
    }

    protected void finalize() throws Throwable {
        closeContext();
        super.finalize();
    }

}
