package application.runtime;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class CommandLineHandler {

    private List optionList = new ArrayList();
    private CommandLineParser parser;
    private Options options = new Options();
    private String commandName ="";
    
    public String getCommandName() {
        return commandName;
    }

    public void setCommandName(String commandName) {
        this.commandName = commandName;
    }
    
    public CommandLine getCommandLineObject(String[] args){
        try {
            Iterator itOption = optionList.iterator();
            while(itOption.hasNext()){
                CommandLineOption opt = (CommandLineOption)itOption.next(); 
                options.addOption(opt.createOption());
                
            }
            
            //Map parameterMap = new HashMap();
            CommandLine cmd = parser.parse(options,args,true);
            
            return cmd;
            
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * コマンドライン引数からパラメータを取り出す
     * @param args　コマンドライン引数
     * @return　引数名と値のマップオブジェクト
     */
    public Map retrieveParameterMap(String[] args){
        
        try {
            Iterator itOption = optionList.iterator();
            while(itOption.hasNext()){
                CommandLineOption opt = (CommandLineOption)itOption.next(); 
                options.addOption(opt.createOption());
            }
            
            Map parameterMap = new HashMap();
            CommandLine cmd = parser.parse(options,args,true);
            
            itOption = optionList.iterator();
            while(itOption.hasNext()){
                CommandLineOption opt = (CommandLineOption)itOption.next(); 
                if(opt.isHasValue()){
                    String[] values = cmd.getOptionValues(opt.getArgName());
                    parameterMap.put(opt.getArgName(),values);
                }
                else {
                    parameterMap.put(opt.getArgName(),null);
                }
            }
            return parameterMap;
            
        } catch (ParseException e) {
            return null;
        }
        
    }
    
    /**
     * ヘルプを表示する
     */
    public void printHelp(){
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp( commandName, options );
    }
    

    public CommandLineParser getParser() {
        return parser;
    }

    public void setParser(CommandLineParser parser) {
        this.parser = parser;      
    }

    public List getOptionList() {
        return optionList;
    }

    public void setOptionList(List optionList) {
        this.optionList = optionList;
    }
    
    
    
   
}
