package application.runtime;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class LocalFileHandler {

    private static Log log = LogFactory.getLog(LocalFileHandler.class);
    private Map fileCache;
    private String rootDirectory;
    private String regexFileFilter;

    private void createFileCache() {

        if (fileCache == null)
            fileCache = new HashMap();
        else
            fileCache.clear();

        Pattern pt;
        if (regexFileFilter != null)
            pt = Pattern.compile(regexFileFilter);
        else
            pt = Pattern.compile(".*");

        File rootFile = new File(rootDirectory);
        if (rootFile.isDirectory() && rootFile.exists()) {
            File[] files = rootFile.listFiles();
            for (int index = 0; index < files.length; index++) {
                File curFile = files[index];
                if (pt.matcher(curFile.getName()).matches() && curFile.isFile())
                    fileCache.put(curFile, new Long(curFile.lastModified()));
            }
        }

    }

    public Set getModifiedFiles() {

        Set result = new HashSet();
        Iterator itCurFile = fileCache.keySet().iterator();
        while (itCurFile.hasNext()) {
            File curFile = (File) itCurFile.next();
            if (curFile.exists()) {
                Long lastModified = (Long) fileCache.get(curFile);
                if (curFile.lastModified() != lastModified.longValue()) {
                    result.add(curFile);
                }
            }
        }
        Iterator itModifiedFile = result.iterator();
        while (itModifiedFile.hasNext()) {
            File modifiedFile = (File) itModifiedFile.next();
            fileCache.put(modifiedFile, new Long(modifiedFile.lastModified()));
        }

        return result;
    }

    public String getRootDirectory() {
        return rootDirectory;
    }

    public void setRootDirectory(String rootDirectory) {
        this.rootDirectory = rootDirectory;
        fileCache = null;
    }

    public Set getFiles() {
        if (fileCache == null)
            this.createFileCache();
        return fileCache.keySet();
    }

    public Set getCanonicalFilePaths() {
        if (fileCache == null)
            this.createFileCache();

        try {
            Set result = new HashSet();
            Iterator itCurFile = fileCache.keySet().iterator();
            while (itCurFile.hasNext()) {
                File curFile = (File) itCurFile.next();
                result.add(curFile.getCanonicalPath());
            }
            return result;
        } catch (Exception e) {
            log.error(e.getLocalizedMessage(), e);
            return null;
        }
    }

    public Set getFileNames() {
        if (fileCache == null)
            this.createFileCache();

        try {
            Set result = new HashSet();
            Iterator itCurFile = fileCache.keySet().iterator();
            while (itCurFile.hasNext()) {
                File curFile = (File) itCurFile.next();
                result.add(curFile.getName());
            }
            return result;
        } catch (Exception e) {
            log.error(e.getLocalizedMessage(), e);
            return null;
        }
    }

    public Set getFileURLs() {

        if (fileCache == null)
            this.createFileCache();

        try {
            Set result = new HashSet();
            Iterator itCurFile = fileCache.keySet().iterator();
            while (itCurFile.hasNext()) {
                File curFile = (File) itCurFile.next();
                result.add(curFile.toURL());
            }
            return result;
        } catch (Exception e) {
            log.error(e.getLocalizedMessage(), e);
            return null;
        }
    }

    public String getRegexFileFilter() {
        return regexFileFilter;
    }

    public void setRegexFileFilter(String regexFileFilter) {
        this.regexFileFilter = regexFileFilter;
        fileCache = null;
    }

}
