package application.startup;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

public class FileFinder {

    public static String[] getFileNames(File directory, String filterRegexString) {

        List files = getFiles(directory, filterRegexString, false);

        if (files != null) {
            List fileNames = new ArrayList(files.size());
            Iterator itFile = files.iterator();
            while (itFile.hasNext()) {
                File file = (File) itFile.next();
                fileNames.add(file.getAbsolutePath());
            }
            if (fileNames.size() > 0)
                return (String[]) fileNames.toArray(new String[0]);
            else
                return new String[0];
        }
        return new String[0];
    }

    public static List getFiles(File directory, String filterRegexString) {
        return getFiles(directory, filterRegexString, false);
    }

    public static List getURLs(File directory, String filterRegexString) {
        return getFiles(directory, filterRegexString, true);
    }

    private static List getFiles(File directory, String filterRegexString, boolean urlObject) {

        if (directory == null || filterRegexString == null)
            return null;

        if (!directory.exists() || !directory.isDirectory())
            return null;

        try {
            Pattern pt = Pattern.compile(filterRegexString);
            List fileList = new ArrayList();
            File[] files = directory.listFiles();
            for (int index = 0; index < files.length; index++) {
                if (pt.matcher(files[index].getName()).matches()) {
                    if (urlObject)
                        fileList.add(files[index].toURL());
                    else
                        fileList.add(files[index]);
                }
            }
            return fileList;
        } catch (Exception e) {
            System.out.println(e.getLocalizedMessage());
            return null;
        }
    }

    public static List splitFilePath(String filepath, String separator) {

        if (filepath == null || separator == null)
            return null;

        StringTokenizer token = new StringTokenizer(filepath, separator);

        try {
            List pathList = new ArrayList();
            while (token.hasMoreTokens()) {

                File file = new File(token.nextToken());
                if (file.exists()) {
                    File target = file.getCanonicalFile();

                    if (!pathList.contains(target)) {
                        pathList.add(target);
                    }
                }
            }

            return pathList;

        } catch (Exception e) {
            System.out.println(e.getLocalizedMessage());
            return null;
        }

    }

}
