package application.startup;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class SystemProps {

	static public final String PROPS_SYSTEM_PROPERTIES_XML = "system.properties.xml";

	static public final String DEFAULT_SYSTEM_PROPERTIES_XML2 = "etc/default.system.properties.xml";

	static public String  DEFAULT_SYSTEM_PROPERTIES_XML = DEFAULT_SYSTEM_PROPERTIES_XML2;
	
	/**
	 * Application Properties
	 */
	static public final String PROPS_APPLICATION_NAME = "application.name";

	static public final String PROPS_APPLICATION_DESC = "application.description";

	static public final String PROPS_APPLICATION_MAIN_CLASS = "application.main.class";

	static public final String PROPS_APPLICATION_HOME = "application.home";

	static public final String PROPS_APPLICATION_START_CLASS_METHOD = "application.start.class.method";

	static public final String PROPS_APPLICATION_SHUTDOWN_CLASS_METHOD = "application.shutdown.class.method";

	static public final String DEFAULT_APPLICATION_START_CLASS_METHOD = "main";

	static public final String DEFAULT_APPLICATION_SHUTDOWN_CLASS_METHOD = "shutdown";

	static public final String DEFAULT_APPLICATION_HOME = ".";

	/**
	 * Environment Variables
	 */
	static public String PROPS_ENV_JAVA_HOME = "JAVA_HOME";

	static public String PROPS_ENV_APPLICATION_HOME = "APPLICATION_HOME";

	static public String DEFAULT_ENV_APPLICATION_HOME = ".";

	/**
	 * System Properties
	 */

	static public final String PROPS_SYS_ETC_PATH = "system.etc.path";

	static public final String PROPS_SYS_SERVER_MODE = "system.server.mode";

	static public final String PROPS_SYS_SILENT_MODE = "system.silent.mode";

	static public final String PROPS_SYS_CONTEXT_CONFIG_XML = "system.context.cfg.xml";

	static public final String PROPS_SYS_INTERNAL_ENCODING = "system.internal.encoding";

	static public final String PROPS_SYS_OS_ENCODING = "system.os.encoding";

	static public final String PROPS_SYS_JAVA_HOME = "system.java.home";

	static public final String DEFAULT_SYS_ETC_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/etc";

	static public final String DEFAULT_SYS_SILENT_MODE = "false";

	static public final String DEFAULT_SYS_SERVER_MODE = "false";

	static public final String DEFAULT_SYS_CONTEXT_CONFIG_XML = DEFAULT_SYS_ETC_PATH
			+ "/system.context.xml";

	static public final String DEFAULT_SYS_ENCODING_INTERNAL = "UTF-8";

	static public final String DEFAULT_SYS_ENCODING_NATIVE = new java.io.OutputStreamWriter(
			new java.io.ByteArrayOutputStream()).getEncoding();

	static public final String DEFAULT_SYS_JAVA_HOME = DEFAULT_ENV_APPLICATION_HOME
			+ "/jre";

	/**
	 * Components
	 */
	static public final String PROPS_SYS_COMPONENT_PATH = "system.component.path";

	static public final String PROPS_SYS_COMPONENT_LIBDIR_NAME = "system.component.libdir.name";

	static public final String PROPS_SYS_COMPONENT_CONFIGDIR_NAME = "system.component.cfgdir.name";

	static public final String PROPS_SYS_COMPONENT_CONFIG_NAME = "system.component.cfg.name";

	static public final String PROPS_SYS_COMPONENT_WORK_PATH = "system.component.work.path";

	static public final String DEFAULT_SYS_COMPONENT_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/components";

	static public final String DEFAULT_SYS_COMPONENT_LIBDIR_NAME = "lib";

	static public final String DEFAULT_SYS_COMPONENT_CONFIGDIR_NAME = "etc";

	static public final String DEFAULT_SYS_COMPONENT_CONFIG_NAME = "component.cfg.xml";

	static public final String DEFAULT_SYS_COMPONENT_WORK_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/components";
	// hayashi
	static public final String DEFAULT_DEBUG_MODE = "false";

	/**
	 * System Java Classes
	 */
	static public final String PROPS_SYS_LIB_PATH = "system.lib.path";

	static public final String PROPS_SYS_CLASSES_PATH = "system.classes.path";

	static public final String DEFAULT_SYS_LIB_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/lib";

	static public final String DEFAULT_SYS_CLASSES_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/classes";

	/**
	 * User application
	 */
	static public final String PROPS_USER_BEANS_PATH = "system.user.beans.path";

	static public final String PROPS_USER_LIB_PATH = "system.user.lib.path";

	static public final String PROPS_USER_CLASSES_PATH = "system.user.classes.path";

	static public final String PROPS_USER_PROPERTIES_PATH = "system.user.properties.path";

	static public final String DEFAULT_USER_BEANS_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/beans";

	static public final String DEFAULT_USER_LIB_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/lib";;

	static public final String DEFAULT_USER_CLASSES_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/classes";

	static public final String DEFAULT_USER_PROPERTIES_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/sysprops";

	/**
	 * Work
	 */
	static public final String PROPS_SYS_WORK_PATH = "system.work.path";

	static public final String PROPS_SYS_WORK_LOG_PATH = "system.work.log.path";

	static public final String DEFAULT_SYS_WORK_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work";

	static public final String DEFAULT_SYS_WORK_LOG_PATH = DEFAULT_ENV_APPLICATION_HOME
			+ "/work/log";

	/**
	 * システムプロパティのデフォルト値の設定
	 * 
	 */
	static public void applyDefaultSystemProperties() {

		String appHome = System.getProperty("application.home");
		if(appHome != null)
			DEFAULT_SYSTEM_PROPERTIES_XML = appHome + "/" + DEFAULT_SYSTEM_PROPERTIES_XML;

		// application properties
		// System.setProperty(PROPS_APPLICATION_HOME, DEFAULT_APPLICATION_HOME);
		// System.setProperty(PROPS_SYS_JAVA_HOME, DEFAULT_SYS_JAVA_HOME);
		System.setProperty(PROPS_APPLICATION_START_CLASS_METHOD,
				DEFAULT_APPLICATION_START_CLASS_METHOD);
		System.setProperty(PROPS_APPLICATION_SHUTDOWN_CLASS_METHOD,
				DEFAULT_APPLICATION_SHUTDOWN_CLASS_METHOD);

		// system properties
		System.setProperty(PROPS_SYS_ETC_PATH, DEFAULT_SYS_ETC_PATH);
		System.setProperty(PROPS_SYS_ETC_PATH, DEFAULT_SYS_ETC_PATH);
		System.setProperty(PROPS_SYS_SILENT_MODE, DEFAULT_SYS_SILENT_MODE);
		System.setProperty(PROPS_SYS_SERVER_MODE, DEFAULT_SYS_SERVER_MODE);
		System.setProperty(PROPS_SYS_CONTEXT_CONFIG_XML,
				DEFAULT_SYS_CONTEXT_CONFIG_XML);
		System.setProperty(PROPS_SYS_INTERNAL_ENCODING,
				DEFAULT_SYS_ENCODING_INTERNAL);
		System.setProperty(PROPS_SYS_OS_ENCODING, DEFAULT_SYS_ENCODING_NATIVE);

		// components
		System
				.setProperty(PROPS_SYS_COMPONENT_PATH,
						DEFAULT_SYS_COMPONENT_PATH);
		System.setProperty(PROPS_SYS_COMPONENT_CONFIG_NAME,
				DEFAULT_SYS_COMPONENT_CONFIG_NAME);
		System.setProperty(PROPS_SYS_COMPONENT_CONFIGDIR_NAME,
				DEFAULT_SYS_COMPONENT_CONFIGDIR_NAME);
		System.setProperty(PROPS_SYS_COMPONENT_LIBDIR_NAME,
				DEFAULT_SYS_COMPONENT_LIBDIR_NAME);
		System.setProperty(PROPS_SYS_COMPONENT_WORK_PATH,
				DEFAULT_SYS_COMPONENT_WORK_PATH);

		// system java classes
		System.setProperty(PROPS_SYS_LIB_PATH, DEFAULT_SYS_LIB_PATH);
		System.setProperty(PROPS_SYS_CLASSES_PATH, DEFAULT_SYS_CLASSES_PATH);

		// user application
		System.setProperty(PROPS_USER_BEANS_PATH, DEFAULT_USER_BEANS_PATH);
		System.setProperty(PROPS_USER_LIB_PATH, DEFAULT_USER_LIB_PATH);
		System.setProperty(PROPS_USER_CLASSES_PATH, DEFAULT_USER_CLASSES_PATH);
		System.setProperty(PROPS_USER_PROPERTIES_PATH,
				DEFAULT_USER_PROPERTIES_PATH);

		// work
		System.setProperty(PROPS_SYS_WORK_PATH, DEFAULT_SYS_WORK_PATH);
		System.setProperty(PROPS_SYS_WORK_LOG_PATH, DEFAULT_SYS_WORK_LOG_PATH);

	}

	/**
	 * プロパティ値を system.properties.xmlから読み込み、システムプロパティに設定する。
	 * system.properties.xmlはクラスパスから検索する。
	 * 
	 * @return 読み込んだプロパティファイルにより生成されたプロパティオブジェクト
	 */
	static public Properties loadSystemProperties() {

		try {
			applyDefaultSystemProperties();

			Properties props = new Properties();

			// retrieve the app.home
			//String home = getHome();

			// load default system properties
			File defaultSyspropsXmlFile = new File(
					DEFAULT_SYSTEM_PROPERTIES_XML);
			if (defaultSyspropsXmlFile.exists()
					&& defaultSyspropsXmlFile.isFile()) {
				Properties defaultProps = new Properties();
				loadPropsFromXml(defaultProps, DEFAULT_SYSTEM_PROPERTIES_XML);

				// replace the variables to values
				Iterator itKey = defaultProps.keySet().iterator();
				while (itKey.hasNext()) {
					Object key = itKey.next();
					String source = defaultProps.getProperty(key.toString());
					String result = compile(source);
					System.setProperty(key.toString(), result);
					props.setProperty(key.toString(), result);
				}
			}

			// loading the system.properties.xml
			File syspropsXmlFile = null;
			if (System.getProperties().containsKey(PROPS_SYSTEM_PROPERTIES_XML)) {
				syspropsXmlFile = new File(System
						.getProperty(PROPS_SYSTEM_PROPERTIES_XML));
			}

			if (syspropsXmlFile != null && syspropsXmlFile.isFile()) {
				Properties systemProps = new Properties();
				loadPropsFromXml(systemProps, syspropsXmlFile.getAbsolutePath());

				// Replacing the variables to values
				Iterator itKey = systemProps.keySet().iterator();
				while (itKey.hasNext()) {
					Object key = itKey.next();
					String source = systemProps.getProperty(key.toString());
					String result = compile(source);
					System.setProperty(key.toString(), result);
					props.setProperty(key.toString(), result);
				}

			}

			// loading user properties
			File userPropsDir;
			if (System.getProperties().containsKey(PROPS_USER_PROPERTIES_PATH)) {
				userPropsDir = new File(System
						.getProperty(PROPS_USER_PROPERTIES_PATH));

				if (userPropsDir.exists() && userPropsDir.isDirectory()) {
					String[] propFileNames = FileFinder.getFileNames(
							userPropsDir, ".*\\.xml");
					for (int curIndex = 0; curIndex < propFileNames.length; curIndex++) {
						// add user properties to system properties
						Properties curProps = new Properties();
						// shumi JAVA1.4対応
						loadPropsFromXml(curProps, propFileNames[curIndex]);
						Iterator iKeys = curProps.keySet().iterator();
						while (iKeys.hasNext()) {
							Object key = iKeys.next();
							String source = curProps
									.getProperty(key.toString());
							String result = compile(source);
							System.setProperty(key.toString(), result);
							props.setProperty(key.toString(), result);
						}

					}
				}
			}
			return props;

		} catch (Exception e) {
			System.out.println(e.getLocalizedMessage());
			return null;
		}
	}

	/**
	 * システムプロパティ及び環境変数を参照し、JAVA_HOME を返す。
	 * システムプロパティ及び環境変数の両方にJAVA_HOMEが設定されている場合は、 システムプロパティを優先する。
	 * 
	 * @return JAVA_HOMEのパス。JAVA_HOMEが指定されていなければnullを返す。
	 */
	public static String getJavaHome() {

		try {
			String javaHome = null;

			if (System.getProperties().containsKey(
					SystemProps.PROPS_SYS_JAVA_HOME))
				javaHome = System.getProperty(SystemProps.PROPS_SYS_JAVA_HOME);
			else {
				//javaHome = System.getenv(SystemProps.PROPS_ENV_JAVA_HOME);
				if (javaHome == null) {
					javaHome = System.getProperty("java.home");
				}
			}

			if (javaHome != null) {
				System.setProperty(SystemProps.PROPS_SYS_JAVA_HOME, javaHome);
				File homeFile = new File(javaHome);
				if (homeFile.exists())
					System.setProperty(SystemProps.PROPS_SYS_JAVA_HOME,
							homeFile.getCanonicalPath());
			}
			return javaHome;

		} catch (IOException e) {
			return null;
		}

	}

	/**
	 * システムプロパティ及び環境変数を参照し、アプリケーションのHOMEを返す。
	 * システムプロパティ及び環境変数の両方に値が設定されている場合は、システムプロパティを優先する。
	 * 
	 * @return HOMEのパス。指定されていなければnullを返す。
	 */
	public static String getHome() {

		try {
			String home = null;
			if (System.getProperties().containsKey(
					SystemProps.PROPS_APPLICATION_HOME))
				home = System.getProperty(SystemProps.PROPS_APPLICATION_HOME);
			else {
				home = System.getenv(SystemProps.PROPS_ENV_APPLICATION_HOME);
				if (home == null)
					home = ".";
			}

			if (home != null) {
				System.setProperty(SystemProps.PROPS_APPLICATION_HOME, home);
				File homeFile = new File(home);
				if (homeFile.exists())
					System.setProperty(SystemProps.PROPS_APPLICATION_HOME,
							homeFile.getCanonicalPath());
			}
			return home;
		} catch (IOException e) {
			return null;
		}

	}

	/**
	 * システムプロパティ変数名を実際の値に置き換えた文字列を返す
	 * 
	 * @param source
	 *            ${変数名}を含む文字列
	 * @return 置換後の文字列
	 */
	static public String compile(String source) {

		// Replacing the variables to values
		Pattern ptVariable = Pattern.compile("\\$\\{[0-9a-zA-Z.]*\\}");
		Matcher matcher = ptVariable.matcher(source);
		StringBuffer resultBuffer = new StringBuffer();
		while (matcher.find()) {
			String variable = matcher.group();
			variable = variable.substring(2, variable.length() - 1);
			String propValue = System.getProperty(variable);
			if (propValue == null)
				propValue = System.getenv(variable);
			if (propValue != null)
				propValue = propValue.replace('\\', '/');

			matcher.appendReplacement(resultBuffer, propValue);

		}
		matcher.appendTail(resultBuffer);
		return resultBuffer.toString();

	}

	static public String get(String key, String defaultValue) {
		return System.getProperty(key, defaultValue);
	}

	static public boolean loadPropsFromXml(Properties props, String file) {
		boolean rc = false;
		props.clear();
		
		try {
			DOMParser parser = new DOMParser();
			parser.setEntityResolver(new EntityResolver() {
				public InputSource resolveEntity(java.lang.String publicId,
						java.lang.String systemId) throws SAXException,
						java.io.IOException {
					if("http://java.sun.com/dtd/properties.dtd".equals(systemId))
					{
						return new InputSource(new ByteArrayInputStream("<?xml version='1.0' encoding='UTF-8'?>".getBytes()));
					}
					return null;
				}
			});

			FileInputStream fis = new FileInputStream(file);
			parser.parse(new InputSource(fis));
			Document doc = parser.getDocument();
			Element root = doc.getDocumentElement();
			if (root.getTagName().equals("properties")) {
				NodeList nodeList = root.getElementsByTagName("entry");
				for (int cnt = 0; cnt < nodeList.getLength(); cnt++) {
					Element em = (Element) nodeList.item(cnt);
					String name = em.getAttribute("key");
					String value = "";
					NodeList childNodeList = em.getChildNodes();
					if ((childNodeList != null)
							&& (childNodeList.getLength() > 0)) {
						Node valueNode = childNodeList.item(0);
						value = valueNode.getNodeValue();
					}
					props.setProperty(name, value);

				}
			}
		} catch (Exception e) {
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		changeClassPathProp(props, PROPS_SYS_LIB_PATH);
		changeClassPathProp(props, PROPS_SYS_CLASSES_PATH);
		changeClassPathProp(props, PROPS_USER_LIB_PATH);
		changeClassPathProp(props, PROPS_USER_CLASSES_PATH);
		return rc;
	}
	
	
	static public boolean changeClassPathProp(Properties props, String propName)
	{
		boolean rc = false;
	
		String path = props.getProperty(propName);
		if(path != null)
		{
			path = path.replaceAll(":", File.pathSeparator);
			path = path.replaceAll(";", File.pathSeparator);
			props.setProperty(propName, path);
		}
		return rc;
	}
}
