package daruma.bus.message;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import daruma.bus.processor.Processor;
import daruma.bus.processor.ProcessorException;



public class StandardMessage implements daruma.bus.message.Message {

    private Payload messageData;
    private List processorList = new ArrayList();
    private Properties properties = new Properties();
    private boolean verboseMode = false;
    
    public List getProcessorList() {
        return processorList;
    }

    public void setProcessorList(List processorList) {
        this.processorList = processorList;
    }

    public daruma.bus.message.Payload getPayload() {
        return messageData;
    }

    public void setPayload(Payload messageData) {
        this.messageData = messageData;
    }

    /**
     * ストリームからバイトデータを所得する
     * @param in
     * @return
     * @throws CsvMessageException
     */
    public byte[] getValueFromStream(InputStream in) throws MessageException{

        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            int len;
        	byte[] buf = new byte[8192];
            while((len=in.read(buf))!=-1){
                out.write(buf, 0, len);
            }
            out.flush();
            return out.toByteArray();
        } catch (Exception e) {
            MessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }
    
    /**
     * ファイルからバイトデータを取得する。
     * @param file
     * @return
     * @throws CsvMessageException
     */
    public byte[] getValueFromFile(File file)
            throws MessageException {
        try {
            FileInputStream fileIn = new FileInputStream(file);
            FileChannel fc = fileIn.getChannel();
            ByteBuffer buffer = ByteBuffer.allocate((int) fc.size());
            fc.read(buffer);
            fileIn.close();
            buffer.flip();
            return buffer.array();
        } catch (Exception e) {
            MessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }



    public void send() throws ProcessorException {
        
        if(this.processorList!=null){
            Iterator itStep = processorList.iterator();
            while(itStep.hasNext()){
                Processor step = (Processor)itStep.next();
                Payload payload = this.getPayload();
                step.process(this);
                if((!verboseMode) && payload!=null){
                    if(!payload.equals(this.getPayload()))
                        payload.dispose();
                }
            }
        }
     
        
    }

    public Properties getProperties() {
       return properties;
    }

    public void setProperties(Properties properties) {
      this.properties = properties;
        
    }

    public boolean isVerboseMode() {
        return verboseMode;
    }

    public void setVerboseMode(boolean verboseMode) {
        this.verboseMode = verboseMode;
    }
    
}
