package daruma.bus.message;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import daruma.common.CommonUtil;

import application.startup.SystemProps;


/**
 * ファイルベースの標準メッセージクラス
 * 
 * @author masanori.matsuoka@netconscious.jp
 * 
 */
public class StandardPayload implements FileBasedPayload {

    static private Log log = LogFactory.getLog(StandardPayload.class);
    protected File workDir = new File(SystemProps.get(SystemProps.PROPS_SYS_WORK_PATH, "./work"));
    protected File sourceFile;

    /**
     * 外部ファイルをメッセージ本体として設定する。
     * 
     * @param file
     *            アタッチするファイル
     */
    public void attach(File file) {
        this.sourceFile = file;
    }

    public File detach() {
        File file = this.sourceFile;
        this.sourceFile = null;
        return file;
    }

    public void dispose() {
        if (sourceFile != null && sourceFile.exists())
            if (!this.sourceFile.delete())
                this.sourceFile.deleteOnExit();
    }

    public FileOutputStream getFileOutputStream() throws MessageException {

        if (this.sourceFile == null) {
            MessageException e = new StandardMessageException("Message body is null.");
            e.fillInStackTrace();
            throw e;
        }

        try {
            FileOutputStream fileout = new FileOutputStream(this.sourceFile);
            return fileout;

        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            MessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }

    }

    public OutputStream getOutputStream() throws MessageException {

        return getFileOutputStream();

    }

    public FileInputStream getFileInputStream() throws MessageException {

        if (this.sourceFile == null) {
            MessageException e = new StandardMessageException("Message body is null.");
            e.fillInStackTrace();
            throw e;
        }
        try {
            FileInputStream fileIn = new FileInputStream(this.sourceFile);
            return fileIn;

        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            MessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

    public InputStream getInputStream() throws MessageException {

        return getFileInputStream();
    }

    public File getWorkDir() {
        return workDir;
    }

    public void setWorkDir(File workDir) {
        this.workDir = workDir;
    }

    public void load(File file) throws MessageException {

        try {
            File workFile = File.createTempFile(StandardPayload.class.getName(), ".tmp", this.workDir);
            InputStream in = new BufferedInputStream(new FileInputStream(file));
            OutputStream out = new BufferedOutputStream(new FileOutputStream(workFile));
            int len;
            byte[] buf = new byte[8192];
            while ((len = in.read(buf)) != -1) {
                out.write(buf, 0, len);
            }
            in.close();
            out.flush();
            out.close();
            this.attach(workFile);

        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            StandardMessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }

    }

    public void save(File file) throws MessageException {
        try {
        	InputStream is = new BufferedInputStream(new FileInputStream(this.sourceFile));
            OutputStream os = new FileOutputStream(file);
        	CommonUtil.save(is, os);
        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            StandardMessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }
    
    public void save(File file, String inputEncoding, String outputEncoding) throws MessageException {
        try {
        	InputStream in = new BufferedInputStream(new FileInputStream(this.sourceFile));
            OutputStream os = new FileOutputStream(file);
            CommonUtil.save(in, os, inputEncoding, outputEncoding);
        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            StandardMessageException ex = new StandardMessageException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

}
