package daruma.bus.processor;

import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.startup.SystemProps;

import daruma.bus.message.Message;

public class ConsoleWriter implements Processor {

    static private Log log = LogFactory.getLog(ConsoleWriter.class);

    static public final String PROPS_INPUT_ENCODING = "consolewriter.input.encoding";
    static public final String PROPS_OUTPUT_ENCODING = "consolewriter.output.encoding";
    
    static public final String DEFAULT_INPUT_ENCODING = SystemProps.get(PROPS_INPUT_ENCODING,
            System.getProperty(SystemProps.PROPS_SYS_OS_ENCODING));
    static public final String DEFAULT_OUTPUT_ENCODING = SystemProps.get(PROPS_OUTPUT_ENCODING,
            System.getProperty(SystemProps.PROPS_SYS_OS_ENCODING));
    
    private String intputEncoding = DEFAULT_INPUT_ENCODING;
    private String outputEncoding = DEFAULT_OUTPUT_ENCODING;
    
    public void process(Message message) throws ProcessorException {
       
        try {
            String intputEncoding = this.intputEncoding;
            String outputEncoding = this.outputEncoding;
        
        if(message.getProperties().containsKey(PROPS_INPUT_ENCODING))
            outputEncoding = message.getProperties().getProperty(PROPS_INPUT_ENCODING);
        
        if(message.getProperties().containsKey(PROPS_OUTPUT_ENCODING))
            outputEncoding = message.getProperties().getProperty(PROPS_OUTPUT_ENCODING);
        
        InputStreamReader in = new InputStreamReader( message.getPayload().getInputStream(),intputEncoding);
        OutputStreamWriter out = new OutputStreamWriter(System.out,outputEncoding);
        
        int len;
    	char[] buf = new char[1024];
        while((len=in.read(buf))!=-1)
            out.write(buf, 0, len);
        out.flush();
        //out.close();
        in.close();
        
      
        
        }
        catch(Exception e){
            log.debug(e.getMessage(),e);
            ProcessorException ex = new ProcessorException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

}
