package daruma.bus.processor;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.startup.SystemProps;

import daruma.bus.message.Message;
import daruma.common.CommonUtil;

public class FileWriter implements Processor {

    static private Log log = LogFactory.getLog(FileWriter.class);

    static public final String PROPS_INPUT_ENCODING = "filewriter.input.encoding";
    static public final String PROPS_OUTPUT_ENCODING = "filewriter.output.encoding";
    static public final String PROPS_OUTPUT_PATH = "filewriter.output.path";
    static public final String VALUE_OUTPUT_PATH_STDOUT = "filewriter.value.output.path.stdout";

    static public final String DEFAULT_OUTPUT_ENCODING = SystemProps.get(PROPS_OUTPUT_ENCODING, System
            .getProperty(SystemProps.PROPS_SYS_OS_ENCODING));
    static public final String DEFAULT_INPUT_ENCODING = SystemProps.get(PROPS_INPUT_ENCODING, System
            .getProperty(SystemProps.PROPS_SYS_OS_ENCODING));
    static public final String DEFAULT_OUTPUT_PATH = SystemProps.get(PROPS_OUTPUT_PATH, VALUE_OUTPUT_PATH_STDOUT);

    private String inputEncoding = DEFAULT_INPUT_ENCODING;
    private String outputEncoding = DEFAULT_OUTPUT_ENCODING;
    private String outputPath = DEFAULT_OUTPUT_PATH;

    private File workDir = new File(SystemProps.get(SystemProps.PROPS_SYS_WORK_PATH, "./work"));

    public void process(Message message) throws ProcessorException {

        try {
            String inputEncoding = this.inputEncoding;
            String outputEncoding = this.outputEncoding;
            String outputPath = this.outputPath;

            if (message.getProperties().containsKey(PROPS_INPUT_ENCODING))
                inputEncoding = message.getProperties().getProperty(PROPS_INPUT_ENCODING);
            if (message.getProperties().containsKey(PROPS_OUTPUT_ENCODING))
                outputEncoding = message.getProperties().getProperty(PROPS_OUTPUT_ENCODING);
            if (message.getProperties().containsKey(PROPS_OUTPUT_PATH))
                outputPath = message.getProperties().getProperty(PROPS_OUTPUT_PATH);

            InputStream filein = message.getPayload().getInputStream();
            if (outputPath.equals(VALUE_OUTPUT_PATH_STDOUT)) {
            	CommonUtil.save(filein, System.out, inputEncoding, outputEncoding);
            }
            else {
                File resultFile = new File(outputPath);
                resultFile.delete();
                if (resultFile.createNewFile()) {
                    OutputStream fos = new  FileOutputStream(resultFile);
                	CommonUtil.save(filein, fos, inputEncoding, outputEncoding);
                }
                else {
                    ProcessorException ex = new ProcessorException("An output file was not created.");
                    ex.fillInStackTrace();
                }
            }
        } catch (Exception e) {
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
            log.debug(e.getMessage(), e);
            ProcessorException ex = new ProcessorException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

}
