package daruma.bus.processor.misp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.startup.SystemProps;

import daruma.bus.message.Message;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.Processor;
import daruma.bus.processor.ProcessorException;

public class GetFeatureFilterReader implements Processor {

    static private Log log = LogFactory.getLog(GetFeatureFilterReader.class);

    static public final String PROPS_FILTER_TYPE = "mispsoapclient.filter.type";

    static public final String FILTERTYPE_XML = "mispsoapclient.filter.xml";
    static public final String FILTERTYPE_S_EXPRESSION = "mispsoapclient.filter.s.expression";

    private File workDir = new File(SystemProps.get(SystemProps.PROPS_SYS_WORK_PATH, "./work"));

    public void process(Message message) throws ProcessorException {

        try {
            Properties props = message.getProperties();

            if (props.containsKey(PROPS_FILTER_TYPE)) {
                if (props.getProperty(PROPS_FILTER_TYPE).equals(FILTERTYPE_XML)) {
                    File file = File.createTempFile(GetFeatureFilterReader.class.getName(), ".xml", this.workDir);
                    InputStream in = new BufferedInputStream(message.getPayload().getInputStream());
                    OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
                    int len;
                	byte[] buf = new byte[8192];
                    while ((len = in.read(buf)) != -1)
                        out.write(buf, 0, len);
                    out.flush();
                    out.close();
                    in.close();
                    StandardPayload payload = new StandardPayload();
                    payload.attach(file);
                    message.setPayload(payload);
                }
            }
        } catch (Exception e) {
            log.debug(e.getMessage(), e);
            ProcessorException ex = new MispSoapException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

    public File getWorkDir() {
        return workDir;
    }

    public void setWorkDir(File workDir) {
        this.workDir = workDir;
    }

}
