package daruma.bus.processor.misp;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import daruma.bus.processor.csv.Const;

public class GetTagValueFilter {
	/**
	 * ログ
	 */
	static private Log log = LogFactory.getLog(GetTagValueFilter.class);

	/**
	 * ターゲット名前空間
	 */
	private String filterTargetNameSpace = "";

	/**
	 * ターゲットタグ名
	 */
	private String filterTargetTagName = "";

	/**
	 * ターゲット名前空間
	 */
	private String filterGetNameSpace = "";

	/**
	 * ターゲットタグ名
	 */
	private String filterGetTagName = "";

	/**
	 * ターゲットレベル
	 */
	private int filterTargetLevel = 0;

	public GetTagValueFilter() {
	}

	public GetTagValueFilter(String nameSpace, String tagName) {
		this.filterGetNameSpace = nameSpace;
		this.filterGetTagName = tagName;
	}

	public String getTagValue(InputStream in){
		return getTagValue(in, 0);
	}
	
	public String getTagValue(InputStream in, int index){
		String val = null;
		List tagValList = getTagValues(in, 1);
		if(tagValList != null)
		{
			if(index < 0)
				index = tagValList.size() + index;
			if((index >= 0) && (index < tagValList.size()))
			{
				val= (String) tagValList.get(index);
			}
			tagValList.clear();
		}
		return val;
	}
	
	public List getTagValues(InputStream in){
		return getTagValues(in, -1);
	}

	private List getTagValues(InputStream in, int maxNum){
		List tagValList = new ArrayList();

		try {
			XMLReader parser = getXMLReader();
			GenericHandler handler = new GenericHandler(tagValList, maxNum);
			parser.setContentHandler(handler);
			parser.parse(new InputSource(in));
		} catch (Exception e) {
			log.debug(e.getMessage(), e);
		}
		return tagValList;
	}

	class GenericHandler extends DefaultHandler {
		private int maxNum; 
		private List tagValList;


		public GenericHandler(List tagValList, int maxNum) {
			super();
			this.tagValList = tagValList;
			this.maxNum = maxNum;
		}

		private int currTotalTagLevel = -1;

		private int currTotalTagIndex = 0;

		private int currTargetTagLevel = -1;

		private int currTargetTagIndex = 0;

		private int currLevel = -1;

		private int currIndex = 0;

		public void startElement(String namespaceURI, String localName,
				String qName, Attributes atts) throws SAXException {
			currTotalTagLevel++;
			if (currTargetTagLevel < 0) {
				if ((filterTargetNameSpace.equals("") || namespaceURI
						.equals(filterTargetNameSpace))
						&& (filterTargetTagName.equals("") || localName
								.equals(filterTargetTagName))) {
					currTargetTagLevel = 0;
					currTargetTagIndex = 0;
				}
			} else {
				currTargetTagLevel++;
			}

			if (currTargetTagLevel >= filterTargetLevel) {
				if (currTargetTagLevel == filterTargetLevel) {
					currLevel = 0;
					currIndex = 0;
				} else
					currLevel++;
				startElement2(namespaceURI, localName, qName, atts);
			} else {
				// outputStr("<" + qName + ">");
			}
		}

		public void characters(char[] ch, int start, int length)
				throws SAXException {
			currTotalTagIndex++;
			if (currTargetTagLevel >= 0) {
				currTargetTagIndex++;
				if (currTargetTagLevel >= filterTargetLevel) {
					currIndex++;
					characters2(ch, start, length);
				}
			} else {
				// outputChars(ch, start, length);
			}
		}

		public void endElement(String namespaceURI, String localName,
				String qName) throws SAXException {
			currTotalTagLevel--;

			if (currTargetTagLevel >= filterTargetLevel) {
				endElement2(namespaceURI, localName, qName);
				if (currLevel >= 0)
					currLevel--;
			} else {
				// outputStr("</" + qName + ">");
			}
			if (currTargetTagLevel >= 0)
				currTargetTagLevel--;
		}

		private String currTagVal = "";

		private boolean blOK = false;

		private void startElement2(String namespaceURI, String localName,
				String qName, Attributes atts) throws SAXException {
			if (namespaceURI.equals(filterGetNameSpace)
					&& localName.equals(filterGetTagName)) {
				blOK = true;
				currTagVal = "";
			}
		}

		private void characters2(char[] ch, int start, int length)
				throws SAXException {
			if (blOK) {
				currTagVal += new String(ch, start, length);
			}
		}

		private void endElement2(String namespaceURI, String localName,
				String qName) throws SAXException {
			if (blOK) {
				if((maxNum < 0) || (tagValList.size() < maxNum))
					tagValList.add(currTagVal);
				blOK = false;
			}
		}
	}

	private XMLReader getXMLReader() throws ParserConfigurationException,
			SAXException {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setFeature(Const.SAX_NS_URI, true);
		SAXParser parser = factory.newSAXParser();
		XMLReader xmlReader = parser.getXMLReader();
		xmlReader.setFeature(Const.SAX_NS_URI, true);
		return xmlReader;
	}
}
