package daruma.bus.processor.misp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.XMLFilterImpl;

import application.startup.SystemProps;

import daruma.bus.message.Message;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.Processor;
import daruma.bus.processor.ProcessorException;
import daruma.bus.processor.csv.Const;
import daruma.bus.processor.csv.MispSchemaUtil;
import daruma.bus.processor.csv.SortXmlTag;

/**
 * ターゲット要素の子要素をソートする <target> <level1> <level21></level21> </level1> <level1>
 * <level22></level22> </level1> </target>
 * 
 * <target> <level1> <level21></level21> <level22></level22> </level1>
 * </target>
 * 
 * @author shumi
 * 
 */
public class SortTagFilter implements Processor {
    /**
     * ログ
     */
    static private Log log = LogFactory.getLog(SortTagFilter.class);

    /**
     * ターゲット名前空間
     */
    private String targetNameSpace = "";

    /**
     * ターゲットタグ名
     */
    private String targetTagName = "";

    private File workDir = new File(SystemProps.get(SystemProps.PROPS_SYS_WORK_PATH, "./work"));

    public SortTagFilter(String nameSpace, String tagName) {
        this.targetNameSpace = nameSpace;
        this.targetTagName = tagName;
    }

    public void process(Message message) throws ProcessorException {
        if (message == null) {
            ProcessorException e = new BuiltinFilterException("A message is null.");
            e.fillInStackTrace();
            throw e;
        }

        try {
            File workFile = File.createTempFile(SortTagFilter.class.getName(), ".xml", workDir);
            OutputStream out = new BufferedOutputStream(new FileOutputStream(workFile));
            InputStream in = new BufferedInputStream(message.getPayload().getInputStream());
            transform(in, out);
            in.close();
            out.flush();
            out.close();

            StandardPayload payload = new StandardPayload();
            payload.attach(workFile);
            message.setPayload(payload);

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            BuiltinFilterException ex = new BuiltinFilterException(e.getMessage());
            ex.fillInStackTrace();
            throw ex;
        }
    }

    protected void transform(InputStream in, OutputStream out) throws ProcessorException {
    	SortXmlTag sortXmlTag = new SortXmlTag(targetNameSpace, targetTagName, "_");
    	sortXmlTag.transform(in, out);
    }
    
    protected void transform2(InputStream in, OutputStream out) throws ProcessorException {

    	try {
            ContentHandler contentHandler = (ContentHandler) getSerializer(out);
            XMLReader parser = getXMLReader();
            XMLFilter filter;
            filter = new SortTagFilterImpl(parser);
            filter.setContentHandler(contentHandler);
            filter.parse(new InputSource(in));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            ProcessorException exception = new BuiltinFilterException(e.getMessage());
            exception.setStackTrace(e.getStackTrace());
            throw exception;
        }

    }

    protected static class SortTagAttribute
    {
    	String name;
    	String value;
    	
    }
    
    protected static class SortTagElement implements Comparator {
        static int STS_NOT_OUTPUTED = 0;

        static int STS_OUTPUTED = 2;

        static int STS_OUTPUTING = 1;

        String tagText;

        long level;

        long index;

        StringBuffer buf;

        int sts;

        SortTagElement(String tagText, long level, long index, StringBuffer buf) {
            this.tagText = tagText;
            this.level = level;
            this.index = index;
            this.buf = buf;
            this.sts = STS_NOT_OUTPUTED;
        }

        public SortTagElement() {
        }

        public int compare(Object arg0, Object arg1) {
            int rc = 0;
            if ((arg0 instanceof SortTagElement) && (arg0 != null) && (arg1 instanceof SortTagElement) && (arg1 != null)) {
                SortTagElement element0 = (SortTagElement) arg0;
                SortTagElement element1 = (SortTagElement) arg1;
                rc = element0.tagText.compareTo(element1.tagText);
                if (rc == 0) {
                    if (element0.index > element1.index)
                        rc = 1;
                    else if (element0.index < element1.index)
                        rc = -1;
                }
            }
            return rc;
        }
    }

    class SortTagFilterImpl extends XMLFilterImpl {
        public SortTagFilterImpl(XMLReader parser) {
            super(parser);
        }

        private int currTagIndex = 0;

        private int currTagLevel = -1;

        private List currTagTree = new ArrayList();

        private List currTagDataList = new ArrayList();
        
        private Map currTagAttrList = new HashMap();

        private int currCharNum = 0;
        
        public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {

            ContentHandler handler = this.getContentHandler();
            if (handler == null)
                return;

            if (currTagLevel >= 0) {
                if (currTagLevel == 0) {
                    currTagIndex = 0;
                    currTagTree.clear();
                    currTagDataList.clear();
                    currTagAttrList.clear();
                }
                currTagTree.add(MispSchemaUtil.makeTagTreeText(namespaceURI, localName));
                String currTagTreeText = MispSchemaUtil.makeTagTreeText(currTagTree);
                AttributesImpl newAtts = null;
                if(currTagAttrList.get(currTagTreeText) == null)
                {
                	newAtts =  new AttributesImpl();
                	currTagAttrList.put(currTagTreeText, newAtts); 
                }
                else newAtts = (AttributesImpl)currTagAttrList.get(currTagTreeText); 
                for(int cnt = 0; cnt < atts.getLength(); cnt ++)
                {
                	int index = newAtts.getIndex(atts.getURI(cnt), atts.getLocalName(cnt));
                	if(index < 0)
                		newAtts.addAttribute(atts.getURI(cnt), atts.getLocalName(cnt), atts.getQName(cnt), atts.getType(cnt), atts.getValue(cnt));
                }
                currCharNum = 0;
                currTagLevel++;
            }
            else {
                if ((targetNameSpace.equals("") || namespaceURI.equals(targetNameSpace))
                        && (targetTagName.equals("") || localName.equals(targetTagName))) {
                    log.debug("Match TARGET XML TAG " + targetTagName);
                    currTagLevel = 0;                    
                }
                handler.startElement(namespaceURI, localName, qName, atts);
            }
        }

        public void characters(char[] ch, int start, int length) throws SAXException {
            ContentHandler handler = this.getContentHandler();
            if (handler == null)
                return;

            currTagIndex++;
            if (currTagLevel > 0) {
                StringBuffer buf = new StringBuffer();
                buf.append(ch, start, length);
                String currTagTreeText = MispSchemaUtil.makeTagTreeText(currTagTree);
                currTagDataList.add(new SortTagElement(currTagTreeText, currTagLevel, currTagIndex, buf));
                currCharNum++;
            }
            else {
                handler.characters(ch, start, length);
            }
        }

        public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
            ContentHandler handler = this.getContentHandler();
            if (handler == null)
                return;

            if (currTagLevel > 0) {
            	if(currCharNum == 0)
				{
					String currTagTreeText = MispSchemaUtil.makeTagTreeText(currTagTree);
					StringBuffer buf = new StringBuffer();
					currTagDataList.add(new SortTagElement(currTagTreeText,
							currTagLevel, currTagIndex, buf));
				}
                if (currTagTree.size() > 0)
                    currTagTree.remove(currTagTree.size() - 1);
            }

            if (currTagLevel == 1) {
                handleTagEnd(handler);
            }

            if (currTagLevel >= 0) {
                currTagLevel--;
            }

            if (currTagLevel < 0) {
                handler.endElement(namespaceURI, localName, qName);
            }

        }

        private void handleTagEnd(ContentHandler handler) throws SAXException {
            int currOutputLevel = 0;
            int currOutputIndex = 0;
            currOutputLevel = 0;
            for (currOutputIndex = 0; currOutputIndex < currTagDataList.size(); currOutputIndex++) {
                SortTagElement element = (SortTagElement) currTagDataList.get(currOutputIndex);
                outputElement(handler, element, currOutputLevel);
            }
            currTagTree.clear();
            currTagDataList.clear();
        }

        private void outputElement(ContentHandler handler, SortTagElement element, int currOutputLevel) throws SAXException {
            if (element.sts == SortTagElement.STS_NOT_OUTPUTED) {
                element.sts = SortTagElement.STS_OUTPUTING;
                String fullName = element.tagText;
                String[] tagTree = MispSchemaUtil.makeTagTreeList(fullName);
                String[] temp;
                for (int cnti = currOutputLevel; cnti < tagTree.length; cnti++) {
                    temp = tagTree[cnti].split(Const.INTERNAL_NAMESPACE_SEP);
                    String currTagText = MispSchemaUtil.makeTagTreeText(tagTree, cnti);
                    AttributesImpl atts ;
                    if(currTagAttrList.get(currTagText) == null)
                    	atts = new AttributesImpl();
                    else atts = (AttributesImpl)currTagAttrList.get(currTagText);
                    currTagAttrList.remove(currTagText);
                    handler.startElement(temp[0], temp[1], "", atts);
                }
                if (element.buf.length() > 0) {
                    char[] data = new String(element.buf).toCharArray();
                    handler.characters(data, 0, data.length);
                }
                for (int cnti = tagTree.length; cnti-- > currOutputLevel;) {
                    String currTagText = MispSchemaUtil.makeTagTreeText(tagTree, cnti);
                    for (int cntj = 0; cntj < currTagDataList.size(); cntj++) {
                        SortTagElement nextElement = (SortTagElement) currTagDataList.get(cntj);
                        if (nextElement.tagText.equals(currTagText)
                                || nextElement.tagText.startsWith(currTagText + Const.INTERNAL_TAG_SEP)) {
                            outputElement(handler, nextElement, cnti + 1);
                        }
                    }
                    temp = tagTree[cnti].split(Const.INTERNAL_NAMESPACE_SEP);
                    handler.endElement(temp[0], temp[1], "");
                }
                element.sts = SortTagElement.STS_OUTPUTED;
            }
        }
    }

    private XMLReader getXMLReader() throws ParserConfigurationException, SAXException {

        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setFeature(Const.SAX_NS_URI, true);
        SAXParser parser = factory.newSAXParser();
        XMLReader xmlReader = parser.getXMLReader();
        xmlReader.setFeature(Const.SAX_NS_URI, true);
        return xmlReader;
    }

    private XMLSerializer getSerializer(OutputStream out) {
        OutputFormat formatter = new OutputFormat();
        formatter.setPreserveSpace(true);
        XMLSerializer serializer = new XMLSerializer(out, formatter);
        return serializer;
    }

}
