package daruma.common;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import application.startup.SystemProps;
import daruma.bus.processor.csv.Const;

/* <<< [06/11/09 20:58 I.Noda] <<< */
import java.nio.CharBuffer ;
import java.io.FileReader ;
import java.io.FileNotFoundException ;
import java.io.IOException ;
/* >>> [06/11/09 20:58 I.Noda] >>> */

public class CommonUtil {
	/**
	 * バッファからファイルを生成する。
	 * 
	 * @param buffer
	 * @return ロードされたファイル
	 * @throws IOException
	 */
	public static File loadFileFromBuffer(byte[] buffer, File workDir)
			throws IOException {

		File file = File.createTempFile(CommonUtil.class.getName(), ".in",
				workDir);
		OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
		InputStream in = new ByteArrayInputStream(buffer);
		int len;
		byte[] buf = new byte[8192];
		while ((len = in.read(buf)) != -1) {
			out.write(buf, 0, len);
		}
		out.flush();
		out.close();
		return file;
	}

	/**
	 * Standard Input の内容をファイル化します。
	 * 
	 * @return ロードされたファイル
	 * @throws IOException
	 */
	public static File loadFileFromStdin(File workDir) throws IOException {

		File file = File.createTempFile(CommonUtil.class.getName(), ".in",
				workDir);
		OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
		int len;
		byte[] buf = new byte[8192];
		while ((len = System.in.read(buf)) != -1) {
			out.write(buf, 0, len);
		}
		out.flush();
		out.close();
		return file;
	}

	public static class FeatureName {
		private String targetNS;

		private String typeName;

		/**
		 * targetNSを取得する
		 * 
		 * @return Returns the targetNS.
		 */
		public String getTargetNS() {
			return targetNS;
		}

		/**
		 * targetNSをセットする
		 * 
		 * @param targetNS
		 *            The targetNS to set.
		 */
		public void setTargetNS(String targetNS) {
			this.targetNS = targetNS;
		}

		/**
		 * typeNameを取得する
		 * 
		 * @return Returns the typeName.
		 */
		public String getTypeName() {
			return typeName;
		}

		/**
		 * typeNameをセットする
		 * 
		 * @param typeName
		 *            The typeName to set.
		 */
		public void setTypeName(String typeName) {
			this.typeName = typeName;
		}
	}


	public static FeatureName getFeatureName(String name, String targetNS) {
		FeatureName featureName = new FeatureName();

		if ((name != null) && (name.length() > 0)) {
			Pattern pattern = Pattern.compile("^\\{(.+)\\}(.+)$");
			Matcher matcher = pattern.matcher(name);
			if (matcher.find()) {
				featureName.targetNS = matcher.group(1);
				featureName.typeName = matcher.group(2);
				return featureName;
			} else if ((targetNS != null) && (targetNS.length() > 0)) {
				featureName.targetNS = targetNS;
				featureName.typeName = name;
				return featureName;
			}
		}

		return null;
	}

       /* !!! [06/11/11 01:31 I.Noda] !!! */
       /** 
	* ファイルの内容を全て文字列に変換
	*/
       public static String readWholeFile(String filename) throws Exception {
	   /* !!! [06/11/11 01:31 I.Noda] !!! 
	    * move from CsvGetFeature.java
	    */
	   try {
	       File file = new File(filename) ;
	       long len = file.length() ;
	       /* !!! [06/11/09 20:58 I.Noda] !!! 
		* FIXME:
		* この部分、XML の文法チェックをすべきか？
		*/
	       CharBuffer buffer = CharBuffer.allocate((int)len) ;
	       FileReader reader =new FileReader(file) ;
	       reader.read(buffer) ;
	       buffer.rewind() ;
	       return buffer.toString() ;
	   } catch(FileNotFoundException ex) {
	       Exception newEx = new Exception(ex) ;
	       newEx.setStackTrace(ex.getStackTrace()) ;
	       throw newEx ;
	   } catch(IOException ex) {
	       Exception newEx = new Exception(ex) ;
	       newEx.setStackTrace(ex.getStackTrace()) ;
	       throw newEx ;
	   }
       }

	/**
	 * リクエスト用XMLのファイルを生成する
	 * 
	 * @param message
	 *            基となるMessageオブジェクト
	 * @param featureType
	 *            対象となるFeatureType
	 * @return ファイルを返す
	 * @throws IOException
	 */
	public static File createGetFeatureRequestXml(File workDir,
			FeatureName featureName, String currTransactionID, boolean blCount, String condStr)
			throws 	IOException {
		String internalCharset = System
				.getProperty(SystemProps.PROPS_SYS_INTERNAL_ENCODING);

		/* !!! [06/11/09 20:58 I.Noda] !!! 
		 * condStr が null の場合は、True が条件
		 */
		if(condStr == null) {
		    condStr = "<" + Const.NS_MISP_PRE + ":True/>" ;
		}
		
		String msg = "<?xml version='1.0' encoding='" + internalCharset + "'?>"
				+ "<" + Const.NS_MISP_PRE + ":GetFeature " + Const.NS_XMLNS_PRE
				+ ":" + Const.NS_MISP_PRE + "='" + Const.NS_MISP_URI + "' "
				+ Const.NS_XMLNS_PRE + "='" + featureName.getTargetNS() + "'>"
				+ "<" + Const.NS_MISP_PRE + ":Query typeName='"
				+ featureName.getTypeName() + "' mode='"
				+ (blCount ? "count" : "features") + "'>" + "<"
				+ Const.NS_MISP_PRE + ":Filter>";
		if (currTransactionID != null) {
			msg += "<" + Const.NS_MISP_PRE + ":And><" + Const.NS_MISP_PRE
					+ ":PropertyIsGreaterThan>";
			msg += "<" + Const.NS_MISP_PRE
					+ ":PropertyName>@_transaction_id_</" + Const.NS_MISP_PRE
					+ ":PropertyName>";
			msg += "<" + Const.NS_MISP_PRE + ":Literal>" + currTransactionID
					+ "</" + Const.NS_MISP_PRE + ":Literal>";
			msg += "</" + Const.NS_MISP_PRE + ":PropertyIsGreaterThan>";
			/* <<< [06/11/09 20:58 I.Noda] <<< */
			//msg += "<" + Const.NS_MISP_PRE + ":True/>";
			/* === [06/11/09 20:58 I.Noda] === */
			msg += condStr ;
			/* >>> [06/11/09 20:58 I.Noda] >>> */
			msg += "</" + Const.NS_MISP_PRE + ":And>";
		} else {
		        /* <<< [06/11/09 20:58 I.Noda] <<< */
			//msg += "<" + Const.NS_MISP_PRE + ":True/>";
		        /* === [06/11/09 20:58 I.Noda] === */
		        msg += condStr ;
		        /* >>> [06/11/09 20:58 I.Noda] >>> */
		}

		msg += "</" + Const.NS_MISP_PRE + ":Filter>" + "</" + Const.NS_MISP_PRE
				+ ":Query></" + Const.NS_MISP_PRE + ":GetFeature>";
		File file = CommonUtil.loadFileFromBuffer(
				msg.getBytes(internalCharset), workDir);
		return file;
	}

	public static void save(InputStream is, OutputStream os) throws IOException {
		OutputStream out = new BufferedOutputStream(os);
		byte[] buf = new byte[8192];
		int len;
		while ((len = is.read(buf)) != -1) {
			out.write(buf, 0, len);
		}
		is.close();
		out.flush();
		out.close();
	}

	public static void save(InputStream is, OutputStream os, String inputEncoding,
			String outputEncoding) throws IOException {
		InputStreamReader in = new InputStreamReader(is, inputEncoding);
		OutputStreamWriter out = new OutputStreamWriter(os, outputEncoding);
		int len;
		char[] buf = new char[8192];
		while ((len = in.read(buf)) != -1) {
			out.write(buf, 0, len);
		}
		in.close();
		out.flush();
		out.close();
	}

}
