package daruma.common;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class DbUtil {
	private static Logger log = LogManager.getLogger(DbUtil.class.getName());
	
	/**
	 * データベースに接続する
	 * 
	 * @param strJdbcDriver
	 *            JDBCドライバ
	 * @param strDbUrl
	 *            データベースURL
	 * @param strDbUser
	 *            データベースユーザー
	 * @param strDbPass
	 *            データベースパスワード
	 * @return 成功した場合はデータベースへのコネクション それ以外はNULLを返す
	 */
	public static Connection connectDb(String strJdbcDriver, String strDbUrl,
			String strDbUser, String strDbPass) {
		Connection conDb = null;
		try {
			Class.forName(strJdbcDriver);
		} catch (ClassNotFoundException ex) {
			log.error("Can't find JDBC driver:" + strJdbcDriver);
		}

		try {
			conDb = DriverManager.getConnection(strDbUrl, strDbUser, strDbPass);
			conDb.setAutoCommit(false);
		} catch (SQLException ex) {
			log.error("Can't connect db:" + strDbUrl);
			log.error("EXCEPTION:" + ex.getMessage());

		}
		return conDb;
	}

	/**
	 * データベースへの接続を切断する
	 * 
	 * @param conDb
	 *            データベースへの接続
	 */
	public static void disconnectDb(Connection conDb) {
		if (conDb != null) {
			try {
				conDb.close();
			} catch (SQLException ex) {
			}
		}
	}
}
