package daruma.main;

import java.io.File;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.FileWriter;
import daruma.bus.processor.misp.MispSoapClient;
import daruma.bus.processor.xslt.XslTransformer;
import daruma.common.CommonUtil;
import daruma.common.CommonUtil.FeatureName;

public class CountFeatureType extends StandardApplication {
	static private Log log = LogFactory.getLog(CountFeatureType.class);

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));

	static public final String PROPS_RETRIEVE_COUNT_XSLT_TEMPLATE = "countfeaturetype.retrieve.count.xslt.template";

	static public final String DEFAULT_RETRIEVE_COUNT_XSLT_TEMPLATE = System
			.getProperty(PROPS_RETRIEVE_COUNT_XSLT_TEMPLATE, "");

	/**
	 * 起動時
	 */
	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * 指定したFeatureTypeのCountを取得します。
	 * 
	 * @param parameters
	 *            コマンドラインの引数
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		CommandLine cmd;
		
//		 コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}
		

		
		try {
			
			StandardMessage message = new StandardMessage();
			StandardPayload payload = new StandardPayload();
			message.setPayload(payload);
			message.setProperties(new Properties());
			// message.setVerboseMode(cmd.hasOption("debug"));
			Properties properties = message.getProperties();

			// delete hasOption 			
			int len = parameters.length;
			int i;
			for(i = 0; i < len ; i++){
				if("-debug".equals(parameters[i])){
					System.setProperty("DEFAULT_DEBUG_MODE","true");
					message.setVerboseMode(true);
				}else{
					message.setVerboseMode(false);
				}
			}
				
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}
			
			// ホスト

				if (cmd.hasOption("host")) {
					properties.setProperty(MispSoapClient.PROPS_SERVER_HOST, cmd
							.getOptionValue("host"));
					properties
							.setProperty(MispSoapClient.PROPS_SERVER_PORT, "5050");
					
				} else {
			// 2006,12,10 hayashi
					System.out.println("ホストの設定が不正です。");
					optHandler.printHelp();
					return rc;
				}

			// 出力ファイルのチェック
			if (cmd.hasOption("out")) {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH, cmd
						.getOptionValue("out"));
			} else {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH,
						FileWriter.VALUE_OUTPUT_PATH_STDOUT);
			}
			message.getProperties().setProperty(
					FileWriter.PROPS_INPUT_ENCODING, "UTF-8");
			message.getProperties().setProperty(
					FileWriter.PROPS_OUTPUT_ENCODING, "UTF-8");

			/* <<< [06/11/11 01:31 I.Noda] <<< */
		
			String condStr = null ;
			if(cmd.hasOption("cond")) {
				condStr = 
					CommonUtil
					.readWholeFile(cmd.getOptionValue("cond")) ;
			}

			/* >>> [06/11/11 01:31 I.Noda] >>> */

			if (cmd.getArgs().length < 1) {
				System.out.println("オプションの設定が不正です。");
				optHandler.printHelp();
				return rc;
			}

			// FeatureType
			String name = cmd.getArgs()[0];
			String targetNS = null;
			if (cmd.getArgs().length > 1)
				targetNS = cmd.getArgs()[1];
			FeatureName featureName = CommonUtil.getFeatureName(name, targetNS);
			if (featureName == null) {
				log.error("feature名形式不正");
				return rc;
			}


			// GetFeature リクエストの作成
			File requestXmlFile = CommonUtil.createGetFeatureRequestXml(
					workDir, featureName, null, true, condStr);
			payload.attach(requestXmlFile);

			// プロセッサ
			MispSoapClient soapClient = new MispSoapClient();
			XslTransformer transformer = new XslTransformer();
			FileWriter fileWriter = new FileWriter();

			// XSLT
				transformer.setXsltFile(System.getProperty(
					PROPS_RETRIEVE_COUNT_XSLT_TEMPLATE,
					DEFAULT_RETRIEVE_COUNT_XSLT_TEMPLATE));
				transformer.setOmitXmlDeclaration("yes");
				transformer.setIndent("no");

			// 処理
			message.setProcessorList(new ArrayList());
			message.getProcessorList().add(soapClient);
			message.getProcessorList().add(transformer);
			message.getProcessorList().add(fileWriter);
			message.send();

			if (!message.isVerboseMode() && message.getPayload() != null)
				message.getPayload().dispose();
			requestXmlFile.delete();
			rc = true;

		} catch (Exception e) {
			// 2006,12,25 hayashi DEBUG
			System.out.println("ホストの接続またはファイルの作成に失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}
}
