package daruma.main;

import java.io.File;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.FileWriter;
import daruma.bus.processor.csv.CSVSchemaReader;
import daruma.bus.processor.xslt.XslTransformer;
import daruma.common.CommonUtil;

public class Csv2Schema extends StandardApplication {

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));

	static public final String PROPS_CREATE_REGISTER_REQUEST_XSLT_TEMPLATE = "csv2schema.create.register.request.xslt.template";

	static public final String DEFAULT_CREATE_REGISTER_REQUEST_XSLT_TEMPLATE = System
			.getProperty(PROPS_CREATE_REGISTER_REQUEST_XSLT_TEMPLATE, "");


	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * CSVファイルからXML Schemaを生成します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			System.out.println("入力ファイルが設定されていません。");
			optHandler.printHelp();
			return rc;
		}

		try {
			StandardMessage message = new StandardMessage();
			StandardPayload payload = new StandardPayload();
			message.setPayload(payload);
			message.setProperties(new Properties());
			message.setVerboseMode(cmd.hasOption("debug"));
			Properties properties = message.getProperties();
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}

			// 入力ファイルのチェック
			if (cmd.hasOption("in")) {
				File file = new File(cmd.getOptionValue("in"));
				payload.load(file);

			} else if (System.in.available() > 0) {
				File file = CommonUtil.loadFileFromStdin(workDir);
				payload.attach(file);
			} else {
				System.out.println("入力ファイルの設定が不正です。");
				optHandler.printHelp();
				return rc;
			}

			// 出力ファイルのチェック
			if (cmd.hasOption("out")) {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH, cmd
						.getOptionValue("out"));
			} else {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH,
						FileWriter.VALUE_OUTPUT_PATH_STDOUT);
			}
			message.getProperties().setProperty(
					FileWriter.PROPS_INPUT_ENCODING, "UTF-8");
			message.getProperties().setProperty(
					FileWriter.PROPS_OUTPUT_ENCODING, "UTF-8");
			
			// プロセッサ
			CSVSchemaReader reader = new CSVSchemaReader();
			XslTransformer transformer = new XslTransformer();
			FileWriter fileWriter = new FileWriter();

			// XSLT
			transformer.setXsltFile(System.getProperty(
					PROPS_CREATE_REGISTER_REQUEST_XSLT_TEMPLATE,
					DEFAULT_CREATE_REGISTER_REQUEST_XSLT_TEMPLATE));
			transformer.setIndent("yes");

			// 処理
			message.setProcessorList(new ArrayList());
			message.getProcessorList().add(reader);
			if (cmd.hasOption("register"))
				message.getProcessorList().add(transformer);
			message.getProcessorList().add(fileWriter);
			message.send();

			if (!message.isVerboseMode() && message.getPayload() != null)
				message.getPayload().dispose();
			rc = true;
		} catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("ファイルの読み込み、または書き込みに失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

}
