package daruma.main;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.csv.CSVFileWriter;
import daruma.bus.processor.csv.Const;
import daruma.bus.processor.csv.MispSchemaAccess;
import daruma.bus.processor.csv.MispSchemaCx;
import daruma.bus.processor.csv.MispSchemaCxAccess;
import daruma.bus.processor.misp.BuiltinFilter;
import daruma.bus.processor.misp.GetFeatureFilterReader;
import daruma.bus.processor.misp.GetTagValueFilter;
import daruma.bus.processor.misp.MispSoapClient;
import daruma.bus.processor.xslt.XslTransformer;
import daruma.common.CommonUtil;
import daruma.common.CommonUtil.FeatureName;

public class CsvGetFeature extends StandardApplication {
	static private Log log = LogFactory.getLog(CsvGetFeature.class);

	static public final String PROPS_ADD_TRANSACTION_XSLT_TEMPLATE = "csvgetfeature.add.transaction.xslt.template";

	static public final String DEFAULT_ADD_TRANSACTION_XSLT_TEMPLATE = System
			.getProperty(PROPS_ADD_TRANSACTION_XSLT_TEMPLATE, "");

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));


	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	private String currTransactionID = null;

	/**
	 * GetFeatureの内容をCSV形式で取得します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}
		try {
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}

			// ホストのチェック
			if (!cmd.hasOption("host")) {
				System.out.println("ホストが設定が不正です。");
				optHandler.printHelp();
				return rc;
			}
            // cfgのチェック
			if (!cmd.hasOption("csvcfg")) {
				// 2006,12,10 hayashi
				System.out.println("configファイルの設定が不正です。。");
				optHandler.printHelp();
				return rc;
			}

			// cxvcfgファイルをロードする
			MispSchemaCxAccess cxAccess = new MispSchemaCxAccess();
			MispSchemaCx schemaCx = cxAccess
				.loadSchemaCxFromXml(cmd.getOptionValue("csvcfg"),
						MispSchemaAccess.DFS_DFT_CHARSET);

			/* <<< [06/11/09 20:58 I.Noda] <<< */
			// 条件ファイルのチェック
			String condStr = null ;
			if(cmd.hasOption("cond")) {
				/* <<< [06/11/11 01:31 I.Noda] <<< */
				//File condFile = 
				//	new File(cmd.getOptionValue("cond")) ;
				//long len = condFile.length() ;
				/* !!! [06/11/09 20:58 I.Noda] !!! 
				 * FIXME:
				 * この部分、XML の文法チェックをすべきか？
				 */
				//CharBuffer buffer = CharBuffer.allocate((int)len) ;
				//FileReader condReader =new FileReader(condFile) ;
				//condReader.read(buffer) ;
				//buffer.rewind() ;
				//condStr = buffer.toString() ;
				/* === [06/11/11 01:31 I.Noda] === */
				condStr = 
					CommonUtil
					.readWholeFile(cmd.getOptionValue("cond")) ;
				/* >>> [06/11/11 01:31 I.Noda] >>> */
			}
			/* >>> [06/11/09 20:58 I.Noda] >>> */

			// 入力ファイルのチェック
			File inFile = null;
			FeatureName featureName = null;
			if (cmd.hasOption("in")) {
				inFile = new File(cmd.getOptionValue("in"));
			} else if (cmd.hasOption("checkFeature")) {
				featureName = CommonUtil.getFeatureName(cmd
						.getOptionValue("checkFeature"), null);
				if (featureName == null) {
					log.error("feature名形式不正");
					return rc;
				}
			} else if (System.in.available() > 0) {
			} else {
				featureName = new FeatureName();
				featureName.setTargetNS(schemaCx.getSchema()
						.getTargetNamespace());
				featureName.setTypeName(schemaCx.getSchema()
						.getFeatureElement().getName());
			}

			String outFile = null;
			if (cmd.hasOption("out")) {
				outFile = cmd.getOptionValue("out");
			}
			if (cmd.hasOption("loop")) {
				rc = processLoop(cmd, inFile, featureName, schemaCx, outFile, condStr);
			} else {

				rc = processFile(cmd, inFile, featureName, schemaCx, outFile, condStr);
			}

		}catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("ファイルの読み込み、または書き込みに失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

	private boolean processLoop(CommandLine cmd, File inFile,
				    FeatureName featureName, 
				    MispSchemaCx schemaCx, 
				    String outFile,
				    String condStr) {
		boolean rc = false;
		try {
			int interval = 10;
			if (cmd.hasOption("interval")) {
				try {
					interval = Integer.parseInt(cmd.getOptionValue("interval"));
				} catch (Exception e) {
				}
			}

			System.out.println("ループモード開始(時間間隔=" + interval + "秒)");
			int cntTime = 0;
			while (true) {
				cntTime++;
				// System.out.println(cntTime + " 回目開始");
				String currOutFile = outFile;
				if (currOutFile != null) {
					String serial = "00000000" + cntTime;
					if (cmd.hasOption("diff"))
						currOutFile += "."
								+ serial.substring(serial.length() - 8, serial
										.length());
				}
				processFile(cmd, inFile, featureName, schemaCx, currOutFile, condStr);
				System.out.println("");
				Thread.sleep(interval * 1000);
			}

		} catch (Exception e) {
			log.error("CsvGetFeature ProcessLoop error");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

	private boolean processFile(CommandLine cmd, File inFile,
				    FeatureName featureName, 
				    MispSchemaCx schemaCx, 
				    String outFile,
				    String condStr) {
		boolean rc = false;
		try {
			// メッセージ作成
			StandardMessage message = new StandardMessage();

			// Payloadファイル設定
			StandardPayload payload = new StandardPayload();
			message.setPayload(payload);
			if (inFile == null) {
				if (featureName != null)
					inFile = CommonUtil.createGetFeatureRequestXml(workDir,
							featureName, null, false, condStr);
				else
					inFile = CommonUtil.loadFileFromStdin(workDir);
				payload.attach(inFile);
			} else
				payload.load(inFile);

			// デバッグモード設定
			message.setVerboseMode(cmd.hasOption("debug"));

			// 各種プロパティ設定
			Properties properties = new Properties();
			message.setProperties(properties);
			// Mispサーバー
			properties.setProperty(MispSoapClient.PROPS_SERVER_HOST, cmd
					.getOptionValue("host"));
			properties.setProperty(MispSoapClient.PROPS_SERVER_PORT, "5050");
			// 出力ファイル
			if (outFile != null)
				properties
						.setProperty(CSVFileWriter.PROPS_OUTPUT_PATH, outFile);
			else
				properties.setProperty(CSVFileWriter.PROPS_OUTPUT_PATH,
						CSVFileWriter.VALUE_OUTPUT_PATH_STDOUT);
			// 出力CSVファイル文字コード
			if (cmd.hasOption("encode")) {
				message.getProperties().setProperty(
						CSVFileWriter.PROPS_OUTPUT_ENCODING,
						cmd.getOptionValue("encode"));
			}

			MispSchemaCxAccess cxAccess = new MispSchemaCxAccess();

			// プロセッサ
			GetFeatureFilterReader filterReader = new GetFeatureFilterReader();
			MispSoapClient soapClient = new MispSoapClient();
			BuiltinFilter builtinFilter = new BuiltinFilter(
					BuiltinFilter.MODE_XML2CSV);
			if (cmd.hasOption("attatchDir"))
				builtinFilter.setAttatchDir(cmd.getOptionValue("attatchDir"));
			builtinFilter.setSchemaCx(schemaCx);

			// 処理1
			File xml2csvXsltFile = File.createTempFile(CsvGetFeature.class
					.getName(), "_xml2csv.xslt", workDir);
			cxAccess.createXsltXml2Csv(schemaCx, xml2csvXsltFile
					.getAbsolutePath(), "UTF-8", "UTF-8");

			message.setProcessorList(new ArrayList());
			if (cmd.hasOption("diff") && (currTransactionID != null)) {
				List paramList = new ArrayList();
				paramList.add("transactionId");
				paramList.add(currTransactionID);
				XslTransformer addTransactionIdTransformer = new XslTransformer();
				addTransactionIdTransformer.setXsltFile(System.getProperty(
						PROPS_ADD_TRANSACTION_XSLT_TEMPLATE,
						DEFAULT_ADD_TRANSACTION_XSLT_TEMPLATE));
				addTransactionIdTransformer.setParameterList(paramList);
				message.getProcessorList().add(addTransactionIdTransformer);
			}
			message.getProcessorList().add(filterReader);
			message.getProcessorList().add(soapClient);
			message.getProcessorList().add(builtinFilter);
			message.send();

			// TransactionID取得
			GetTagValueFilter tagValueFilter = new GetTagValueFilter(
					Const.NS_MISP_URI, "MostRecentTransactionID");
			String transactionID = tagValueFilter.getTagValue(message
					.getPayload().getInputStream());
			if (transactionID != null)
				currTransactionID = transactionID;

			// 処理2
			XslTransformer csv2xmlTransformer = new XslTransformer();
			csv2xmlTransformer.setXsltFile(xml2csvXsltFile.getCanonicalPath());
			CSVFileWriter csvWriter = new CSVFileWriter();
			message.setProcessorList(new ArrayList());
			message.getProcessorList().add(csv2xmlTransformer);
			message.getProcessorList().add(csvWriter);
			message.send();

			if (!message.isVerboseMode() && message.getPayload() != null)
				message.getPayload().dispose();
			if (!message.isVerboseMode())
				xml2csvXsltFile.delete();

			rc = true;
		} catch (Exception e) {
			log.error("CsvGetFeature ProcessFile error");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}

		}
		return rc;
	}
}
