package daruma.main;

import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import daruma.common.CommonUtil;
import daruma.common.DbUtil;
import daruma.common.CommonUtil.FeatureName;

public class MaintainFeatureType extends StandardApplication {
	static private Log log = LogFactory.getLog(MaintainFeatureType.class);

	private final static String PROPS_DEL_SCHEMA_MODE = "maintain.schema.deletemode";

	private final static String PROPS_DEL_DATA_MODE = "maintain.data.deletemode";

	private final static String PROPS_DFT_HOST = "maintain.default.host";

	private final static String PROPS_DFT_PORT = "maintain.default.port";

	private final static String PROPS_DFT_USER = "maintain.default.user";

	private final static String PROPS_DFT_PASS = "maintain.default.pass";

	private final static String PROPS_DFT_DB = "maintain.default.db";

	private final static String DEL_SCHEMA_MODE_FLG = "DEL_FLG";

	private final static String DEL_SCHEMA_MODE_DEL = "DELETE";

	private final static String DEL_DATA_MODE_NONE = "NONE";

	private final static String DEL_DATA_MODE_DEL = "DELETE";

	private final static String DEL_DATA_MODE_DROP = "DROP";

	private String TBL_ELEMENT = "__element__";

	private String TBL_SCHEMA = "__schema__";

	private String TBL_TYPE = "__type__";

	private String TBL_ALIAS = "__universal_name_alias__";
	

	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * 指定したFeatureTypeのスキーマを表示します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);

		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}
		try {
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}
			
			String action = "";
			if (cmd.hasOption("action")) {
				action = cmd.getOptionValue("action");
			}
			if (!isValidAction(action)) {
				System.out.println("不正のアクション：" + action);
				optHandler.printHelp();
				return rc;
			}

			FeatureName featureName = null;
			if (isSingleAction(action)) {
				// FeatureType
				String name = null;
				if (cmd.getArgs().length > 0)
					name = cmd.getArgs()[0];
				String targetNS = null;
				if (cmd.getArgs().length > 1)
					targetNS = cmd.getArgs()[1];
				featureName = CommonUtil.getFeatureName(name, targetNS);
				if (featureName == null) {
					log.error("feature名形式不正");
					return rc;
				}
			}
			Connection connDb = connectionDb(optHandler, cmd);

			if (connDb != null) {
				rc = maintainFeatureType(connDb, action, featureName);
			}

		} catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("データベースへの接続に失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

	private boolean isSingleAction(String action) {
		return action.equals("delete") || action.equals("cleardata")
				|| action.equals("countdata") || action.equals("describe");

	}

	private boolean isValidAction(String action) {
		return action.equals("delete") || action.equals("deleteall")
				|| action.equals("cleardata") || action.equals("countdata")
				|| action.equals("listall") || action.equals("describe");
	}

	private boolean maintainFeatureType(Connection connDb, String action,
			FeatureName featureName) {
		boolean rc = true;
		try {
			if (action.equals("deleteall") || action.equals("listall")) {
				rc = maintainAllFeatureType(connDb, action);
			} else {
				SchemaInfo info = findSchema(connDb, featureName.getTypeName(),
						featureName.getTargetNS());

				if (info != null) {
					if (action.equals("delete")) {
						rc = deleteFeatureType(connDb, info);
					} else if (action.equals("cleardata")) {
						rc = clearFeatureType(connDb, info, false);
					} else if (action.equals("countdata")) {
						rc = countFeatureType(connDb, info);
					} else if (action.equals("describe")) {
						rc = describeFeatureType(connDb, info);
					} else {
						System.out.println("不正のアクション：" + action);
						rc = false;
					}
				} else {
					System.out.println("{" + featureName.getTargetNS() + "}"
							+ featureName.getTypeName() + "が見つかりません");
				}
			}
		} catch (SQLException e) {
			rc = false;
			e.printStackTrace();
		}

		try {
			if (rc)
				connDb.commit();
			else
				connDb.rollback();
		} catch (SQLException e) {
			rc = false;
		}
		return rc;

	}

	private boolean maintainAllFeatureType(Connection connDb, String action)
			throws SQLException {
		boolean rc = true;
		String sql = "SELECT element_name, element_namespace FROM "
				+ TBL_ELEMENT + " WHERE deleted=0 ";
		PreparedStatement pstmt = connDb.prepareStatement(sql);
		ResultSet rst = pstmt.executeQuery();

		int cnt = 0;
		while (rst.next()) {
			cnt++;
			Blob blob;
			blob = rst.getBlob("element_name");
			String element_name = new String(blob.getBytes(1, (int) blob
					.length()));
			blob = rst.getBlob("element_namespace");
			String element_namespace = new String(blob.getBytes(1, (int) blob
					.length()));

			SchemaInfo info = findSchema(connDb, element_name,
					element_namespace);
			if (info != null) {
				if (action.equals("deleteall")) {
					if (!deleteFeatureType(connDb, info)) {
						rc = false;
						break;
					}
				} else if (action.equals("listall")) {
					System.out.println(cnt + ":{" + element_namespace + "}"
							+ element_name);
				}
			}
		}
		pstmt.close();
		if (rc) {
			if (action.equals("deleteall"))
				System.out.println("全FeatureTypeを削除しました");
		}
		return rc;
	}

	private boolean describeFeatureType(Connection connDb, SchemaInfo info)
			throws SQLException {
		System.out.println("{" + info.elementNamespace + "}" + info.elementName
				+ "のスキーマは");
		System.out.println(info.schemaDef);
		return true;
	}

	private boolean countFeatureType(Connection connDb, SchemaInfo info)
			throws SQLException {
		String sql = "SELECT count(*) as num FROM " + info.mainTblName;
		PreparedStatement pstmt = connDb.prepareStatement(sql);
		ResultSet rst = pstmt.executeQuery();
		int num = 0;
		if (rst.next()) {
			num = rst.getInt("num");
		}
		pstmt.close();

		System.out.println("{" + info.elementNamespace + "}" + info.elementName
				+ "のデータ数は" + num);
		return true;
	}

	private boolean clearFeatureType(Connection connDb, SchemaInfo info, boolean internal)
			throws SQLException {
		String sql;

		for (int cnt = 0; cnt < info.subTblNameList.size(); cnt++) {
			String tblName = (String) info.subTblNameList.get(cnt);
			sql = "DELETE FROM " + tblName;
			PreparedStatement pstmt = connDb.prepareStatement(sql);
			pstmt.executeUpdate();
			pstmt.close();
		}
		sql = "DELETE FROM " + info.mainTblName;
		PreparedStatement pstmt = connDb.prepareStatement(sql);
		pstmt.executeUpdate();
		pstmt.close();
		if(!internal)
		System.out.println("{" + info.elementNamespace + "}" + info.elementName
				+ "のデータをクリアしました");
		return true;
	}

	private boolean deleteFeatureType(Connection connDb, SchemaInfo info)
			throws SQLException {
		boolean rc = true;
		String delSchemaMode = System.getProperty(PROPS_DEL_SCHEMA_MODE,
				DEL_SCHEMA_MODE_FLG);
		String delDataMode = System.getProperty(PROPS_DEL_DATA_MODE,
				DEL_DATA_MODE_NONE);

		PreparedStatement pstmt;
		String sql;
		if (delSchemaMode.equals(DEL_SCHEMA_MODE_DEL)) {
			sql = "DELETE FROM "
					+ TBL_ELEMENT
					+ " WHERE deleted=0 and element_name=? and element_namespace=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.elementName);
			pstmt.setString(2, info.elementNamespace);
			pstmt.executeUpdate();
			pstmt.close();

			sql = "DELETE FROM " + TBL_SCHEMA
					+ " WHERE deleted=0 and schema_id=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.schemaUri);
			pstmt.executeUpdate();
			pstmt.close();

			sql = "DELETE FROM " + TBL_TYPE
					+ " WHERE deleted=0 and schema_uri=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.schemaUri);
			pstmt.executeUpdate();
			pstmt.close();
		} else {
			sql = "UPDATE "
					+ TBL_ELEMENT
					+ " SET deleted=1 WHERE deleted=0 and element_name=? and element_namespace=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.elementName);
			pstmt.setString(2, info.elementNamespace);
			pstmt.executeUpdate();
			pstmt.close();

			sql = "UPDATE " + TBL_SCHEMA
					+ " SET deleted=1 WHERE deleted=0 and schema_id=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.schemaUri);
			pstmt.executeUpdate();
			pstmt.close();

			sql = "UPDATE " + TBL_TYPE
					+ " SET deleted=1 WHERE deleted=0 and schema_uri=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, info.schemaUri);
			pstmt.executeUpdate();
			pstmt.close();
		}
		if (delDataMode.equals(DEL_DATA_MODE_DEL)) {
			clearFeatureType(connDb, info, true);
		} else if (delDataMode.equals(DEL_DATA_MODE_DROP)) {
			for (int cnt = 0; cnt < info.subTblNameList.size(); cnt++) {
				String tblName = (String) info.subTblNameList.get(cnt);
				sql = "DROP TABLE IF EXISTS " + tblName;
				pstmt = connDb.prepareStatement(sql);
				pstmt.executeUpdate();
				pstmt.close();
			}
			sql = "DROP TABLE IF EXISTS " + info.mainTblName;
			pstmt = connDb.prepareStatement(sql);
			pstmt.executeUpdate();
			pstmt.close();
		} else {
		}

		if (rc) {
			System.out.println("{" + info.elementNamespace + "}"
					+ info.elementName + "を削除しました");
		} else {
			System.out.println("{" + info.elementNamespace + "}"
					+ info.elementName + "を削除できませんしました");
		}
		return rc;
	}

	private Connection connectionDb(CommandLineHandler optHandler,
			CommandLine cmd) {
		// ホスト
		String host = System.getProperty(PROPS_DFT_HOST, "localhost");
		String port = System.getProperty(PROPS_DFT_PORT, "3306");
		String user = System.getProperty(PROPS_DFT_USER, "root");
		String pass = System.getProperty(PROPS_DFT_PASS, "");
		String db = System.getProperty(PROPS_DFT_DB, "daruma$devel");

		if (cmd.hasOption("host")) {
			host = cmd.getOptionValue("host");
		}
		if (cmd.hasOption("port")) {
			port = cmd.getOptionValue("port");
		}
		if (cmd.hasOption("user")) {
			user = cmd.getOptionValue("user");
		}
		if (cmd.hasOption("pass")) {
			pass = cmd.getOptionValue("pass");
		}
		if (cmd.hasOption("db")) {
			db = cmd.getOptionValue("db");
		}
		String jdbcDriver = "com.mysql.jdbc.Driver";
		String jdbcUri = "jdbc:mysql://" + host + ":" + port + "/" + db;
		Connection dbConn = DbUtil.connectDb(jdbcDriver, jdbcUri, user, pass);
		return dbConn;
	}

	class SchemaTypeInfo {
		private String typeName;

		private String typeNamespace;

		private int idNum;

		private String id;

		private String alias;

		/**
		 * aliasを取得する
		 * 
		 * @return Returns the alias.
		 */
		public String getAlias() {
			return alias;
		}

		/**
		 * aliasをセットする
		 * 
		 * @param alias
		 *            The alias to set.
		 */
		public void setAlias(String alias) {
			this.alias = alias;
		}

		/**
		 * idを取得する
		 * 
		 * @return Returns the id.
		 */
		public String getId() {
			return id;
		}

		/**
		 * idをセットする
		 * 
		 * @param id
		 *            The id to set.
		 */
		public void setId(String id) {
			this.id = id;
		}

		/**
		 * idNumを取得する
		 * 
		 * @return Returns the idNum.
		 */
		public int getIdNum() {
			return idNum;
		}

		/**
		 * idNumをセットする
		 * 
		 * @param idNum
		 *            The idNum to set.
		 */
		public void setIdNum(int idNum) {
			this.idNum = idNum;
		}

		/**
		 * typeNameを取得する
		 * 
		 * @return Returns the typeName.
		 */
		public String getTypeName() {
			return typeName;
		}

		/**
		 * typeNameをセットする
		 * 
		 * @param typeName
		 *            The typeName to set.
		 */
		public void setTypeName(String typeName) {
			this.typeName = typeName;
		}

		/**
		 * typeNamespaceを取得する
		 * 
		 * @return Returns the typeNamespace.
		 */
		public String getTypeNamespace() {
			return typeNamespace;
		}

		/**
		 * typeNamespaceをセットする
		 * 
		 * @param typeNamespace
		 *            The typeNamespace to set.
		 */
		public void setTypeNamespace(String typeNamespace) {
			this.typeNamespace = typeNamespace;
		}

	}

	class SchemaInfo {
		private String elementName;

		private String elementNamespace;

		private SchemaTypeInfo elementTypeInfo = new SchemaTypeInfo();

		private String schemaUri;

		private String schemaDef;

		private String mainTblName;

		private List subTblNameList = new ArrayList();

		/**
		 * element_nameを取得する
		 * 
		 * @return Returns the element_name.
		 */
		public String getElementName() {
			return elementName;
		}

		/**
		 * element_nameをセットする
		 * 
		 * @param element_name
		 *            The element_name to set.
		 */
		public void setElement_name(String element_name) {
			this.elementName = element_name;
		}

		/**
		 * element_namespaceを取得する
		 * 
		 * @return Returns the element_namespace.
		 */
		public String getElementNamespace() {
			return elementNamespace;
		}

		/**
		 * element_namespaceをセットする
		 * 
		 * @param element_namespace
		 *            The element_namespace to set.
		 */
		public void setElement_namespace(String element_namespace) {
			this.elementNamespace = element_namespace;
		}

		/**
		 * schemaDefを取得する
		 * 
		 * @return Returns the schemaDef.
		 */
		public String getSchemaDef() {
			return schemaDef;
		}

		/**
		 * schemaDefをセットする
		 * 
		 * @param schemaDef
		 *            The schemaDef to set.
		 */
		public void setSchemaDef(String schemaDef) {
			this.schemaDef = schemaDef;
		}

		/**
		 * schemaUriを取得する
		 * 
		 * @return Returns the schemaUri.
		 */
		public String getSchemaUri() {
			return schemaUri;
		}

		/**
		 * schemaUriをセットする
		 * 
		 * @param schemaUri
		 *            The schemaUri to set.
		 */
		public void setSchemaUri(String schemaUri) {
			this.schemaUri = schemaUri;
		}

	}

	SchemaInfo findSchema(Connection connDb, String element_name,
			String element_namespace) {
		PreparedStatement pstmt = null;
		ResultSet rst = null;
		SchemaInfo schemaInfo = new SchemaInfo();
		boolean rc = false;

		try {
			schemaInfo.elementName = element_name;
			schemaInfo.elementNamespace = element_namespace;

			String sql = "SELECT a.type_name as type_name, a.type_namespace as type_namespace,"
					+ " c.schema_id as schema_id, c.schema_definition as schema_definition,"
					+ " d.id_number as id_number, d.id as id, d.alias as alias"
					+ " FROM "
					+ TBL_ELEMENT
					+ " a, "
					+ TBL_TYPE
					+ " b,"
					+ TBL_SCHEMA
					+ " c,"
					+ TBL_ALIAS
					+ " d"
					+ " WHERE a.deleted=0 "
					+ " and b.deleted=0 "
					+ " and c.deleted=0 "
					+ " and d.deleted=0 "
					+ " and a.type_namespace = b.type_namespace"
					+ " and a.type_name = b.type_name"
					+ " and b.schema_uri = c.schema_id"
					+ " and a.element_name=? and a.element_namespace=?"
					+ " and d.local_name=? and d.namespace=?";
			pstmt = connDb.prepareStatement(sql);
			pstmt.setString(1, element_name);
			pstmt.setString(2, element_namespace);
			pstmt.setString(3, element_name);
			pstmt.setString(4, element_namespace);
			rst = pstmt.executeQuery();
			if (rst.next()) {
				Blob blob;
				blob = rst.getBlob("type_name");
				schemaInfo.elementTypeInfo.typeName = new String(blob.getBytes(
						1, (int) blob.length()));
				blob = rst.getBlob("type_namespace");
				schemaInfo.elementTypeInfo.typeNamespace = new String(blob
						.getBytes(1, (int) blob.length()));
				blob = rst.getBlob("schema_id");
				schemaInfo.schemaUri = new String(blob.getBytes(1, (int) blob
						.length()));
				blob = rst.getBlob("schema_definition");
				schemaInfo.schemaDef = new String(blob.getBytes(1, (int) blob
						.length()));

				schemaInfo.elementTypeInfo.idNum = rst.getInt("id_number");
				blob = rst.getBlob("id");
				schemaInfo.elementTypeInfo.id = new String(blob.getBytes(1,
						(int) blob.length()));
				blob = rst.getBlob("alias");
				schemaInfo.elementTypeInfo.alias = new String(blob.getBytes(1,
						(int) blob.length()));

				schemaInfo.mainTblName = schemaInfo.elementTypeInfo.id;
				rst.close();
				pstmt.close();

				sql = "show tables";
				pstmt = connDb.prepareStatement(sql);
				rst = pstmt.executeQuery();
				String subTablePattern = "^" + schemaInfo.mainTblName
						+ "\\$.*$";
				while (rst.next()) {
					String tblName = rst.getString(1);
					if (tblName.matches(subTablePattern)) {
						schemaInfo.subTblNameList.add(tblName);
					}
				}
				rst.close();
				pstmt.close();

				rc = true;

			} else {
			}
		} catch (Exception e) {
		
			e.printStackTrace();

		} finally {
			if (pstmt != null)
				try {
					pstmt.close();
				} catch (SQLException e) {
				}
			if (rst != null)
				try {
					pstmt.close();
				} catch (SQLException e) {
				}
		}

		return rc ? schemaInfo : null;
	}

}
