package daruma.main;

import java.io.File;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.FileWriter;
import daruma.bus.processor.csv.Const;
import daruma.bus.processor.misp.MispSoapClient;
import daruma.bus.processor.xslt.XslTransformer;
import daruma.common.CommonUtil;

public class ShowFeatureTypes extends StandardApplication {

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));

	static public final String PROPS_WFS2TEXT_XSLT_TEMPLATE = "showfeaturetypes.wfs2text.xslt.template";

	static public final String DEFAULT_WFS2TEXT_XSLT_TEMPLATE = System
			.getProperty(PROPS_WFS2TEXT_XSLT_TEMPLATE, "");

	
	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * 登録されているFeatureTypeを列挙します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}
		try {
			StandardMessage message = new StandardMessage();
			StandardPayload payload = new StandardPayload();
			message.setPayload(payload);
			message.setProperties(new Properties());
			message.setVerboseMode(cmd.hasOption("debug"));
			Properties properties = message.getProperties();

			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}
			
			// ホスト
			if (cmd.hasOption("host")) {
				properties.setProperty(MispSoapClient.PROPS_SERVER_HOST, cmd
						.getOptionValue("host"));
				properties
						.setProperty(MispSoapClient.PROPS_SERVER_PORT, "5050");
			} else {
				// 2006,12,10 hayashi
				System.out.println("ホストの設定が不正です。");
				optHandler.printHelp();
				return rc;
			}

			// 出力ファイルのチェック
			if (cmd.hasOption("out")) {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH, cmd
						.getOptionValue("out"));
			} else {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH,
						FileWriter.VALUE_OUTPUT_PATH_STDOUT);
			}
			message.getProperties().setProperty(
					FileWriter.PROPS_INPUT_ENCODING, "UTF-8");
			message.getProperties().setProperty(
					FileWriter.PROPS_OUTPUT_ENCODING,
					cmd.hasOption("encode") ? cmd.getOptionValue("encode")
							: "UTF-8");

			// GetCapabilities リクエストの作成
			String internalCharset = System
					.getProperty(SystemProps.PROPS_SYS_INTERNAL_ENCODING);
			String msg = "<?xml version='1.0' encoding='UTF-8'?><"
					+ Const.NS_MISP_PRE
					+ ":GetCapabilities version='1.0.0' service='"
					+ Const.NS_WFS_PRE + "' " + Const.NS_XMLNS_PRE + "='"
					+ Const.NS_WFS_URI + "' " + Const.NS_XMLNS_PRE + ":"
					+ Const.NS_MISP_PRE + "='" + Const.NS_MISP_URI + "'/>";
			File requestXmlFile = CommonUtil.loadFileFromBuffer(msg
					.getBytes(internalCharset), workDir);
			payload.attach(requestXmlFile);

			// プロセッサ
			MispSoapClient soapClient = new MispSoapClient();
			XslTransformer transformer = new XslTransformer();
			FileWriter fileWriter = new FileWriter();

			// XSLT
			transformer.setXsltFile(System.getProperty(
					PROPS_WFS2TEXT_XSLT_TEMPLATE,
					DEFAULT_WFS2TEXT_XSLT_TEMPLATE));
			transformer.setOmitXmlDeclaration("yes");
			transformer.setIndent("no");

			// 処理
			message.setProcessorList(new ArrayList());
			message.getProcessorList().add(soapClient);
			message.getProcessorList().add(transformer);
			message.getProcessorList().add(fileWriter);

			message.send();

			if (!message.isVerboseMode() && message.getPayload() != null)
				message.getPayload().dispose();
			if (!message.isVerboseMode())
				requestXmlFile.delete();

			rc = true;

		} catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("ホストの接続またはファイルの書き込みに失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}
}
