package daruma.main;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.FileWriter;
import daruma.bus.processor.csv.Const;
import daruma.bus.processor.misp.GetFeatureFilterReader;
import daruma.bus.processor.misp.GetTagValueFilter;
import daruma.bus.processor.misp.MispSoapClient;
import daruma.bus.processor.xslt.XslTransformer;
import daruma.common.CommonUtil;
import daruma.common.CommonUtil.FeatureName;

public class XmlGetFeature extends StandardApplication {
	static private Log log = LogFactory.getLog(XmlGetFeature.class);

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));

	static public final String PROPS_RETRIEVE_SOAP_BODY_XSLT_TEMPLATE = "xmlgetfeature.retrieve.soap.body.xslt.template";

	static public final String PROPS_RETRIEVE_FEATURE_COLLECTION_XSLT_TEMPLATE = "xmlgetfeature.retrieve.feature.collection.xslt.template";

	static public final String DEFAULT_RETRIEVE_SOAP_BODY_XSLT_TEMPLATE = System
			.getProperty(PROPS_RETRIEVE_SOAP_BODY_XSLT_TEMPLATE, "");

	static public final String DEFAULT_RETRIEVE_FEATURE_COLLECTION_XSLT_TEMPLATE = System
			.getProperty(PROPS_RETRIEVE_FEATURE_COLLECTION_XSLT_TEMPLATE, "");

	static public final String PROPS_ADD_TRANSACTION_XSLT_TEMPLATE = "xmlgetfeature.add.transaction.xslt.template";

	static public final String DEFAULT_ADD_TRANSACTION_XSLT_TEMPLATE = System
			.getProperty(PROPS_ADD_TRANSACTION_XSLT_TEMPLATE, "");

	
	protected int onStart(String[] parameters) {
		boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * GetFeatureの内容をCSV形式で取得します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}

		try {
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}
			
			// ホストのチェック
			if (!cmd.hasOption("host")) {
				// 2006,12,10 hayashi
				System.out.println("ホストの設定が不正です。");
				optHandler.printHelp();
				return rc;
			}

			// 入力ファイルのチェック
			File inFile = null;
			FeatureName featureName = null;
			if (cmd.hasOption("in")) {
				inFile = new File(cmd.getOptionValue("in"));
			} else if (cmd.hasOption("checkFeature")) {
				featureName = CommonUtil.getFeatureName(cmd
						.getOptionValue("checkFeature"), null);
				if (featureName == null) {
					log.error("feature名形式不正");
					return rc;
				}
			} else if (System.in.available() > 0) {
			} else {
				// 2006,12,10 hayashi
				System.out.println("入力ファイルの設定が不正です。");
				optHandler.printHelp();
				return rc;
			}

			String outFile = null;
			if (cmd.hasOption("out")) {
				outFile = cmd.getOptionValue("out");
			}

			/* <<< [06/11/11 01:31 I.Noda] <<< */
			String condStr = null ;
			if(cmd.hasOption("cond")) {
			    condStr = 
				CommonUtil
				.readWholeFile(cmd.getOptionValue("cond")) ;
			}
			/* >>> [06/11/11 01:31 I.Noda] >>> */

			if (cmd.hasOption("loop")) {
				rc = processLoop(cmd, inFile, featureName, outFile, condStr);
			} else {

				rc = processFile(cmd, inFile, featureName, outFile, condStr);
			}
		} catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("ホストの接続またはファイルの読み込みに失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

	private String currTransactionID = null;

	private boolean processLoop(CommandLine cmd, File inFile,
			FeatureName featureName, String outFile,
				    String condStr) {
		boolean rc = false;
		try {
			int interval = 10;
			if (cmd.hasOption("interval")) {
				try {
					interval = Integer.parseInt(cmd.getOptionValue("interval"));
				} catch (Exception e) {
				}
			}

			System.out.println("ループモード開始(時間間隔=" + interval + "秒)");
			int cntTime = 0;
			while (true) {
				cntTime++;
				// System.out.println(cntTime + " 回目開始");
				String currOutFile = outFile;
				if (currOutFile != null) {
					String serial = "00000000" + cntTime;
					if (cmd.hasOption("diff"))
						currOutFile += "."
								+ serial.substring(serial.length() - 8, serial
										.length());
				}
				processFile(cmd, inFile, featureName, currOutFile, condStr);
				System.out.println("");
				Thread.sleep(interval * 1000);
			}

		} catch (Exception e) {
			log.error("CsvGetFeature ProcessLoop error");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

	private boolean processFile(CommandLine cmd, File inFile,
			FeatureName featureName, String outFile,
				    String condStr) {
		boolean rc = false;
		try {
			// メッセージ作成
			StandardMessage message = new StandardMessage();

			// Payloadファイル設定
			StandardPayload payload = new StandardPayload();
			message.setPayload(payload);
			if (inFile == null) {
  			        if (featureName != null) {
					inFile = CommonUtil.createGetFeatureRequestXml(workDir,
							featureName, null, false, condStr);
				} else {
					inFile = CommonUtil.loadFileFromStdin(workDir);
				}
				payload.attach(inFile);
			} else
				payload.load(inFile);

			// デバッグモード設定
			message.setVerboseMode(cmd.hasOption("debug"));

			// 各種プロパティ設定
			Properties properties = new Properties();
			message.setProperties(properties);
			// Mispサーバー
			properties.setProperty(MispSoapClient.PROPS_SERVER_HOST, cmd
					.getOptionValue("host"));
			properties.setProperty(MispSoapClient.PROPS_SERVER_PORT, "5050");

			// 出力ファイル
			if (cmd.hasOption("out")) {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH, outFile);
			} else {
				properties.setProperty(FileWriter.PROPS_OUTPUT_PATH,
						FileWriter.VALUE_OUTPUT_PATH_STDOUT);
			}
			message.getProperties().setProperty(
					FileWriter.PROPS_INPUT_ENCODING, "UTF-8");
			message.getProperties().setProperty(
					FileWriter.PROPS_OUTPUT_ENCODING, cmd.hasOption("encode") ? cmd.getOptionValue("encode") : "UTF-8");

			// プロセッサ
			GetFeatureFilterReader filterReader = new GetFeatureFilterReader();
			MispSoapClient soapClient = new MispSoapClient();
			message.setProcessorList(new ArrayList());
			if (cmd.hasOption("diff") && (currTransactionID != null)) {
				List paramList = new ArrayList();
				paramList.add("transactionId");
				paramList.add(currTransactionID);
				XslTransformer addTransactionIdTransformer = new XslTransformer();
				addTransactionIdTransformer.setXsltFile(System.getProperty(
						PROPS_ADD_TRANSACTION_XSLT_TEMPLATE,
						DEFAULT_ADD_TRANSACTION_XSLT_TEMPLATE));
				addTransactionIdTransformer.setParameterList(paramList);
				message.getProcessorList().add(addTransactionIdTransformer);
			}
			message.getProcessorList().add(filterReader);
			message.getProcessorList().add(soapClient);
			message.send();

			GetTagValueFilter tagValueFilter = new GetTagValueFilter(
					Const.NS_MISP_URI, "MostRecentTransactionID");
			String transactionID = tagValueFilter.getTagValue(message
					.getPayload().getInputStream());
			if (transactionID != null)
				currTransactionID = transactionID;

			XslTransformer soapBodyTransformer = new XslTransformer();
			FileWriter fileWriter = new FileWriter();
			String propFile;
			if (cmd.hasOption("featureCollection")) {
				propFile = System.getProperty(
						PROPS_RETRIEVE_FEATURE_COLLECTION_XSLT_TEMPLATE,
						DEFAULT_RETRIEVE_FEATURE_COLLECTION_XSLT_TEMPLATE);
			} else {
				propFile = System.getProperty(
						PROPS_RETRIEVE_SOAP_BODY_XSLT_TEMPLATE,
						DEFAULT_RETRIEVE_SOAP_BODY_XSLT_TEMPLATE);
			}
			soapBodyTransformer.setXsltFile(propFile);
			message.setProcessorList(new ArrayList());
			message.getProcessorList().add(soapBodyTransformer);
			message.getProcessorList().add(fileWriter);
			message.send();

			if (!message.isVerboseMode() && message.getPayload() != null)
				message.getPayload().dispose();

			rc = true;
		} catch (Exception e) {
			log.error("XmlGetFeature ProcessFile error");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}

		}
		return rc;
	}

}
