package daruma.main;

import java.io.File;

import org.apache.commons.cli.CommandLine;
import application.runtime.CommandLineHandler;
import application.runtime.StandardApplication;
import application.runtime.SystemContext;
import application.startup.SystemProps;
import daruma.bus.processor.csv.MispSchema;
import daruma.bus.processor.csv.MispSchemaAccess;

public class Xsd2Dfs extends StandardApplication {

	static protected File workDir = new File(System
			.getProperty(SystemProps.PROPS_SYS_WORK_PATH));


	protected int onStart(String[] parameters) {
        boolean rc = execute(parameters);
		return (rc ? 0 : 1);
	}

	/**
	 * CSVファイルからXML Schemaを生成します。
	 * 
	 * @param parameters
	 */
	public boolean execute(String[] parameters) {
		boolean rc = false;
		// コマンドライン処理クラスの取得
		CommandLineHandler optHandler = (CommandLineHandler) SystemContext
				.getBean("commandLineHelper");
		CommandLine cmd = optHandler.getCommandLineObject(parameters);
		if (cmd == null) {
			// 2006,12,10 hayashi
			System.out.println("オプションの設定が不正です。");
			optHandler.printHelp();
			return rc;
		}
		try {
			if(cmd.getArgs().length < 2)
			{
				// 2006,12,10 hayashi
				System.out.println("入力ファイルまたは出力ファイルが設定されていません。");
				optHandler.printHelp();
				return rc;
			}
			
			//  2006,12,25 hayashi デバッグモードチェック
			if (cmd.hasOption("debug")) {
				System.setProperty("DEFAULT_DEBUG_MODE","true");
			}
			
			String inFile = cmd.getArgs()[0];
			String outFile = cmd.getArgs()[1];
			
			MispSchemaAccess access = new MispSchemaAccess();
			MispSchema schema = access.loadSchemaFromXml(inFile, MispSchemaAccess.DFS_DFT_CHARSET);
			if(schema != null)
			{
				if(access.saveSchemaToCsv(schema, outFile, MispSchemaAccess.DFS_DFT_CHARSET, 
						MispSchemaAccess.DFS_DFT_SEPARATOR,MispSchemaAccess.DFS_DFT_PARENTHESIS, MispSchemaAccess.DFS_DFT_COMMENT))
				{
					rc = true;
				}
				else
				{
					System.out.println("ファイル書き込み失敗");
				}
			}
			else
			{
				System.out.println("ファイル読み込み失敗");
			}

		} catch (Exception e) {
			// 2006,12,10 hayashi
			System.out.println("ファイルの読み込みまたは書き込みに失敗しました。");
			if(System.getProperty("DEFAULT_DEBUG_MODE") == "true"){
				e.printStackTrace();
			}
		}
		return rc;
	}

}
