package daruma.test;

import java.io.File;
import java.util.ArrayList;
import java.util.Properties;

import application.startup.SystemProps;

import junit.framework.Assert;
import junit.framework.TestCase;
import daruma.bus.message.StandardMessage;
import daruma.bus.message.StandardPayload;
import daruma.bus.processor.ConsoleWriter;
import daruma.bus.processor.csv.CSVFileReader;
import daruma.bus.processor.csv.CSVFileWriter;
import daruma.bus.processor.csv.CSVSchemaReader;
import daruma.bus.processor.misp.BuiltinFilter;
import daruma.bus.processor.misp.FeatureTypeFilter;
import daruma.bus.processor.misp.SortTagFilter;
import daruma.bus.processor.misp.XpathFilter;
import daruma.bus.processor.xslt.XslTransformer;

public class ProcessorTest extends TestCase {
	private boolean viewResultOnConsole = false;

	protected void setUp() throws Exception {
		SystemProps.loadSystemProperties();
	}

	private void doTest(String[][] testParamsList, ProcessorTester tester) {
		try {
			for (int cnt = 0; cnt < testParamsList.length; cnt++) {
				if (!tester.test(testParamsList[cnt]))
					break;
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail("doTest exception");
		}
	}

	private void doTestError(String[][] testParamsList, ProcessorTester tester) {
		try {
			tester.testSuccess = false;
			tester.testError = true;
			tester.testException = true;
			for (int cnt = 0; cnt < testParamsList.length; cnt++) {
				tester.test(testParamsList[cnt]);
				Thread.sleep(100);
			}
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail("doTest exception");
		}
	}

	
	public void testBuiltinCsv2Xml() {
		doTest(ProcessorTestParams.csv2xmlParamsList,
				new BuiltinCsv2XmlTester());
	}

	class BuiltinCsv2XmlTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 2);
				System.out.println("testBuiltinCsv2Xml start:" + testParams[0]);
				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(
						new BuiltinFilter(BuiltinFilter.MODE_CSV2XML));
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}
				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testBuiltinCsv2Xml result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testBuiltinCsv2Xml:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testBuiltinCsv2Xml exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testBuiltinCsv2Xml success");
			System.out.println("testBuiltinCsv2Xml "
					+ (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testBuiltinXml2Csv() {
		doTest(ProcessorTestParams.xml2csvParamsList,
				new BuiltinXml2CsvTester());
	}

	class BuiltinXml2CsvTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 2);
				System.out.println("testBuiltinXml2Csv start:" + testParams[0]);
				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(
						new BuiltinFilter(BuiltinFilter.MODE_XML2CSV));
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}
				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testBuiltinXml2Csv result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testBuiltinXml2Csv:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testBuiltinXml2Csv exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testBuiltinXml2Csv success");
			System.out.println("testBuiltinXml2Csv "
					+ (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testSortTag() {
		doTest(ProcessorTestParams.sorttagParamsList, new SortTagTester());
	}

	class SortTagTester extends ProcessorTester {

		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 4);
				System.out.println("testSortTag start:" + testParams[0]);
				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(
						new SortTagFilter(testParams[2], testParams[3]));
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}
				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testSortTag result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testSortTag:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testSortTag exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testSortTag success");
			System.out.println("testSortTag " + (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testXpath() {
		doTest(ProcessorTestParams.xpathParamsList, new XpathFilterTester());
	}

	class XpathFilterTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 4);
				System.out.println("testXpath start:" + testParams[0]);
				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				XpathFilter filter = new XpathFilter(testParams[2], testParams[3]);
				filter.setSeparator("=");
				message.getProcessorList().add(filter);
				message.getProperties().setProperty(
						XpathFilter.PROPS_OUTPUT_ENCODING, "Shift_JIS");

				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}
				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testXpath result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testXpath:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testXpath exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testXpath success");
			System.out.println("testXpath " + (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testXsltTransformer() {
		doTest(ProcessorTestParams.xsltParamsList, new XsltTransformerTester());
	}

	class XsltTransformerTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 3);
				System.out
						.println("testXsltTransformer start:" + testParams[0]);

				XslTransformer transformer = new XslTransformer();
				transformer.setXsltFile(testParams[1]);

				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				payload.load(new File(testParams[0]));
				message.setPayload(payload);

				message.setProcessorList(new ArrayList());
				message.getProcessorList().add(transformer);

				message.setProperties(new Properties());
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}

				message.send();

				if (testParams[2].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[2], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testXsltTransformer result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testXsltTransformer:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testXsltTransformer exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testXsltTransformer success");
			System.out.println("testXsltTransformer "
					+ (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testCsvReader() {
		doTest(ProcessorTestParams.csvreaderParamsList, new CsvReaderTester());
	}

	class CsvReaderTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 2);
				System.out.println("testCsvReader start:" + testParams[0]);

				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				CSVFileReader csvReader = new CSVFileReader();
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(csvReader);
				message.getProperties().setProperty(
						CSVFileReader.PROPS_INPUT_ENCODING, "Shift_JIS");

				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}

				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_STRING)) {
					if(!testError) Assert.fail("testCsvReader result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testCsvReader:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testCsvReader exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testCsvReader success");
			System.out.println("testCsvReader " + (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testCsvWriter() {
		doTest(ProcessorTestParams.csvwriterParamsList, new CsvWriterTester());
	}

	class CsvWriterTester extends ProcessorTester {
		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 2);
				System.out.println("testCsvWriter start:" + testParams[0]);

				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				CSVFileWriter csvWriter = new CSVFileWriter();
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(csvWriter);
				message.getProperties().setProperty(
						CSVFileWriter.PROPS_OUTPUT_ENCODING, "Shift_JIS");
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}

				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testCsvWriter result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testCsvWriter:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testCsvWriter exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testCsvWriter success");
			System.out.println("testCsvWriter " + (rc ? "success" : "fail"));
			return rc;
		}
	}

	public void testCsvSchemaReader() {
		doTest(ProcessorTestParams.csvsSchemaReaderParamsList,
				new CsvSchemaReaderTester());
	}

	class CsvSchemaReaderTester extends ProcessorTester {

		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 3);
				System.out
						.println("testCsvSchemaReader start:" + testParams[0]);

				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				CSVSchemaReader csvSchemaReader = new CSVSchemaReader();
				if(testParams[2].length() > 0)
					csvSchemaReader.setSeparator(testParams[2].charAt(0));
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(csvSchemaReader);
				message.getProperties().setProperty(
						CSVSchemaReader.PROPS_INPUT_ENCODING, "Shift_JIS");
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}

				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testCsvSchemaReader result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testCsvSchemaReader:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testCsvSchemaReader exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testCsvSchemaReader success");
			System.out.println("testCsvSchemaReader "
					+ (rc ? "success" : "fail"));
			return rc;

		}
	}
	
	public void testCsvSchemaReaderError() {
		doTestError(ProcessorTestParams.csvsSchemaReaderErrorParamsList,
				new CsvSchemaReaderTester());
	}


	
	public void testFetureTypeFilter() {
		doTest(ProcessorTestParams.featureTypeFilterParamsList,
				new FeatureTypeFilterTester());
	}

	class FeatureTypeFilterTester extends ProcessorTester {

		public boolean test(String[] testParams) {
			boolean rc = false;
			try {
				assertEquals("Parameter number isn't ok", testParams.length, 2);
				System.out
						.println("testFetureTypeFilter start:" + testParams[0]);

				StandardMessage message = new StandardMessage();
				StandardPayload payload = new StandardPayload();
				FeatureTypeFilter featureTypeFilter = new FeatureTypeFilter();
				message.setProcessorList(new ArrayList());
				payload.load(new File(testParams[0]));
				message.setPayload(payload);
				message.getProcessorList().add(featureTypeFilter);
				message.getProperties().setProperty(
						FeatureTypeFilter.PROPS_OUTPUT_ENCODING, "Shift_JIS");
				if (viewResultOnConsole) {
					ConsoleWriter writer = new ConsoleWriter();
					message.getProcessorList().add(writer);
				}

				message.send();
				if (testParams[1].length() > 0 && !testutil.isSameFile(message.getPayload().getInputStream(),
						testParams[1], testutil.MODE_TEXT_IGNORE_LINE_WS)) {
					if(!testError) Assert.fail("testFetureTypeFilter result isn't same");
				} else {
					rc = true;
				}
			} catch (Exception e) {
				if(testException) System.out.println("testFetureTypeFilter:" + e.getMessage()); 
				else
				{
					e.printStackTrace();
					if(!testException) Assert.fail("testFetureTypeFilter exception");
				}
			}
			if(rc && !testSuccess) Assert.fail("testFetureTypeFilter success");
			System.out.println("testFetureTypeFilter "
					+ (rc ? "success" : "fail"));
			return rc;

		}
	}

}
