package daruma.test;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class testutil {
	/**
	 * テキストモード
	 */
	private static long FLG_TEXT = 0x01;

	/**
	 * 文字列モード
	 */
	private static long FLG_STRING = 0x02;

	/**
	 * 空行を無視
	 */
	private static long FLG_TEXT_IGNORE_LINE = 0x100;

	/**
	 * 行の先頭と末尾のホワイトスペースを無視
	 */
	private static long FLG_TEXT_IGNORE_LINE_WS = 0x200;

	/**
	 * バイナリモード
	 */
	public static long MODE_BINARY = 0;

	/**
	 * テキストモード
	 */
	public static long MODE_TEXT = FLG_TEXT;

	/**
	 * 文字列モード
	 */
	public static long MODE_STRING = FLG_STRING;

	/**
	 * テキストモード(空行を無視)
	 */
	public static long MODE_TEXT_IGNORE_LINE = FLG_TEXT | FLG_TEXT_IGNORE_LINE;

	/**
	 * テキストモード(行の先頭と末尾のホワイトスペースおよび空行を無視)
	 */
	public static long MODE_TEXT_IGNORE_LINE_WS = FLG_TEXT
			| FLG_TEXT_IGNORE_LINE | FLG_TEXT_IGNORE_LINE_WS;

	public static boolean isSameFile(String file1, String file2, long flag) {
		boolean rc = false;
		FileInputStream fis1 = null;
		FileInputStream fis2 = null;
		try {
			fis1 = new FileInputStream(file1);
		} catch (FileNotFoundException e) {
			System.err.println("Fail to open file:" + file1);
			e.printStackTrace();
		}

		try {
			fis2 = new FileInputStream(file2);
		} catch (FileNotFoundException e) {
			System.err.println("Fail to open file:" + file2);
			e.printStackTrace();
		}

		if ((fis1 != null) && (fis2 != null)) {
			rc = isSameFile(fis1, fis2, flag);
		}
		return rc;
	}

	public static boolean isSameFile(InputStream fis1, String file2, long flag) {

		boolean rc = false;
		FileInputStream fis2 = null;

		try {
			fis2 = new FileInputStream(file2);
		} catch (FileNotFoundException e) {
			System.err.println("Fail to open file:" + file2);
			e.printStackTrace();
		}

		if ((fis1 != null) && (fis2 != null)) {
			rc = isSameFile(fis1, fis2, flag);
		}
		return rc;

	}

	public static boolean isSameFile(InputStream fis1, InputStream fis2,
			long flag) {
		boolean rc = false;
		if ((flag & FLG_STRING) == FLG_STRING) {
			rc = isSameStringFile(fis1, fis2);
		}
		else if ((flag & FLG_TEXT) == FLG_TEXT) {
			rc = isSameTextFile(fis1, fis2, flag);
		} else {
			rc = isSameBinaryFile(fis1, fis2);
		}
		return rc;
	}

	private static boolean isSameBinaryFile(InputStream fis1, InputStream fis2) {
		boolean rc = false;
		if ((fis1 != null) && (fis2 != null)) {
			int BUF_SIZE = 8192;
			byte[] buf1 = new byte[BUF_SIZE];
			byte[] buf2 = new byte[BUF_SIZE];
			while (true) {
				int len1 = 0;
				int len2 = 0;
				try {
					len1 = fis1.read(buf1);
				} catch (IOException e) {
					System.err.println("Read file fail");
					e.printStackTrace();
					break;
				}
				try {
					len2 = fis2.read(buf2);
				} catch (IOException e) {
					System.err.println("Read file fail");
					e.printStackTrace();
					break;
				}
				if (len1 != len2) {
					break;
				}
				if (byteCmp(buf1, buf2, len1) != 0)
					break;
				if ((len1 < 0) || (len2 < 0)) {
					rc = true;
					break;
				}
			}
		}

		return rc;
	}

	private static int byteCmp(byte[] buf1, byte[] buf2, int len) {
		for (int cnt = 0; cnt < len; cnt++) {
			if (buf1[cnt] < buf2[cnt])
				return -1;
			else if (buf1[cnt] > buf2[cnt])
				return 1;
		}
		return 0;
	}

	private static boolean isSameTextFile(InputStream fis1, InputStream fis2,
			long flag) {
		boolean rc = false;
		if ((fis1 != null) && (fis2 != null)) {
			BufferedReader reader1 = new BufferedReader(new InputStreamReader(
					fis1));
			BufferedReader reader2 = new BufferedReader(new InputStreamReader(
					fis2));
			if ((reader1 != null) && (reader2 != null)) {
				String line1 = "";
				String line2 = "";
				boolean ignore1 = false;
				boolean ignore2 = false;
				long cnt1 = 0;
				long cnt2 = 0;
				while (true) {
					/*
					 * 一行ずつ読む
					 */
					if (!ignore1) {
						try {
							line1 = reader1.readLine();
							if (line1 != null)
								cnt1++;
						} catch (IOException e) {
							System.err.println("Read file fail");
							e.printStackTrace();
							break;
						}
					}
					if (!ignore2) {
						try {
							line2 = reader2.readLine();
							if (line2 != null)
								cnt2++;
						} catch (IOException e) {
							System.err.println("Read file fail");
							e.printStackTrace();
							break;
						}
					}

					/*
					 * 終了をチェックする
					 */
					if ((line1 == null) && (line2 == null)) {
						rc = true;
						break;
					}

					/**
					 * 行の先頭と末尾のホワイトスペースを削除する
					 */
					if ((flag & FLG_TEXT_IGNORE_LINE_WS) == FLG_TEXT_IGNORE_LINE_WS) {
						if (line1 != null)
							line1 = line1.trim();
						if (line2 != null)
							line2 = line2.trim();
					}

					/*
					 * 空行をチェックする
					 */
					if ((flag & FLG_TEXT_IGNORE_LINE) == FLG_TEXT_IGNORE_LINE) {
						ignore2 = ((line1 != null) && line1.length() == 0);
						ignore1 = ((line2 != null) && (line2.length() == 0));
						if (ignore1 != ignore2)
							continue;
					}
					ignore1 = false;
					ignore2 = false;

					if (line1 == null) {
						System.err.println("File1 end");
						break;
					} else if (line2 == null) {
						System.err.println("File2 end");
						break;
					}
					if (!line2.equals(line1)) {
						System.err.println("DIFF1:" + cnt1 + ":" + line1);
						System.err.println("DIFF2:" + cnt2 + ":" + line2);
						break;
					}

				}
			}
		}

		return rc;
	}

	private static boolean isSameStringFile(InputStream fis1, InputStream fis2) {
		boolean rc = false;
		if ((fis1 != null) && (fis2 != null)) {
			BufferedReader reader1 = new BufferedReader(new InputStreamReader(
					fis1));
			BufferedReader reader2 = new BufferedReader(new InputStreamReader(
					fis2));
			if ((reader1 != null) && (reader2 != null)) {
				String total1 = "";
				String total2 = "";
				String line1 = "";
				String line2 = "";
				long cnt1 = 0;
				long cnt2 = 0;
				while (true) {
					/*
					 * 一行ずつ読む
					 */
					if (total1.length() <= total2.length()) {
						try {
							line1 = reader1.readLine();
							if (line1 != null) {
								total1 += line1.trim();
								cnt1++;
							}
						} catch (IOException e) {
							System.err.println("Read file fail");
							e.printStackTrace();
							break;
						}
					}
					if (total2.length() <= total1.length()) {
						try {
							line2 = reader2.readLine();
							if (line2 != null) {
								total2 += line2.trim();
								cnt2++;
							}
						} catch (IOException e) {
							System.err.println("Read file fail");
							e.printStackTrace();
							break;
						}
					}

					int sameLen = -1;
					if (total1.length() >= total2.length()) {
						if (total1.startsWith(total2))
							sameLen = total2.length();
					} else {
						if (total2.startsWith(total1))
							sameLen = total1.length();
					}

					if (sameLen < 0) {
						System.err.println("DIFF1:" + cnt1 + ":" + total1);
						System.err.println("DIFF2:" + cnt2 + ":" + total2);
						break;
					}
					total1 = total1.substring(sameLen);
					total2 = total2.substring(sameLen);

					/*
					 * 終了をチェックする
					 */
					if ((line1 == null) && (line2 == null)) {
						if ((total1.length() == total2.length())
								&& total1.equals(total2)) {
							rc = true;
						} else {
							System.err.println("DIFF1:" + cnt1 + ":" + total1);
							System.err.println("DIFF2:" + cnt2 + ":" + total2);
						}
						break;
					}
				}
			}
		}
		return rc;
	}

}
