/* */

import java.io.PrintStream;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * Generate "GetFeature" MISP text to pickup one data from DaRuMa
 *</p>
 *<dl>
 * <dd>ˡ:</dd>
 *<dt>
 *   java PeekFeature {[--namespace &lt;[prefix]=urn&gt;] ...} &lt;feature name&gt; &lt;property name&gt; [&lt;property value&gt;...]<br><br>
 *
 *    "--namespace" ץȤäɲä٤prefix֤̾Ȥ
 *    ꤹ롣 prefixξϥǥեȤ֤̾Ȥʤ롣
 *    ̤ɸϤ˽Ϥ롣
 *</dt>
 *
 * <dd>:</dd>
 *<dt>
 *   java PeekFeature --namespace =http://staff.aist.go.jp/i.noda/Rescue/RandomCity/2.0 --namespace rcbase=http://staff.aist.go.jp/i.noda/Rescue/RandomCity/base Node rcbase:id 1
 *</dt>
 *</dl>
 *
 * @author YOKOTA Hiroshi
 */
public class PeekFeature {
	private static HashMap<String, String> namespace = new HashMap<String, String>();

	/**
	 * Ȥɽ
	 */
	private static void printUsage(PrintStream w) {
		w.printf("Usage: java PeekFeature [--namespace <[prefix]=urn> ...] <feature name> <property name> [<property value>...]%n%n" +
			 "example: java PeekFeature --namespace =http://staff.aist.go.jp/i.noda/Rescue/RandomCity/2.0 --namespace rcbase=http://staff.aist.go.jp/i.noda/Rescue/RandomCity/base Node rcbase:id 1 2 3 4 5%n");
	}

	/**
	 * namespace  prefix бɽ롣
	 */
	private static ArrayList<String> gen_namespace(ArrayList<String> args) {
		ArrayList<String> ret = new ArrayList<String>();

		for(int i = 0; i < args.size(); i++) {
			if (args.get(i).equals("--namespace")) {
				String ns[] = args.get(i + 1).split("=", 2);

				namespace.put(ns[0], ns[1]);
				i++;
			} else {
				ret.add(args.get(i));
			}
		}

		return ret;
	}


	/**
	 * ޥɥ饤󥪥ץ󤫤 MISP ʸ롣
	 */
	public static void main(String args[]) {
		PrintStream o = System.out;

		ArrayFileExtractor afe = new ArrayFileExtractor("-@", args);
		ArrayList<String> p_args = gen_namespace(afe.toExtractedList());

		if(p_args.size() < 2) {
			printUsage(System.err);
			System.exit(1);
		}


		/*
		 * parse options
		 */
		String feature_name = p_args.get(0);
		p_args.remove(0);
		String prop_name    = p_args.get(0);
		p_args.remove(0);

		ArrayList<String> prop_vals = p_args;


		/*
		 * Output XML text
		 */

		// header
		o.printf("<?xml version='1.0'?>%n" +
			 "<!-- namespaces -->%n" +
			 "<misp:GetFeature xmlns:misp='http://www.infosharp.org/misp'%n");

		// namespace
		for(String key: namespace.keySet()) {
			if(key.length() == 0) {
				o.println("                 xmlns='" + namespace.get(key) + "'");
			} else {
				o.println("                 xmlns:" + key + "='" + namespace.get(key) + "'");
			}
		}

		// body and footer
		o.printf("                 >%n" +
			 " <misp:Query mode='features' typeName='" + feature_name + "'> <!-- feature name -->%n" +
			 "  <misp:Filter>%n");

		if (prop_vals.size() > 1) {
			o.println("   <misp:Or>");
		}

		for(String i: prop_vals) {
			o.printf("    <misp:PropertyIsEqualTo>%n" +
				 "     <misp:PropertyName>" + prop_name + "</misp:PropertyName> <!-- property name -->%n");

			o.println("     <misp:Literal>" + i + "</misp:Literal> <!-- property value -->");
			o.printf("    </misp:PropertyIsEqualTo>%n");
		}

		if (prop_vals.size() > 1) {
			o.println("   </misp:Or>");
		}

		o.printf("  </misp:Filter>%n" +
			 " </misp:Query>%n" +
			 "</misp:GetFeature>%n");
	}
}

/* end */
