#!/usr/bin/perl
#
# Fix CSV data structure for DaRuMa CSV client
#

use strict;
use warnings;

# Use "\r" for line separator.
$/ = "\r";

sub fix1() {
    my $in = "Table_of_Gathering_Spaces_for_Regional_Evacuation.csv";
    my $out = $in . ".out";

    open(IN,  "<:encoding(shift_jis)", $in) || die;
    open(OUT, ">:encoding(shift_jis)", $out) || die;

    my $line = <IN>;
    print OUT "#" . $line;

    while (my $line = <IN>) {
	my @item = split(/,/, $line);

	$item[1] -= 10000 if $item[1] =~ /\d+/;
	$item[2] -= 50000 if $item[2] =~ /\d+/;

	print OUT join(",", @item);
    }

    close(IN);
    close(OUT);

    rename($out, $in);
}

sub fix2() {
    my $in = "Table_of_Neighborhood_Associations.csv";
    my $out = $in . ".out";

    open(IN,  "<:encoding(shift_jis)", $in) || die;
    open(OUT, ">:encoding(shift_jis)", $out) || die;

    my $line = <IN>;
    print OUT "#" . $line;

    while (my $line = <IN>) {
	next if $line eq "\r";
	next if $line =~ /^,,,,,,,/;

	my @item = split(/,/, $line);

	$item[1] -= 50000 if $item[1] =~ /\d+/;

	print OUT join(",", @item);
    }

    close(IN);
    close(OUT);

    rename($out, $in);
}

sub fix3() {
    my $in = "Table_of_Population.csv";
    my $out = $in . ".out";

    open(IN,  "<:encoding(shift_jis)", $in) || die;
    open(OUT, ">:encoding(shift_jis)", $out) || die;

    my $line = <IN>;
    print OUT "#" . $line;

    while (my $line = <IN>) {
	my @item = split(/,/, $line);

	$item[1] -= 50000 if $item[1] =~ /\d+/;

	print OUT join(",", @item);
    }

    close(IN);
    close(OUT);

    rename($out, $in);
}

sub add_sharp($) {
    (my $in) = @_;
    my $out = $in . ".out";

    open(IN,  "<:encoding(shift_jis)", $in) || die;
    open(OUT, ">:encoding(shift_jis)", $out) || die;

    my $line = <IN>;
    print OUT "#" . $line;

    while(my $line = <IN>) {
	print OUT $line;
    }

    close(IN);
    close(OUT);

    rename($out, $in);
}

# special fixes
&fix1();
&fix2();
&fix3();

# normal fixes
for my $i ("Table_of_Aza_and_Chome.csv",
	   "Table_of_Designated_Shelters.csv",
	   "Table_of_Districts_for_Evacuation.csv",
	   "Table_of_Public_Schools.csv",
	   "Table_of_School_Districts.csv") {
    &add_sharp($i);
}

# end
