#!/usr/bin/perl
#
# XSD to DFS converter
#

use strict;
use warnings;

use File::Basename;
use XML::XPath;

my $filename = $ARGV[0];

###########################################################################
my %tagmap;
open(FD, "<", "bin/tag-swap-map.txt") || die;

while(my $l = <FD>) {
    chomp($l);

    next if($l =~ /^\#/);
    next if($l =~ /^$/);

    (my $short, my $long) =  split(" ", $l);

    $tagmap{$short} = $long;
}

close(FD);
###########################################################################

my $target = q(tag:h-yokota$staff.aist.go.jp$mitsuke$v3.0);

my $doc = XML::XPath->new(filename => $filename);


my $feature_name = $doc->find(q(//xs:element[@substitutionGroup="gml:_Feature"]/@name));
my $feature_type = $doc->find(q(//xs:element[@substitutionGroup="gml:_Feature"]/@type));
$feature_type =~ s/ogr://;

my $id	      = $target . "/" . $feature_name;
my $target_ns = $target;

print $feature_name;
print $feature_type;

print <<_EOF ;
"#DaRuMa","format=SchemaCSV","version=0.0"
"ID","${id}"
"Annotation",""
"Namespace","misp","http://www.infosharp.org/misp"
"Namespace","gml","http://www.opengis.net/gml"
"Namespace","xsd","http://www.w3.org/2001/XMLSchema"
"TargetNamespace","${target_ns}"
"Import","http://www.opengis.net/gml"
"","location","http://schemas.opengis.net/gml/3.1.1/base/feature.xsd"
"FeatureType","${feature_name}","${feature_type}"
"ElementType","${feature_type}"
_EOF

my $elements = $doc->find("//xs:complexType/xs:complexContent/xs:extension/xs:sequence/*");


foreach my $node ($elements->get_nodelist) {

    my $context = XML::XPath->new(context => $node);

    my $type;
    if($context->exists(q(@type))) {
	$type = $context->findvalue(q(@type));
    } else {
	$type = $context->findvalue(q(xs:simpleType/xs:restriction/@base));
    }

    $type =~ s/^xs:/xsd:/;

    $type = "xsd:float" if ("$type" eq "xsd:decimal");

    my $indexed;
    if ("$type" eq "gml:GeometryPropertyType") {
	$indexed = "true";
    } else {
	$indexed = "false";
    }

    my $tag = $tagmap{$context->findvalue(q(@name))};
    if (!defined($tag)) {
	$tag = $context->findvalue(q(@name));
    }

    print qq("","element","$tag","$type","$indexed",\");

    print $context->findvalue(q(@minOccurs));
    print q(",");
    print $context->findvalue(q(@maxOccurs));
    print "\"\n";
}

# end
